#ifndef __MDL_7_H__
#define __MDL_7_H__

#include "..\\mdl7\\comp_version.h"

#ifdef _MDL7_EXPORT
	#define MDL7_DECLSPEC	_declspec(dllexport)
#else
#ifdef _MDL7_IMPORT
	#define MDL7_DECLSPEC		_declspec(dllimport)
#else
	#define MDL7_DECLSPEC
#endif
#endif


/*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
Structs:

struct MD7_HEADER
struct MD7_BONE
struct MD7_GROUP
struct MD7_SKIN
struct MD7_COLORVALUE
struct MD7_MATERIAL
struct MD7_SKINPOINT
struct MD7_TRIANGLE
struct MD7_MAINVERTEX
struct MD7_FRAMEVERTEX
struct MD7_BONETRANS
struct MD7_FRAME
struct MDLIDPO_HEADER
struct MDLIDP2_HEADER
struct MDL3_HEADER
struct MDL6_HEADER
struct MDL3AND6_UNI_HEADER
struct MDL3_VERTEX
struct MDL4_VERTEX
struct IDP2_FRAME
struct MDL3_FRAME
struct MDL4_FRAME
struct IDP2_SKIN
struct MDL3_SKIN
struct MDL5_SKIN
struct MDLIDPO_SKINPOINT
struct MDL3_SKINPOINT
struct MDLIDPO_TRIANGLE
struct MDL3_TRIANGLE

Hierarchy Chart:
	
	+ MDL7W

	+ MDLR_BASE
		|	
		+- MDL7R
		|
		+- MDLR
			|
			+- MDLIDPOR
				|
				+- MDL3R
					|
					+- MDLIDP2R
					|
					+- MDL4R
						|
						+- MDL5R
							|
							+- MDL6R

+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/



/****************************************************************************************************/
//#define USE_FILE_STC		// schalter zwischen 'fopen' und '_open'
/****************************************************************************************************/


#define MDL_CURRVERSION	0    // fr write funktionen

//structure definitions:	used '#pragma pack(1)'
/****************************************************************************************************/
#pragma pack(1) 
struct MD7_HEADER
{
	char	ident[4];		// "MDL7"
	long	version;		// '0', 'sub'- version number;
	long	bones_num;
	long	groups_num;
	long	mdl7data_size;	// this file size (head included), except entlump and medlump. 
	long	entlump_size;	// entity info size (not yet used, always 0)
	long	medlump_size;	// size of data for MED in bytes

	unsigned short md7_bone_stc_size;
	unsigned short md7_skin_stc_size;
	unsigned short md7_colorvalue_stc_size;
	unsigned short md7_material_stc_size;
	unsigned short md7_skinpoint_stc_size;
	unsigned short md7_triangle_stc_size;
	unsigned short md7_mainvertex_stc_size;
	unsigned short md7_framevertex_stc_size;
	unsigned short md7_bonetrans_stc_size;
	unsigned short md7_frame_stc_size;
};


#define MAX_BONENAMESIZE	20 //32 // '0' incuded

struct MD7_BONE
{
	unsigned short parent_index;
	BYTE _unused_[2]; // 
	float x,y,z;
//ws, 12.05.2003: bone name mitspeichern...
	char name[MAX_BONENAMESIZE];
};

#define MAX_GROUPNAMESIZE	16

struct MD7_GROUP
{
	unsigned char	typ;		// = '1' -> triangle based Mesh

//ws, 26.04.2006 max weights per vertex
//	BYTE	_unused_[3];
	BYTE	deformers;
	BYTE	max_weights;
	BYTE	_unused_;

	long	groupdata_size; // size of data for this group in bytes ( MD7_GROUP stc. included).
	char	name[MAX_GROUPNAMESIZE];
	long	numskins;
	long	num_stpts;	// number of 2D skin vertices
	long	numtris;
	long	numverts;
	long	numframes;
};



struct MD7_DEFORMER
{
	BYTE	deformer_version;		// 0
	BYTE	deformer_typ;			// 0 - bones
	BYTE	_unused_[2];
	long	group_index;
	long	elements;
	long	deformerdata_size;
};

struct MD7_DEFORMERELEMENT
{
	long	element_index;		// bei deformer_typ==0 (==bones) element_index == bone index
	char	element_name[MAX_BONENAMESIZE];
	long	weights;
};

struct MD7_DEFORMERWEIGHT
{
	long	index;				// bei deformer_typ==0 (==bones) index == vertex index
	float	weight;
};


/*
Deformer definition:

[MD7_DEFORMER [MD7_DEFORMERELEMENT [MD7_DEFORMERWEIGHT] X  MD7_DEFORMERELEMENT::weights ] X MD7_DEFORMER::elements]

*/





/*
Skin Typ Definitions:

0,	0000 0000 00000 0000,	8 bit,  BPP_8BIT_PAL256
1,	xxxx xxxx 00000 0001,	index auf material (xxxx xxxx)

2,	0000 0000 00000 0010,	565, BPP_16BIT_RGB
3,	0000 0000 00000 0011,	(4)444 argb, BPP_16BIT_ARGB
4,	0000 0000 00000 0100,	888 rgb, BPP_24BIT
5,	0000 0000 00000 0101,	(8)888 argb, BPP_A32BIT_MDL5

6,	0000 0000 00000 0110,	DDS File
7,	0000 0000 00000 0111,	Externe Datei (Datei name)

10,	0000 0000 00000 1010,	565 + mips, BPP_16BIT_RGB
11,	0000 0000 00000 1011,	4444 + mips, BPP_16BIT_ARGB
12,	0000 0000 00000 1100,	888 rgb, BPP_24BIT
13,	0000 0000 00000 1101,	8888 + mips, BPP_A32BIT_MDL5

+	0000 0000 00001 0000 - material definition
+	0000 0000 00010 0000 - effect definition
*/

#define	M7SKINTYP_MIPFLAG				0x08
#define	M7SKINTYP_MATERIAL				0x10
#define	M7SKINTYP_MATERIAL_ASCDEF		0x20
#define	M7SKINTYP_RGBFLAG				0x80

#define	M7SKINTYP_8BIT_PAL256			0x00
#define	M7SKINTYP_16BIT_565				0x02
#define	M7SKINTYP_16BIT_4444			0x03
#define	M7SKINTYP_24BIT_888				0x04
#define	M7SKINTYP_32BIT_8888			0x05
#define	M7SKINTYP_DDS_FILE				0x06
#define	M7SKINTYP_EXT_FILE				0x07

#define	M7SKINTYP_16BIT_565M			(M7SKINTYP_16BIT_565 | M7SKINTYP_MIPFLAG)
#define	M7SKINTYP_16BIT_4444M			(M7SKINTYP_16BIT_4444 | M7SKINTYP_MIPFLAG)
#define	M7SKINTYP_24BIT_888M			(M7SKINTYP_24BIT_888 | M7SKINTYP_MIPFLAG)
#define	M7SKINTYP_32BIT_8888M			(M7SKINTYP_32BIT_8888 | M7SKINTYP_MIPFLAG)

#define	M7SKINTYP_MATERIAL_INDEX		0x01

#define	M7SKINTYP_TEXMASK				0x07

#define MAX_TEXNAMESIZE		16

struct MD7_SKIN
{
	unsigned char	typ;
	BYTE			_unused_[3];
	long			width;
	long			height;
	char			texture_name[MAX_TEXNAMESIZE];	//Skin bzw. Texture oder Material Name

	
};

struct MD7_COLORVALUE
{
	float r,g,b,a;
};

struct MD7_MATERIAL
{
	MD7_COLORVALUE	Diffuse;        /* Diffuse color RGBA */
    MD7_COLORVALUE	Ambient;        /* Ambient color RGB */
    MD7_COLORVALUE	Specular;       /* Specular 'shininess' */
    MD7_COLORVALUE	Emissive;       /* Emissive color RGB */
    float			Power;          /* Sharpness if specular highlight */
};

typedef int MD7_MATERIAL_ASCDEFSIZE;

struct MD7_SKINPOINT
{
	float s,t;	// 0.0f - 1.0f
};

struct MD7_SKINSET {
    unsigned short   st_index[3];	// st index ( MD7_SKINPOINT list)
	int material;					// material index;
};

struct MD7_TRIANGLE
{
	unsigned short   v_index[3]; 	// vertex index  ( MD7_MAINVERTEX list)
//	unsigned short   st_index[3];	// st index ( MD7_SKINPOINT list)
//12.01.05
//	int material;					// material index;

//17.04.2008 // zwei skinsets
	MD7_SKINSET  skinsets[2];
}; 





#define NUM_NORMALINDICES	162
#define MD7_MAINVERTEX120503_STCSIZE		16
#define MD7_MAINVERTEX030305_STCSIZE		26

struct MD7_MAINVERTEX
{
	float x,y,z;
	unsigned short bone_index;
//ws, 12.05.2003: vertex normals
	//unsigned short norm162index;
	union {
	unsigned short norm162index;
	float norm[3];
	};
};

#define MD7_FRAMEVERTEX120503_STCSIZE		16
#define MD7_FRAMEVERTEX030305_STCSIZE		26

struct MD7_FRAMEVERTEX
{
	float x,y,z;
	unsigned short vertindex;		//	the index of this vertex, 0.. MD7_GROUP::numverts - 1
//ws, 12.05.2003: vertex normal abspeichern
	//unsigned short norm162index; //float xn,yn,zn;
//ws, 03.03.2005: vertex normal abspeichern, und 'unsigned short normals162index' rausnmen
	union {
	unsigned short norm162index;
	float norm[3];
	};
};

struct MD7_BONETRANS
{
	float	m [4*4];				// [4*3]
	unsigned short bone_index;		//	the index of this vertex, 0.. MD7_HEADER::bones_num - 1
	BYTE _unused_[2];
};


#define MAX_FRAMENAMESIZE		16

struct MD7_FRAME
{
	char	frame_name[MAX_FRAMENAMESIZE];
	long	vertices_count;			//	length of vertices list
	long	transmatrix_count;		//	length of vertices list
};


//25.04.2006 beta:  Level of Deteils info als 'medlump' abspeichern
struct MD7_LUMP
{
	int lump_id;
	int lump_size;
};

struct MD7_LUMPNODE
{
	int	lump_id;			// LoD lump id == 1;
	int	lump_size;			//
	int	lump_itemsnumber;	// groups number
};

struct MD7_LODGROUP
{
	int group_index;
	int num_lod;
};

struct MD7_LODDATA
{
	float level_value;  // level_value < 100.0 && level_value > 0.0
	int	  numtris;
};



/****************************************************************************************************/

struct MDLIDPO_HEADER
{
	char	ident[4];           // 0x4F504449 = "IDPO" for IDPOLYGON
	long	version;            // Version = 6
	float	scale[3];           // Model scale factors.
	float	scale_origin[3];	// Model origin.
	float	boundingradius;		// Model bounding radius.
	float	eyeposition[3];		// Eye position (useless?)
	long	numskins;			// the number of skin textures
	long	skinwidth;			// Width of skin texture -> must be multiple of 8
	long	skinheight;			// Height of skin texture -> must be multiple of 8
	long	numverts;			// Number of vertices
	long	numtris;			// Number of triangles surfaces
	long	numframes;			// Number of frames
	long	synctype;			// 0= synchron, 1= random
	long	flags;				// 0 (see Alias models)
	float	size;				// average size of triangles
};


struct MDLIDP2_HEADER  // .md2 files
{
	char	ident[4];       // "IDP2" for IDPOLYGON
	int		version;		// equals 8
	int		skinwidth;		// skinwidth;
	int		skinheight;		// skinheight;
	int		framesize;		// byte size of each frame
	int		numskins;
	int		numverts;		//->'mdl::head.numverts' Num of SKIN-VERTICES
	int		numstpts;			// greater than num_xyz for seams
	int		numtris;		//->'mdl::head.numtris'	Num of Triangles
	int		num_glcmds;		// dwords in strip/fan command list
	int		numframes;
	int		ofs_skins;		// each skin is a MAX_SKINNAME string
	int		ofs_st;			// byte offset from start for stverts
	int		ofs_tris;		// offset for dtriangles
	int		ofs_frames;		// offset for first frame
	int		ofs_glcmds;
	int		ofs_end;		// end of file
};

struct MDL3_HEADER
{
	char	ident[4];
	int		version;
	float	scale[3];
	float   scale_origin[3];
	float   boundingradius;
    float   eyeposition[3];
	int		numskins;
	int		skinwidth;
	int		skinheight;
	int		numverts;
	int		numtris;
	int		numframes;
	int		numstpts;
	int		flags;
	float	medlump;
};

struct MDL6_HEADER
{
	char	ident[4];
	int		version;
	float	scale[3];
	float   scale_origin[3];
	long	unused2;      // not used
	long	entlump;      // entity info size
	long	vertlump;     // 
	long	bonelump;
	int		numskins;
	int		skinwidth;
	int		skinheight;
	int		numverts;
	int		numtris;
	int		numframes;
	int		numstpts;
	int		flags;
	float	medlump;
};

struct MDL3AND6_UNI_HEADER
{
	char	ident[4];
	int		version;
	float	scale[3];
	float   scale_origin[3];
	long	diffirens[4]; // verschiedene values fr MDL3 und MDL6
	int		numskins;
	int		skinwidth;
	int		skinheight;
	int		numverts;
	int		numtris;
	int		numframes;
	int		numstpts;
	int		flags;
	float	medlump;
};


struct MDL3_VERTEX
{
	unsigned char	p_pos[3];			// X,Y,Z coordinate, packed on 0-255
	unsigned char	normals162index;	// index of the vertex normal
};

struct MDL4_VERTEX
{
	unsigned short	p_pos[3];			// X,Y,Z coordinate, packed on 0-65535
	unsigned char	normals162index;	// index of the vertex normal
	unsigned char	pad;				// fuer spaetere Nutzung.
};


struct IDP2_FRAME
{
	float			scale[3];
	float			origin[3];
	char			frame_name[MAX_FRAMENAMESIZE];  //16
};


struct MDL3_FRAME
{
	long			group;				// muss '0' sein
	MDL3_VERTEX		min,max;
	char			frame_name[MAX_FRAMENAMESIZE]; // 16
};


struct MDL4_FRAME
{
	long			group;				// muss '2' sein
	MDL4_VERTEX		min,max;
	char			frame_name[MAX_FRAMENAMESIZE]; // 16
};


#define MAX_IDP2_FILENAMESIZE		64

struct IDP2_SKIN
{
	char file_name[MAX_IDP2_FILENAMESIZE];
};


struct MDL3_SKIN
{
	int typ;
};

struct MDL5_SKIN
{
	int typ;
	int width, height;
};

struct MDLIDPO_SKINPOINT
{
	long onseam;                 // 0 or 0x20
	long s;
	long t;
};

struct MDL3_SKINPOINT
{
	short   s,t;
};

struct MDLIDPO_TRIANGLE
{
	long facesfront;
	long v_index[3];
};

struct MDL3_TRIANGLE
{
     unsigned short   v_index[3]; 	// vertex index  ( MD7_MAINVERTEX list)
     unsigned short   st_index[3];	// st index ( MD7_SKINPOINT list)
}; 
#pragma pack() 
/****************************************************************************************************/






int MDL7_DECLSPEC mdl7_generate_162_normals_index(float x, float y, float z);
bool MDL7_DECLSPEC mdl7_get_162_normal(int index, float & x, float & y, float & z);






class MDL7_DECLSPEC	MDL7W 
{
#ifdef USE_FILE_STC
	FILE*		m_Fp;
#else
	int			m_Fh;
#endif

	MD7_HEADER	hd;
	
	union {
	MD7_GROUP	 gr;
	MD7_LODGROUP lump_gr;
	MD7_DEFORMER df;
	};

	union {
	long		gr_pos;
	long		df_pos;
	};
	
	MD7_SKIN	sk;
	long		sk_pos;
	long		sk_bypp;
	bool		sk_mips;

	union {
	MD7_FRAME	fr;
	MD7_LODDATA	lump_loddata;
	MD7_DEFORMERELEMENT	df_e;	
	};
	
	union {
	long		fr_pos;
	long		df_e_pos;
	};

	
	MD7_LUMPNODE lump_hd;
	long		 lump_pos;


public:
	MDL7W();
	~MDL7W();

//--
	MD7_HEADER & Head() { return hd; }
	bool WriteEmptyBlock ( long bytes);

//--
	void * Open(const char * file);
	
	bool Bone(int parent_index, float x, float y, float z, const char * name);
		
	bool Group ( unsigned char typ=1, const char * name=NULL);
	
	bool Skin(unsigned char typ /* ? */=0, const char * name=NULL);
	bool SkinSize ( int width, int height, int bytes_perpixel, bool mips);
	bool SkinBits ( const unsigned char * mip1, const unsigned char * mip2, const unsigned char * mip3, const unsigned char * mip4 );
	bool SkinMaterial(const float * diffuse, const float * ambient, const float * specular, const float * emissive, float power);
	bool SkinMaterial(int datablock_size, const char * data_block);
	bool SkinDone();

	bool SkinPoint(float s, float t);
	bool Triangle(int v1, int v2, int v3, int sp1, int sp2, int sp3, int material = -1);
	bool Triangle(int v1, int v2, int v3, int sp1, int sp2, int sp3, int material, int sp21, int sp22, int sp23, int material2);
//ws, 03.03.2005
	//bool Vertex(float x, float y, float z, int bone_index, int normal_index);
	bool Vertex(float x, float y, float z, int bone_index, float nx, float ny, float nz);

	bool Frame(const char * name=NULL);
	bool FrameDone();
//ws, 03.03.2005
	//bool FVertex(int vertex_index, float x, float y, float z, int normal_index);
	bool FVertex(int vertex_index, float x, float y, float z, float nx, float ny, float nz);
	bool FBoneMatrix(int bone_index, const float * m);
	
	bool GroupDone ();
	
	bool Close();

	
//--
//25.04.2006 LoD-s: 

	bool LumpLoD ();
	bool LumpLoDGroup(int group_index);
	bool LumpLoDData(float level_value);
	bool LumpLoDTriangle(int v1, int v2, int v3, int sp1, int sp2, int sp3, int material = -1);
	bool LumpLoDTriangle(int v1, int v2, int v3, int sp1, int sp2, int sp3, int material, int sp21, int sp22, int sp23, int material2);
	bool LumpLoDDataDone();
	bool LumpLoDGroupDone();
	bool LumpLoDDone ();


//26.04.2007 multiply bones per vertex
	bool GroupDone ( BYTE deformers, BYTE maxweights);

	bool Deformer ( unsigned char typ, int group);
	bool DeformerElement ( int def_obj_index, const char * def_obj_name = NULL );
	bool DeformerWeight ( int obj_index, float weight);
	bool DeformerElementDone();
	bool DeformerDone ();
};





typedef enum {
	TMDL_UNKNOWN,
	TMDL_IDPO,
	TMDL_IDP2,
	TMDL_MDL2,
	TMDL_MDL3,
	TMDL_MDL4,
	TMDL_MDL5,
	TMDL_MDL6,
	TMDL_MDL7,
} TMDLFILE;


class MDL7_DECLSPEC MDLR_BASE
{
friend class MDLIDP2R;
friend class MDLIDPOR;
friend class MDL3R;
friend class MDL4R;
friend class MDL5R;
friend class MDL6R;
friend class MDL7R;

#ifdef USE_FILE_STC
	FILE*		m_Fp;
#else
	int			m_Fh;
#endif

	unsigned char*		m_FBuf;
	unsigned char*		m_FBufPtr;
	long				m_FBufSize;
public:
	TMDLFILE			m_FileTyp;
	
public:
	MDLR_BASE();
	virtual ~MDLR_BASE();
	bool Skip ( long bytes);
	long GetFilePos();
	bool SetFilePos(long pos);
	void * LockFp();
	void UnlockFp();
	unsigned char * Read ( long size);

	virtual void * Open ( const char * file);
	virtual long Load ( unsigned char * ptr, long size = 0x7fffffff) = 0;
	virtual bool Close ();

	virtual bool SkipBones()=0;
	virtual bool SkipGroup()=0;
	virtual bool SkipSkin()=0;
	virtual bool SkipSkins()=0;
	virtual bool SkipSPoints()=0;
	virtual bool SkipTriangles()=0;
	virtual bool SkipVertices()=0;
	virtual bool SkipFrame()=0;
	virtual bool SkipFrames()=0;
};




class MDL7_DECLSPEC MDL7R : public MDLR_BASE
{
public:
	MD7_HEADER		*hd;
	
	union {
	MD7_GROUP		*gr;
	MD7_LUMPNODE	*lump;
	MD7_LUMP		*mlump;
	};

	MD7_BONE		* bn;
	MD7_MATERIAL	* mt;	
	MD7_SKIN		* sk;
	int				  sk_bypp;		
	bool			  sk_mips;
	MD7_SKINPOINT	* sp;
	MD7_TRIANGLE	* tr;
	MD7_MAINVERTEX	* mv;
	MD7_FRAME		* fr;
	MD7_FRAMEVERTEX	* fv;
	MD7_BONETRANS	* bt;
	union {
	int				  mt_asclen;
	long			  lump_pos;	
	};
//ws, 26.04.2006
	int				  deformers;
	int				  max_weights;
	MD7_DEFORMER		* df;
	MD7_DEFORMERELEMENT	* de;	
	MD7_DEFORMERWEIGHT	* dw;

public:
	MDL7R();
	virtual ~MDL7R();

	const MD7_HEADER & Head();

	virtual void * Open(const char * file);
	virtual long Load(unsigned char * ptr, long size = 0x7fffffff); 
	
	virtual bool Head(long & bones_num, long & groups_num,long & mdllump_size, long & entlump_size, long & medlump_size);
	
	virtual bool Bone(int & parent_index,float & x, float & y, float & z, char * name = NULL );
		
	virtual bool Group(unsigned char & typ, long & groupdata_size, char * name = NULL);
	virtual bool Group(long	& skins, long & stpts,long & tris, long & verts, long & frames);
	
	virtual bool Skin(unsigned char & typ, char * name);
	virtual bool SkinSize ( int & width, int & height, int & bytes_perpixel, bool & mips);
	virtual bool SkinBits ( unsigned char ** mip1=NULL, unsigned char ** mip2=NULL, unsigned char ** mip3=NULL, unsigned char ** mip4=NULL);
	virtual bool SkinMaterial(float * diffuse, float * ambient, float * specular, float * emissive, float & power);
	virtual bool SkinMaterialAscDef(int & strlen);
	virtual bool SkinMaterialAscDef(char * bub);
		
	virtual bool SkinPoint(float & s, float & t);
	virtual bool Triangle(int & v1, int & v2, int & v3, int & sp1, int & sp2, int & sp3, int & material);
	virtual bool Triangle(int & v1, int & v2, int & v3, int & sp1, int & sp2, int & sp3, int & material, int & sp21, int & sp22, int & sp23, int & material2);
	virtual bool Vertex(float & x, float & y, float & z, int & bone_index/*, int & normal_index*/);
	virtual bool VertexN(float &x,float &y,float &z);
	
	virtual bool Frame(int & verts, int & mtrs, char * name = NULL);
	virtual bool FVertex(int & vertex_index, float & x, float & y, float & z/*, int & normal_index*/);
	virtual bool FVertexN(float &x,float &y,float &z);
	virtual bool FBoneMatrix(int & bone_index, float * m);
	
	
	virtual bool Close();
//--

	virtual bool SkipBones();
	virtual bool SkipGroup();
	virtual bool SkipSkin();
	virtual bool SkipSkins();
	virtual bool SkipSPoints();
	virtual bool SkipTriangles();
	virtual bool SkipVertices();
	virtual bool SkipFrame();
	virtual bool SkipFrames();

//--
//25.04.2006 
	bool OpenEntityLump();
	bool GetEntityLump(int & typ,int & items_num);
	bool SkipEntityLump();
	bool GetLoDGroup(int & index, int & numLoD); 
	bool GetLoD(float & level, int & tris); 
	bool GetLoDTriangle(int & v1, int & v2, int & v3, int & sp1, int & sp2, int & sp3, int & material);
	bool GetLoDTriangle(int & v1, int & v2, int & v3, int & sp1, int & sp2, int & sp3, int & material, int & sp21, int & sp22, int & sp23, int & material2);
	
	bool OpenMedLump();
	bool GetMedLump(int & typ, int & size);
	bool SkipMedLump();

//26.04.2007 multiply bones per vertex
	bool Deformer ( unsigned char & typ, int & group, int & elements, long * deformerdata_size = NULL);
	bool DeformerElement ( int & def_obj_index, char * def_obj_name, int & num_weights);
	bool DeformerWeight ( int & obj_index, float & weight);

	bool SkipDeformers();
	bool SkipDeformer();
	bool SkipDeformerElements();
};


class MDL7_DECLSPEC MDLR : public MDLR_BASE
{
public:
	virtual int NumSkins()=0;
	virtual int NumVertices()=0;
	virtual int NumTriangles()=0;
	virtual int NumSkinPts()=0;
	virtual int NumFrames()=0;
		
	virtual bool Head ( void ** head)=0;
	virtual bool Skin(unsigned char & typ, int & width, int & height, int & bytes_perpixel, bool & mips)=0;
	//virtual bool Skin ( char * buf, int size) =0;
	virtual bool SkinBits ( unsigned char ** mip1=NULL, unsigned char ** mip2=NULL, unsigned char ** mip3=NULL, unsigned char ** mip4=NULL)=0;
	virtual bool SkinPoint(short & s, short & t)=0;
	virtual bool Triangle(int & v1, int & v2, int & v3, int & sp1, int & sp2, int & sp3)=0;
	virtual bool Frame ( char * name = NULL, unsigned short * min=NULL, unsigned short * max=NULL, float * scale=NULL, float * origin=NULL)=0;
	virtual bool Vertex ()=0;
	virtual bool Vertex ( unsigned short * p_pos, int & normal_index)=0;
	virtual bool VertexN(float &x,float &y,float &z) { return false; }
};



class MDL7_DECLSPEC MDLIDPOR : public MDLR
{
public:
	union {
	void			*hd;
	MDLIDPO_HEADER	*hdidpo;
	MDLIDP2_HEADER	*hdidp2;
	MDL3_HEADER		*hd3;
	MDL6_HEADER		*hd6;
	MDL3AND6_UNI_HEADER *uni_hd;
	};

	int				sk_bypp;		
	bool			sk_mips;

	union {
	MDLIDPO_SKINPOINT * spidpo;
	MDL3_SKINPOINT	* sp;
	};

	union {
	MDLIDPO_TRIANGLE * tridpo;
	MDL3_TRIANGLE	* tr;
	};
	
	union {
	MDL3_VERTEX		* v3;
	MDL4_VERTEX		* v4;
	};

	union {
	IDP2_FRAME		* fridp2;
	MDL3_FRAME		* fr3;
	MDL4_FRAME		* fr4;
	};
public:
	MDLIDPOR();
	virtual ~MDLIDPOR();

	virtual int NumSkins();
	virtual int NumVertices();
	virtual int NumTriangles();
	virtual int NumSkinPts();
	virtual int NumFrames();

	virtual long Load ( unsigned char * ptr, long size = 0x7fffffff); 
	virtual bool Close();
	virtual bool Head ( void ** head);

	virtual bool Skin(unsigned char & typ, int & width, int & height, int & bytes_perpixel, bool & mips);
	//virtual bool Skin ( char * buf, int size);
	virtual bool SkinBits ( unsigned char ** mip1=NULL, unsigned char ** mip2=NULL, unsigned char ** mip3=NULL, unsigned char ** mip4=NULL);
	virtual bool SkinPoint(short & s, short & t);
	virtual bool Triangle(int & v1, int & v2, int & v3, int & sp1, int & sp2, int & sp3);
	virtual bool Frame ( char * name = NULL, unsigned short * min=NULL, unsigned short * max=NULL, float * scale=NULL, float * origin=NULL);
	virtual bool Vertex ();
	virtual bool Vertex ( unsigned short * p_pos, int & normal_index);

	virtual bool SkipBones();
	virtual bool SkipGroup();
	virtual bool SkipSkin();
	virtual bool SkipSkins();
	virtual bool SkipSPoints();
	virtual bool SkipTriangles();
	virtual bool SkipVertices();
	virtual bool SkipFrame();
	virtual bool SkipFrames();
};


class MDL7_DECLSPEC MDL3R : public MDLIDPOR
{
public:

public:
	MDL3R();
	virtual ~MDL3R();
	
	virtual int NumSkins();
	virtual int NumVertices();
	virtual int NumTriangles();
	virtual int NumSkinPts();
	virtual int NumFrames();
	
	long Load ( unsigned char * ptr, long size = 0x7fffffff); 
	bool SkinPoint(short & s, short & t);
	bool Triangle(int & v1, int & v2, int & v3, int & sp1, int & sp2, int & sp3);

	bool SkipSPoints();
	bool SkipTriangles();

};


class MDL7_DECLSPEC MDLIDP2R : public MDL3R
{
public:
	IDP2_SKIN * sk;

public:
	MDLIDP2R();
	virtual ~MDLIDP2R();
	
	int NumSkins();
	int NumVertices();
	int NumTriangles();
	int NumSkinPts();
	int NumFrames();

	long Load ( unsigned char * ptr, long size = 0x7fffffff); 
	
	virtual bool Skin ( unsigned char & typ, int & width, int & height, int & bytes_perpixel, bool & mips);
//	virtual bool Skin ( char * buf, int size);
	virtual bool SkinBits ( unsigned char ** mip1=NULL, unsigned char ** mip2=NULL, unsigned char ** mip3=NULL, unsigned char ** mip4=NULL);
	virtual bool Frame ( char * name = NULL, unsigned short * min=NULL, unsigned short * max=NULL, float * scale=NULL, float * origin=NULL);

	bool SkipSkin();
//	bool SkipSkins();
};



class MDL7_DECLSPEC MDL4R : public MDL3R
{
public:
	MDL4R();
	virtual ~MDL4R();
	long Load ( unsigned char * ptr, long size = 0x7fffffff); 
	bool Frame ( char * name = NULL, unsigned short * min=NULL, unsigned short * max=NULL, float * scale=NULL, float * origin=NULL);
	bool Vertex ();
	bool Vertex ( unsigned short * p_pos, int & normal_index);

	bool SkipFrame();
};


class MDL7_DECLSPEC MDL5R : public MDL4R
{
public:
	MDL5_SKIN		* sk;
	
public:
	MDL5R();
	virtual ~MDL5R();
	
	long Load ( unsigned char * ptr, long size = 0x7fffffff); 
	bool Skin(unsigned char & typ, int & width, int & height, int & bytes_perpixel, bool & mips);
	bool SkinBits ( unsigned char ** mip1=NULL, unsigned char ** mip2=NULL, unsigned char ** mip3=NULL, unsigned char ** mip4=NULL);
	bool Close();
//--
};



class MDL7_DECLSPEC MDL6R : public MDL5R
{
public:
	MD7_BONE		* bn;
	MD7_BONETRANS	* bt;

public:
	MDL6R();
	~MDL6R();

	long Load ( unsigned char * ptr, long size = 0x7fffffff); 
	bool Bones();
	bool Bone(int & parent_index,float & x, float & y, float & z, char * name = NULL );
	bool FBoneMatrix(int & bone_index, float * m);
	bool Close();
};


#endif