#ifndef __INC_VECTOR_STC_H__
#define __INC_VECTOR_STC_H__

#ifndef _INC_MATH
#include <math.h>
#endif


struct VECTOR4_STC;
struct IVECTOR3_STC;

struct  VECTOR3_STC 
{
	union {
		struct {
			float x;
			float y;
			float z;
		};

		float m[3];
	};



	inline void set ( const VECTOR3_STC & src) { x = src.x, y = src.y; z = src.z; }
	inline void set ( float xp, float yp, float zp) { x=xp, y=yp; z=zp; }
	inline void setd ( double xp, double yp, double zp) { x=(float)xp, y=(float)yp; z=(float)zp; }
	inline void set ( float xp, float yp) { x=xp, y=yp; }
	inline void set ( const IVECTOR3_STC & src);
	
	inline VECTOR3_STC & add (const VECTOR3_STC & src)  {  x+=src.x; y+=src.y; z+=src.z; return *this; }
	inline VECTOR3_STC & add (float v)  {  x+=v; y+=v; z+=v; return *this; }
	inline VECTOR3_STC & add (float sx, float sy, float sz)  {  x+=sx; y+=sy; z+=sz; return *this; }
	inline VECTOR3_STC & add (float sx, float sy)  {  x+=sx; y+=sy; return *this; }

	inline VECTOR3_STC & sub (const VECTOR3_STC & src)  { 	x-=src.x; y-=src.y; z-=src.z; return *this; }
	inline VECTOR3_STC & sub (float v)  { x-=v; y-=v; z-=v; return *this; }
	inline VECTOR3_STC & sub (float sx, float sy, float sz)  { x-=sx; y-=sy; z-=sz; return *this; }
	inline VECTOR3_STC & sub (float sx, float sy)  { x-=sx; y-=sy; return *this; }
		
	inline VECTOR3_STC & mul ( const VECTOR3_STC & src) { x*=src.x; y*=src.y; z*=src.z; return *this; }
	inline VECTOR3_STC & mul ( const float & v) { x*=v; y*=v; z*=v; return *this; }
	inline VECTOR3_STC & mul ( float sx, float sy, float sz) { x*=sx; y*=sy; z*=sz; return *this; }
	inline VECTOR3_STC & mul ( int v) { x*=v; y*=v; z*=v; return *this; }

	inline VECTOR3_STC & div ( const VECTOR3_STC & src)  { x/=src.x; y/=src.y; z/=src.z; return *this; }
	inline VECTOR3_STC & div ( const float & v)  { x/=v; y/=v; z/=v; return *this; }
	inline VECTOR3_STC & div ( float sx, float sy, float sz) { x/=sx; y/=sy; z/=sz; return *this; }
	inline VECTOR3_STC & div ( int v)  { x/=v; y/=v; z/=v; return *this; }

	inline BOOL equal (const VECTOR3_STC & v) const { return  ( x==v.x && y==v.y && z==v.z); }
	inline BOOL equal (float sx, float sy, float sz) const { return  ( x==sx && y==sy && z==sz); }
	inline BOOL equal (const VECTOR3_STC & v, float d) const { return (fabs(x-v.x) <= d && fabs(y-v.y) <= d && fabs(z-v.z) <= d); }
	inline BOOL equal (float sx, float sy, float sz, float d) const { return (fabs(x-sx) <= d && fabs(y-sy) <= d && fabs(z-sz) <= d); }

	inline void vabs() { x = (float)fabs(x); y = (float)fabs(y); z = (float)fabs(z);}

	inline float get_biggest() const { 
		if ( fabs(x) >= fabs(y))
			return fabs(x) > fabs(z) ? x : z;

		if ( fabs(y) >= fabs(x))
			return fabs(y) > fabs(z) ? y : z;

		return z;
	}

	inline float get_biggest(VECTOR3_STC & o) const { 
		float r=0;
		o.set(0,0,0);
		if ( fabs(x) >= fabs(y))
		{
			if ( fabs(x) > fabs(z))
				r = o.x = x;
			else
				r = o.z = z;
		}
		else
		if ( fabs(y) >= fabs(x))
		{
			if ( fabs(y) > fabs(z))
				r = o.y = y;
			else
				r = o.z = z;
		}
		else
			r = o.z=z;

		return (float)fabs(r);
	}

	operator float* () { return & (x); }

};


struct  VECTOR2_STC 
{
	float x;
	float y;


	inline void set (float _x, float _y) { x=_x; y=_y;}
	inline void add (float _x, float _y) { x+=_x; y+=_y;}
	inline void sub (float _x, float _y) { x-=_x; y-=_y;}
	inline void mul (float v) { x*=v; y*=v;}

	inline void set ( const VECTOR2_STC & v) { x=v.x; y=v.y;}
	inline void add ( const VECTOR2_STC & v) { x+=v.x; y+=v.y;}
	inline void sub ( const VECTOR2_STC & v) { x-=v.x; y-=v.y;}
	inline void div (float v) { x/=v; y/=v;}

	inline bool equal(const VECTOR2_STC & v2, float delta ) const { return (fabs(x-v2.x) <= delta  &&  fabs(y-v2.y) <= delta); }
};




struct VECTOR4_STC 
{
	union {
		struct {
			float x;
			float y;
			float z;
			float w;
		};

		float m[4];
	};

	inline void set ( const VECTOR4_STC & src) { x = src.x, y = src.y; z = src.z; w = src.w; }
	inline void set ( float xp, float yp, float zp, float wp) { x=xp, y=yp; z=zp; w=wp;}
	inline void setd ( double xp, double yp, double zp, double wp) { x=(float)xp, y=(float)yp; z=(float)zp; w=(float)wp; }
	inline void set ( float xp, float yp, float zp) { x=xp, y=yp; z=zp; w=1.0f;}
	inline void setd ( double xp, double yp, double zp) { x=(float)xp, y=(float)yp; z=(float)zp; w=1.0f; }
};


#endif
