#ifndef __mdl_h_
#define __mdl_h_

#include "inc\\vector.h"
#include "mdl7.h"

class cBone {
public:
	char name[MAX_BONENAMESIZE]; //bone name
	float x,y,z; // initial position
	int parent; // parent bone index
};


class cVertex{
public:
	float x,y,z; //position
	float nx,ny,nz; //normal
	int bone; // bone index
};

class cSkinpoint{
public:
	float s,t; 
};

class cFace {
public:
	int p1,p2,p3;
	int sp1,sp2,sp3;
	int skin;
};

class cLoDFace {
public:
	float level;
	cFace face;
};


class cFrameVertex {
public:
	int vertexindex;
	float x,y,z; //position
	float nx,ny,nz; //normal
};


class cAnimationsMatrix {
public:
	int bone;
	float matrix4x4[16];
};

class cFrame {
public:
	char name [ MAX_FRAMENAMESIZE];
	
	int vertices;
	cFrameVertex * verticeslist;

	int matrices;
	cAnimationsMatrix * mtrxlist;

public:
	int numvertices() { return vertices; }
	int nummatrices() { return matrices; }
};

class cGroup {
public:
	char name[MAX_GROUPNAMESIZE];
	int vertices;
	cVertex * vertexlist;

	int skinpoints;
	cSkinpoint * skinpointslist;

	int faces;
	cFace * faceslist;

	int lodfaces;
	cLoDFace * lodfacelist;

	int frames;
	cFrame * framelist;

public:
	int numvertices() const { return vertices; }
	int numskinpoints() const { return skinpoints; }
	int numfaces() const { return faces; }
	int numlodfaces() const { return lodfaces; }
	int numframes() const { return frames; }
};


class cTexture {
public:
	int cx,cy;
	int bytesperpixel;
	bool rgb;
	int ddsfilesize;
	union {
		unsigned char * pic;
		unsigned char * mip1;
		unsigned char * dds;
	};
	unsigned char * mip2;
	unsigned char * mip3;
	unsigned char * mip4;
};

class cMaterial {
public:
	float diffuse[4];
	float ambient[4];
	float specular[4];
	float emissive[4];
	float power;
};

class cEffect {
public:
	bool filename;
	char * text;
	int textlen;
};


class cSkin {
public:
	char name[MAX_TEXNAMESIZE];	
	
	int sharedtex;
	char * picfile;
	cTexture * pic;
	cMaterial * material;
	cEffect * effect;
};


//provisorium
class mdl7data {
public:
	int bones; // 4
	cBone * bones_list; 
	
	int skins;
	cSkin * skinlist;

	int groups; // 4
	cGroup * group_list;


public:
	int numbones() const { return bones; }
	int numskins() const { return skins; }
	int numgroups() const { return groups; }
	void error() const {}
};



//static model data
extern mdl7data	mdl;


#endif