#include "windows.h"
#include "mdl7.h"
#include "save_mdl7.h"


template <class T> bool save_skins ( T & o, const mdl7data & dt);
template <class T> bool save_frames ( T & mdl7, const cGroup & gr);
template <class T> bool save_lod_faces ( T & mdl7, const mdl7data & dt);

bool save_mdl7 ( const char * file, const mdl7data & dt)
{
	MDL7W	mdl7;
	int		handle;

	handle = (int) mdl7.Open ( file);

	if ( handle < 0)
		return 0;

	if ( dt.numbones() )
	{
		for ( int i=0; i<dt.numbones(); i++)
		{
			if ( !mdl7.Bone (
				dt.bones_list[i].parent,
				dt.bones_list[i].x,
				dt.bones_list[i].y,
				dt.bones_list[i].z,
				dt.bones_list[i].name)
				)
			{ dt.error(); return false; }
		}
	}

	if ( dt.numgroups())
	{
		for ( int g=0; g<dt.numgroups(); g++)
		{
			int i;
			cGroup & gr = dt.group_list[g];

			if ( !mdl7.Group(1,gr.name)) 
				{ dt.error(); return false; }

			if ( g == 0)
				save_skins ( mdl7, dt);

 
			if ( gr.numskinpoints())
			{
				for ( i=0; i<gr.numskinpoints(); i++)
				{
					if ( !mdl7.SkinPoint(gr.skinpointslist[i].s,gr.skinpointslist[i].t)) 
					{ dt.error(); return false; }
				}
			}

			for ( i=0; i<gr.numfaces(); i++)
			{
				if ( !mdl7.Triangle(
						gr.faceslist[i].p1,
						gr.faceslist[i].p2,
						gr.faceslist[i].p3,
						gr.faceslist[i].sp1,
						gr.faceslist[i].sp2,
						gr.faceslist[i].sp3,
						gr.faceslist[i].skin)
					)
				{ dt.error(); return false; }
			}
		
			for ( i=0; i<gr.numvertices(); i++)
			{
				if ( ! mdl7.Vertex ( 
						gr.vertexlist[i].x, 
						gr.vertexlist[i].y, 
						gr.vertexlist[i].z, 

						gr.vertexlist[i].bone, 

						gr.vertexlist[i].nx, 
						gr.vertexlist[i].ny, 
						gr.vertexlist[i].nz)
					)
				{ dt.error(); return false; }
			}

			if ( !save_frames ( mdl7, gr))
				{ dt.error(); return false; }

			mdl7.GroupDone();
		}
	}

	if ( ! save_lod_faces(mdl7,dt))
		{ dt.error(); return false; }
	
	return mdl7.Close();
}




void get_texturetyp ( cTexture & tex, int & typ, int & bypp, bool & mips)
{
	if ( tex.ddsfilesize > 0)
	{
		typ = M7SKINTYP_DDS_FILE;
		bypp = 1;
	}
	else
	if ( tex.bytesperpixel == 1)
	{
		typ = M7SKINTYP_8BIT_PAL256; // 0
		bypp = 1;
	}
	else
	if ( tex.bytesperpixel == 2)
	{
		typ = M7SKINTYP_16BIT_565;
		bypp = 2;
	}
	else
	if ( tex.bytesperpixel == 3)
	{
		typ = M7SKINTYP_24BIT_888;
		bypp = 3;
	}
	else
	if ( tex.bytesperpixel == 4)
	{
		typ = M7SKINTYP_32BIT_8888;
		bypp = 4;
	}

	if ( bypp > 1 && tex.mip1 && tex.mip2 && tex.mip3 && tex.mip4)
	{
		typ |= M7SKINTYP_MIPFLAG;
		mips = true;
	}

	if ( tex.rgb)
		typ |= M7SKINTYP_RGBFLAG;
}


template <class T> bool save_skins ( T & o, const mdl7data & dt)
{
	for ( int i=0; i<dt.numskins(); i++)
	{
		cSkin & skin = dt.skinlist[i];
		int mtrl_typ = 0;
		int x_size=0;
		int y_size=0;
		int bypp=0;
		bool mips=0;

		
		if ( skin.sharedtex >= 0)
		{
			mtrl_typ = M7SKINTYP_MATERIAL_INDEX;
			x_size = skin.sharedtex;
			y_size = 0;
			bypp = 0;
		}
		else
		if ( skin.picfile)
		{
			mtrl_typ = M7SKINTYP_EXT_FILE;
			x_size = strlen(skin.picfile) + 1;
			y_size = 1;
			bypp = 1;
		}
		else
		if ( skin.pic)
		{
			get_texturetyp ( *skin.pic,mtrl_typ,bypp,mips);

			x_size = skin.pic->cx;
			y_size = skin.pic->cy;

			if ( mtrl_typ == M7SKINTYP_DDS_FILE) // dds file
			{
				x_size = skin.pic->ddsfilesize;
				y_size = 1;
				bypp = 1;
			}
		}


		if ( skin.material)
			mtrl_typ |= M7SKINTYP_MATERIAL;

		if ( skin.effect)
			mtrl_typ |= M7SKINTYP_MATERIAL_ASCDEF;


		o.Skin ( mtrl_typ, skin.name);
		o.SkinSize ( x_size, y_size, bypp, mips);

		if ( skin.sharedtex >= 0)
		{
			;
		}
		else
		if ( skin.picfile )
		{
			o.SkinBits ( (BYTE*)skin.picfile,0,0,0);
		}
		else
		if ( skin.pic )
		{
			o.SkinBits( 
				skin.pic->pic,
				mips ? skin.pic->mip2 : NULL,
				mips ? skin.pic->mip3 : NULL,
				mips ? skin.pic->mip4 : NULL );
		}

		
		if ( skin.material)
			o.SkinMaterial(
				skin.material->diffuse, skin.material->ambient, 
				skin.material->specular, skin.material->emissive, skin.material->power);


		if ( skin.effect)
		{
			if ( skin.effect->filename)
				o.SkinMaterial ( strlen(skin.effect->text)+1, skin.effect->text);
			else
				o.SkinMaterial ( skin.effect->textlen, skin.effect->text);
		}
		
		o.SkinDone();
	}

	return true;

}


template <class T> bool save_frames ( T & mdl7, const cGroup & gr)
{
	for ( int f=0; f<gr.numframes(); f++)
	{
		cFrame & fr = gr.framelist[f];

		if ( ! mdl7.Frame ( fr.name))
			return false;

		int i;
		for ( i=0; i<fr.numvertices(); i++)
		{
			if( ! mdl7.FVertex( 
					fr.verticeslist[i].vertexindex,
					fr.verticeslist[i].x,
					fr.verticeslist[i].y,
					fr.verticeslist[i].z,
					fr.verticeslist[i].nx,
					fr.verticeslist[i].ny,
					fr.verticeslist[i].nz)
					)
					return false;
		}

		for ( i=0; i<fr.nummatrices(); i++)
		{
			if ( ! mdl7.FBoneMatrix (
					fr.mtrxlist[i].bone,
					fr.mtrxlist[i].matrix4x4)
					)
					return false;
		}

		mdl7.FrameDone();
	}

	return true;
}


template <class T> bool save_lod_faces ( T & mdl7, const mdl7data & dt)
{
	{
		int lods=0;
		for ( int g=0; g<dt.numgroups(); g++)
			lods += dt.group_list[g].numlodfaces();
		if ( !lods)
			return true;
	}
	
	
	if ( ! mdl7.LumpLoD())
		return false;

	for ( int g=0; g<dt.numgroups(); g++)
	{
		cGroup & gr = dt.group_list[g];
		if ( !gr.numlodfaces())
			continue;

		if ( !mdl7.LumpLoDGroup(g))
			return false;

		float level = gr.lodfacelist[0].level;

		for ( int fs=0; fs<gr.numlodfaces(); )
		{
			if ( !mdl7.LumpLoDData(level))
				return false;
				
			for ( ; fs<gr.numlodfaces() ; fs++)
			{
				if ( level != gr.lodfacelist[fs].level)
				{
					level = gr.lodfacelist[fs].level;
					break;
				}

				if ( ! mdl7.LumpLoDTriangle(
					gr.lodfacelist[fs].face.p1,
					gr.lodfacelist[fs].face.p2,
					gr.lodfacelist[fs].face.p3,
					gr.lodfacelist[fs].face.sp1,
					gr.lodfacelist[fs].face.sp2,
					gr.lodfacelist[fs].face.sp3,
					gr.lodfacelist[fs].face.skin))		
						return false;
			}

			mdl7.LumpLoDDataDone();
		}
				
		mdl7.LumpLoDGroupDone();
	}

	mdl7.LumpLoDDone ();
	return true;
}