/*
PERAGRO -latin for "moving through"
		 a plugin for gamestudio based on recast/detour 
		 to make use of pathfinding on a navigational mesh;

note:	-uses some altered recast/detour source
		-uses some irrlicht source/library for matrix functions
		-uses gamestudio dll plugin files and mdl7 functions
		
		-my source code is free please do not claim
		 to have written it yourself !

		WJBENDER
*/

#include "stdafx.h"

//acknex
#define DLL_USE	
#include "..\\acknex\\adll.h"

//navbuilder
#include "buildmeshstruct.h"
#include "navbuilder.h"
#include "process.h"

//used for matrix transform functions
#include <irrlicht.h>
using namespace irr;
using namespace core;

//threads 
#include "workerthread.h"
workerthread *m_worker;

class navbuilder* m_navbuilder;

//tempory lists
m_link* my_temp_ent_list;
m_link* my_temp_pnt_list;
m_link *tmp_tiles_list;
m_link *tiledata_list;


//////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////
//************************************
// DLL STARTUP
BOOL APIENTRY DllMain( HMODULE hModule,DWORD  ul_reason_for_call,LPVOID lpReserved)
{
	engine_bind();
	return TRUE;
}

//////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////
//************************************
// NAVMESH CLASS/OBJECT FUNCTIONS
//
DLLFUNC void peragro_create()
{
	my_temp_ent_list=new m_link();
	my_temp_pnt_list=new m_link();
	tmp_tiles_list=new m_link();
	tiledata_list=new m_link();

	if(!my_temp_ent_list || !my_temp_pnt_list)
	{
		MessageBoxA(NULL,"peragro_create: could not create tmp lists","error",MB_OK);
	}

	m_navbuilder=new navbuilder();
	if(!m_navbuilder)
	{
		MessageBoxA(NULL,"peragro_create: could not create","error",MB_OK);
	}
}

DLLFUNC void peragro_destroy()
{
	if(m_navbuilder)
	{
		my_temp_ent_list->~m_link();
		my_temp_pnt_list->~m_link(); 
		tmp_tiles_list->~m_link();
		tiledata_list->~m_link();
		if(m_worker)
		{
			m_worker->~workerthread();
			delete m_worker;
			m_worker=0;
		}
		m_navbuilder->~navbuilder(); 
	}
}

DLLFUNC int peragro_valid()
{
	if(m_navbuilder) return 1;
	return 0;
}

//////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////
//************************************
// CUSTOM PARAM BUILD VALUES
//
DLLFUNC custombuild* cbuild_create()
{
	custombuild* newbuild=new custombuild;
	if(newbuild)
	{
		newbuild->CellSize  = 0.3f;
		newbuild->CellHeight= 0.2f;
		newbuild->AgentHeight=2.0f;
		newbuild->AgentRadius= 0.6f;
		newbuild->AgentMaxClimb = 0.9f;
		newbuild->AgentMaxSlope = 45.0f;
		newbuild->RegionMinSize = 8;
		newbuild->RegionMergeSize = 20;
		newbuild->monotonePartitioning =0;
		newbuild->EdgeMaxLen  = 12.0f;
		newbuild->EdgeMaxError= 1.3f;
		newbuild->VertsPerPoly = 6.0f;
		newbuild->DetailSampleDist= 6.0f;
		newbuild->DetailSampleMaxError =1.0f;
		newbuild->TileSize=32;
		newbuild->buildalltiles=1;
		return newbuild;
	}
	MessageBoxA(NULL,"create custombuild :could not create","error",MB_OK);
	return NULL;
}

DLLFUNC void cbuild_remove(custombuild* mybuild)
{
	if(mybuild)
	{
		delete mybuild;
		mybuild=0;
	}
}

DLLFUNC void cbuild_save(char* filename,custombuild *step)
{
	if(m_navbuilder && step)m_navbuilder->Export_Custombuild(filename,step);
	if(!m_navbuilder || !step)MessageBoxA(NULL,"export custombuild: no navbuilder or input param","error",MB_OK);
}

DLLFUNC custombuild* cbuild_load(char* filename)
{
	if(m_navbuilder)return(m_navbuilder->Load_Custombuild(filename));
	if(!m_navbuilder)MessageBoxA(NULL,"load custombuild: no navbuilder","error",MB_OK);
	return NULL;
}

//////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////
//************************************
// NAVMESH BUILDING 
//

//	MAIN WORKER BUILD THREAD
DWORD WINAPI Thread_Build_Navmesh(LPVOID param)
{ 
	if(m_navbuilder)
	{
		custombuild* step=(custombuild*)param;
		int i=m_navbuilder->Nav_From_Custombuild(step);
		return 1;
	}
	if(!m_navbuilder)
	{
		MessageBoxA(NULL,"build navmesh: no navbuilder.","error",MB_OK);
	}
	return 1;
} 

DLLFUNC void* peragro_init(custombuild* step)
{
	 if(m_navbuilder && step)
	 {
		m_worker=new workerthread();
		HANDLE hthread=m_worker->get_handle();
		m_worker->set_func(Thread_Build_Navmesh);
		m_worker->set_param((custombuild*)step);
		m_worker->signal_run();
		Sleep(3);

		//delete(step);
		if(!hthread)
		{
			MessageBoxA(NULL,"init navbuilder: thread failed","error",MB_OK);
			return NULL;
		}
		return (HANDLE*)hthread;
	 }
	 if(!m_navbuilder || !step)
	 {
		MessageBoxA(NULL,"init navbuilder: no navbuilder or input param","error",MB_OK);
	 }
 return NULL;
}

//////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////
//************************************
// LIST FUNCTIONS
//
DLLFUNC void clear_meshlist()
{
	if(!m_navbuilder)MessageBoxA(NULL,"clear list: no navbuilder","error",MB_OK);
	if(m_navbuilder)
	{
		m_link *cur;
		for(cur=m_navbuilder->my_build_list->LHead;cur!=NULL;cur=cur->LNext)
		{
			my_mesh *pmesh=(my_mesh*)cur->Data;
			m_navbuilder->my_build_list->LPos =cur;
			m_navbuilder->my_build_list->Data=NULL;
			m_navbuilder->my_build_list->Remove();
			delete(pmesh);
			pmesh=NULL;
		}
	}
}

DLLFUNC void add_meshlist(void* mesh)
{
	if(!m_navbuilder)MessageBoxA(NULL,"add list: no navbuilder","error",MB_OK);
	if(m_navbuilder)m_navbuilder->my_build_list->Add((my_mesh*)mesh);
}

DLLFUNC void* get_meshlist_ent(ENTITY* ent,int chunk)
{
	if(!ent)MessageBoxA(NULL,"get list ent: no ent","error",MB_OK);
	if(!m_navbuilder)MessageBoxA(NULL,"get list ent: no navbuilder","error",MB_OK);
	if(m_navbuilder && ent)
	{
		m_link *cur;
		for(cur=m_navbuilder->my_build_list->LHead;cur!=NULL;cur=cur->LNext)
		{
			my_mesh *pmesh=(my_mesh*)cur->Data;
			if(pmesh->ent==ent)
			{
				if(pmesh->chunk==chunk)
				{
					return(my_mesh*)pmesh;
				}
			}
		}
	}
	return NULL;
}

DLLFUNC void rem_meshlist_ent(ENTITY* ent,int chunk)
{
	if(!ent)MessageBoxA(NULL,"remove list ent: no ent","error",MB_OK);
	if(!m_navbuilder)MessageBoxA(NULL,"remove list ent: no navbuilder","error",MB_OK);
	if(m_navbuilder && ent)
	{
		m_link *cur;
		for(cur=m_navbuilder->my_build_list->LHead;cur!=NULL;cur=cur->LNext)
		{
			my_mesh *pmesh=(my_mesh*)cur->Data;
			if(pmesh->ent==ent)
			{
				if(pmesh->chunk==chunk && chunk>-1)
				{
					m_navbuilder->my_build_list->LPos =cur;
					m_navbuilder->my_build_list->Data=NULL;
					m_navbuilder->my_build_list->Remove();
					delete(pmesh);
					pmesh=NULL;
				}
				else if(chunk==-1)
				{
					m_navbuilder->my_build_list->LPos =cur;
					m_navbuilder->my_build_list->Data=NULL;
					m_navbuilder->my_build_list->Remove();
					delete(pmesh);
					pmesh=NULL;
				}
			}
		}
	}
}

DLLFUNC void rem_meshlist(void* build_mesh)
{
	if(!m_navbuilder)MessageBoxA(NULL,"rem mesh list: no navbuilder","error",MB_OK);
	if(m_navbuilder)
	{
		m_link *cur;
		for(cur=m_navbuilder->my_build_list->LHead;cur!=NULL;cur=cur->LNext)
		{
			my_mesh *pmesh=(my_mesh*)cur->Data;
			if(pmesh==build_mesh)
			{
				m_navbuilder->my_build_list->LPos =cur;
				m_navbuilder->my_build_list->Data=NULL;
				m_navbuilder->my_build_list->Remove();
				delete(pmesh);
				pmesh=NULL;
				delete(pmesh);
				pmesh=NULL;
			}
		}
	}
}

//////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////
//************************************
// INPUT GEOMETRY DATA 
//

DLLFUNC void trans_mesh_ent(void* mesh,ENTITY* ent)
{
	if(!ent)MessageBoxA(NULL,"transform mesh: no ent","error",MB_OK);
	if(!m_navbuilder)MessageBoxA(NULL,"transform mesh: no navbuilder","error",MB_OK);
	if(m_navbuilder && ent)
	{
		my_mesh *pmesh=(my_mesh*)mesh;
		if(pmesh)
		{
			int vcount=pmesh->v_count; 
			int tcount=pmesh->t_count;
			
			for(int v=0;v<vcount;v++)
			{
				float rx,ry,rz;
				float sx,sy,sz;
				float px,py,pz;
				rx=_FLOAT(ent->roll);
				ry=_FLOAT(ent->pan);
				rz=_FLOAT(ent->tilt);

				sx=_FLOAT(ent->scale_x);
				sy=_FLOAT(ent->scale_y);
				sz=_FLOAT(ent->scale_z);

				px=_FLOAT(ent->x);
				pz=_FLOAT(ent->y);
				py=_FLOAT(ent->z);

				CMatrix4<float> mat;
				//get matrix from entity angle
				float matang[16];
				//ang_to_matrix((ANGLE*)&ent->pan,matang);
				ang_to_matrix((ANGLE*)&ent->pan,matang);
				//setup matrix rotation
				for(int i=0;i<16;i++)
				{
					mat[i]=matang[i];
				}
				//setup matrix translation
				mat.setTranslation(vector3d<float>(px,py,pz));
				//vertex vector to be transformed
				vector3d<float> in;
				//scale vertices
				pmesh->vertices[v*3]*=sx;
				pmesh->vertices[v*3+1]*=sz;
				pmesh->vertices[v*3+2]*=sy;
				
				in.X=pmesh->vertices[v*3];
				in.Y=pmesh->vertices[v*3+1];
				in.Z=pmesh->vertices[v*3+2];
				//rotate vertices
				mat.rotateVect(in);
				//translate vertices
				mat.translateVect(in);
				//update transformed vertices
				pmesh->vertices[v*3]  =in.X;
				pmesh->vertices[v*3+1]=in.Y;
				pmesh->vertices[v*3+2]=in.Z;
			}
		}
	}
}

DLLFUNC void trans_mesh_arg(void* mesh,ANGLE* angle,VECTOR* scale,VECTOR* pos)
{
	if(!m_navbuilder)MessageBoxA(NULL,"transform mesh: no navbuilder","error",MB_OK);
	if(m_navbuilder)
	{
		my_mesh *pmesh=(my_mesh*)mesh;
		if(pmesh)
		{
			int vcount=pmesh->v_count; 
			int tcount=pmesh->t_count;
			
			for(int v=0;v<vcount;v++)
			{
				float sx,sy,sz;
				float px,py,pz;
				sx=_FLOAT(scale->x);
				sy=_FLOAT(scale->y);
				sz=_FLOAT(scale->z);

				px=_FLOAT(pos->x);
				pz=_FLOAT(pos->y);
				py=_FLOAT(pos->z);
	
				CMatrix4<float> mat;
				//get matrix from entity angle
				float matang[16];
				ang_to_matrix(angle,matang);
				//setup matrix rotation
				for(int i=0;i<16;i++)
				{
					mat[i]=matang[i];
				}
				//setup matrix translation
				mat.setTranslation(vector3d<float>(px,py,pz));
				//vertex vector to be transformed
				vector3d<float> in;
				//scale vertices
				pmesh->vertices[v*3]*=sx;
				pmesh->vertices[v*3+1]*=sz;
				pmesh->vertices[v*3+2]*=sy;
				
				in.X=pmesh->vertices[v*3];
				in.Y=pmesh->vertices[v*3+1];
				in.Z=pmesh->vertices[v*3+2];
				//rotate vertices
				mat.rotateVect(in);
				//translate vertices
				mat.translateVect(in);
				//update transformed vertices
				pmesh->vertices[v*3]=in.X;
				pmesh->vertices[v*3+1]=in.Y;
				pmesh->vertices[v*3+2]=in.Z;
			}
		}
	}
}

DLLFUNC void* mesh_from_ent(ENTITY* ent,int _lod)
{
	if(m_navbuilder)
	{
		if(ent)
		{
			D3DVERTEX *vbuffer=NULL;
			short *ibuffer=NULL;
			int numvertices=0;
			int numtriangles=0;
			numtriangles=_INT(ent_buffers(ent,_VAR(0),_VAR(_lod),(void**)&vbuffer,(void**)&ibuffer,NULL));
			if(numtriangles>0)
			{
				my_mesh* new_mesh=new my_mesh();

				for(int t=0;t<numtriangles;t++)
				{
					int p1=ibuffer[t*3];
					int p2=ibuffer[t*3+1];
					int p3=ibuffer[t*3+2];
					if(p1>p2 && p1>p3 && p1>numvertices)numvertices=p1;
					if(p2>p1 && p2>p3 && p2>numvertices)numvertices=p2;
					if(p3>p1 && p3>p2 && p3>numvertices)numvertices=p3;
				}
			////////////////////////////////////////////////////////
				numvertices+=1;
			///////////////////////////////////////////////////////

				new_mesh->vertices=(float*)malloc(sizeof(float)*numvertices*3);
				new_mesh->triangles=(int*)malloc(sizeof(int)*numtriangles*3);
				
				for(int v=0;v<numvertices;v++)
				{
					new_mesh->vertices[v*3]=vbuffer[v].x;
					new_mesh->vertices[v*3+1]=vbuffer[v].y;
					new_mesh->vertices[v*3+2]=vbuffer[v].z;
				}

				for(int t=0;t<numtriangles;t++)
				{
					new_mesh->triangles[t*3]=ibuffer[t*3];
					new_mesh->triangles[t*3+1]=ibuffer[t*3+1];
					new_mesh->triangles[t*3+2]=ibuffer[t*3+2];
				}

				new_mesh->t_count=numtriangles;
				new_mesh->v_count=numvertices;
				new_mesh->ent=ent;
				new_mesh->chunk=0;
				//delete(vbuffer);
				//delete(ibuffer);
				trans_mesh_ent((my_mesh*)new_mesh,ent);
				add_meshlist((my_mesh*)new_mesh);
				return(my_mesh*)new_mesh;
			}
		}
	}
	MessageBoxA(NULL,"mesh from ent: no ent or navbuilder","error",MB_OK);
	return NULL;
}

DLLFUNC void* mesh_from_hmp_chunk(ENTITY* ent,int chunk,int _lod)
{
	if(m_navbuilder)
	{
		if(ent)
		{
			D3DVERTEX *vbuffer=NULL;
			short *ibuffer=NULL;
			//int _lod=0;
			int numvertices=0;
			my_mesh* new_mesh=new my_mesh();

			int totvertices=0;
			int tottriangles=0;
			int numtriangles=0;

			numtriangles=_INT(ent_buffers(ent,_VAR(chunk),_VAR(_lod),(void**)&vbuffer,(void**)&ibuffer,NULL));

			for(int t=0;t<numtriangles;t++)
			{
				int p1=ibuffer[t*3];
				int p2=ibuffer[t*3+1];
				int p3=ibuffer[t*3+2];
				if(p1>p2 && p1>p3 && p1>numvertices)numvertices=p1;
				if(p2>p1 && p2>p3 && p2>numvertices)numvertices=p2;
				if(p3>p1 && p3>p2 && p3>numvertices)numvertices=p3;
			}
	////////////////////////////////////////////////////////
			numvertices+=1;
	///////////////////////////////////////////////////////
			
			new_mesh->triangles=(int*)malloc(sizeof(int)*numtriangles*3);

				for(int t=0;t<numtriangles;t++)
				{
					new_mesh->triangles[t*3]=ibuffer[t*3];
					new_mesh->triangles[t*3+1]=ibuffer[t*3+1];
					new_mesh->triangles[t*3+2]=ibuffer[t*3+2];
				}

			new_mesh->vertices=(float*)malloc(sizeof(float)*numvertices*3);

			for(int i=0;i<numvertices;i++)
			{
				new_mesh->vertices[i*3]  =vbuffer[i].x; 
				new_mesh->vertices[i*3+1]=vbuffer[i].y; 
				new_mesh->vertices[i*3+2]=vbuffer[i].z; 
			}
		
			new_mesh->t_count=numtriangles;
			new_mesh->v_count=numvertices;
			new_mesh->ent=ent;
			new_mesh->chunk=chunk;

			add_meshlist((my_mesh*)new_mesh);
			return(my_mesh*)new_mesh;
		}
	}
	MessageBoxA(NULL,"mesh from hmp chunk:no ent or navbuilder","error",MB_OK);
return NULL;
}

DLLFUNC void mesh_from_hmp(ENTITY* ent,int chunks,int _lod)
{
	if(!m_navbuilder || !ent)	MessageBoxA(NULL,"mesh from hmp: no ent or navbuilder","error",MB_OK);
	if(m_navbuilder)
	{
		if(ent)
		{
			int totvertices=0;
			int tottriangles=0;

			for (int i=0;i<chunks;i+=1)
			{
				int numvertices=0;
				D3DVERTEX *vbuffer;
				short *ibuffer;
				//int lod=0;
				my_mesh* new_mesh=new my_mesh();

				int numtriangles=0;
				numtriangles=_INT(ent_buffers(ent,_VAR(i),_VAR(_lod),(void**)&vbuffer,(void**)&ibuffer,NULL));

				for(int t=0;t<numtriangles;t++)
				{
					int p1=ibuffer[t*3];
					int p2=ibuffer[t*3+1];
					int p3=ibuffer[t*3+2];
					if(p1>p2 && p1>p3 && p1>numvertices)numvertices=p1;
					if(p2>p1 && p2>p3 && p2>numvertices)numvertices=p2;
					if(p3>p1 && p3>p2 && p3>numvertices)numvertices=p3;
				}
	////////////////////////////////////////////////////////
				numvertices+=1;
	///////////////////////////////////////////////////////
		
				new_mesh->triangles=(int*)malloc(sizeof(int)*numtriangles*3);

				for(int t=0;t<numtriangles;t++)
				{
					new_mesh->triangles[t*3]=ibuffer[t*3];
					new_mesh->triangles[t*3+1]=ibuffer[t*3+1];
					new_mesh->triangles[t*3+2]=ibuffer[t*3+2];
				}

				new_mesh->vertices=(float*)malloc(sizeof(float)*numvertices*3);

				for(int v=0;v<numvertices;v++)
				{
					new_mesh->vertices[v*3]=vbuffer[v].x;
					new_mesh->vertices[v*3+1]=vbuffer[v].y;
					new_mesh->vertices[v*3+2]=vbuffer[v].z;
				}

				new_mesh->t_count=numtriangles;
				new_mesh->v_count=numvertices;
				new_mesh->ent=ent;
				new_mesh->chunk=i;
				add_meshlist((my_mesh*)new_mesh);
			}
		}
	}
}

DLLFUNC void mesh_from_level_blocks(ENTITY* level,int chunks,int _lod)
{
	if(!m_navbuilder || !level)	MessageBoxA(NULL,"mesh from level blocks: no ent or navbuilder","error",MB_OK);

	if(m_navbuilder)
	{
		if(level)
		{
			for(int i=0;i<chunks;i++)
			{

			short numvertices=0;
			D3DVERTEX *vbuffer=NULL;
			short *ibuffer=NULL;
			//int lod=0;
			my_mesh* levelmesh=new my_mesh();
			
			long numtriangles=_INT(ent_buffers(level,_VAR(i),_VAR(_lod),(void**)&vbuffer,(void**)&ibuffer,NULL));
				for(int t=0;t<(int)numtriangles;t++)
				{
					short p1=ibuffer[t*3];
					short p2=ibuffer[t*3+1];
					short p3=ibuffer[t*3+2];
					if(p1>p2 && p1>p3 && p1>numvertices)numvertices=p1;
					if(p2>p1 && p2>p3 && p2>numvertices)numvertices=p2;
					if(p3>p1 && p3>p2 && p3>numvertices)numvertices=p3;
				}
		////////////////////////////////////////////////////////
				numvertices+=1;
		///////////////////////////////////////////////////////
				levelmesh->triangles=(int*)malloc(sizeof(int)*numtriangles*3);

					for(int t=0;t<numtriangles;t++)
					{
						levelmesh->triangles[t*3]=ibuffer[t*3];
						levelmesh->triangles[t*3+1]=ibuffer[t*3+1];
						levelmesh->triangles[t*3+2]=ibuffer[t*3+2];
					}

				levelmesh->vertices=(float*)malloc(sizeof(float)*numvertices*3);
				
				for(int v=0;v<numvertices;v++)
				{
					levelmesh->vertices[v*3]=vbuffer[v].x;
					levelmesh->vertices[v*3+1]=vbuffer[v].y;
					levelmesh->vertices[v*3+2]=vbuffer[v].z;
				}
				levelmesh->t_count=numtriangles;
				levelmesh->v_count=numvertices;
				levelmesh->ent=level;
				levelmesh->chunk=i;
				//delete(vbuffer);
				//delete(ibuffer);
				add_meshlist((my_mesh*)levelmesh);
			}
		}
	}
}

DLLFUNC void* mesh_from_level_blocks_chunk(ENTITY* level,int chunks,int _lod)
{
	if(m_navbuilder)
	{
		if(level)
		{
			short numvertices=0;
			D3DVERTEX *vbuffer=NULL;
			short *ibuffer=NULL;
			//int lod=0;
			my_mesh* levelmesh=new my_mesh();
			
			long numtriangles=_INT(ent_buffers(level,_VAR(chunks),_VAR(_lod),(void**)&vbuffer,(void**)&ibuffer,NULL));
				for(int t=0;t<numtriangles;t++)
				{
					short p1=ibuffer[t*3];
					short p2=ibuffer[t*3+1];
					short p3=ibuffer[t*3+2];
					if(p1>p2 && p1>p3 && p1>numvertices)numvertices=p1;
					if(p2>p1 && p2>p3 && p2>numvertices)numvertices=p2;
					if(p3>p1 && p3>p2 && p3>numvertices)numvertices=p3;
				}
		////////////////////////////////////////////////////////
				numvertices+=1;
		///////////////////////////////////////////////////////
				levelmesh->triangles=(int*)malloc(sizeof(int)*numtriangles*3);

					for(int t=0;t<numtriangles;t++)
					{
						levelmesh->triangles[t*3]=ibuffer[t*3];
						levelmesh->triangles[t*3+1]=ibuffer[t*3+1];
						levelmesh->triangles[t*3+2]=ibuffer[t*3+2];
					}

				levelmesh->vertices=(float*)malloc(sizeof(float)*numvertices*3);
				
				for(int v=0;v<numvertices;v++)
				{
					levelmesh->vertices[v*3]=vbuffer[v].x;
					levelmesh->vertices[v*3+1]=vbuffer[v].y;
					levelmesh->vertices[v*3+2]=vbuffer[v].z;
				}
				levelmesh->t_count=numtriangles;
				levelmesh->v_count=numvertices;
				levelmesh->ent=level;
				levelmesh->chunk=chunks;
				//delete(vbuffer);
				//delete(ibuffer);
				add_meshlist((my_mesh*)levelmesh);
				return (my_mesh*)levelmesh;
		}
	}
	MessageBoxA(NULL,"mesh from level blocks-chunk: no ent or navbuilder","error",MB_OK);
	return NULL;
}

DLLFUNC void* mesh_from_buffers(D3DVERTEX* vbuffer,short* tbuffer,int verts,int tris)
{
	if(m_navbuilder && vbuffer && tbuffer && verts && tris)
	{
		my_mesh* newmesh=new my_mesh();
		newmesh->chunk=0;
		newmesh->ent=NULL;
		newmesh->t_count=tris;
		newmesh->v_count=verts;
		newmesh->triangles=(int*)malloc(sizeof(int)*tris*3);

		newmesh->vertices=(float*)malloc(sizeof(float)*verts*3);
		memcpy(newmesh->triangles,&tbuffer[0],sizeof(short)*tris*3);
		for(int v=0;v<verts;v++)
		{
			newmesh->vertices[v*3]=vbuffer[v].x;
			newmesh->vertices[v*3+1]=vbuffer[v].y;
			newmesh->vertices[v*3+2]=vbuffer[v].z;
		}
		add_meshlist((my_mesh*)newmesh);
		return (my_mesh*)newmesh;
	}
	MessageBoxA(NULL,"mesh from buffers: no navbuilder or buffers","error",MB_OK);
return NULL;
}

DLLFUNC void* mesh_from_obj_file(char* filename)
{
	if(m_navbuilder)
	{
		rcMeshLoaderObj *m_obj=new rcMeshLoaderObj();
		if(m_obj->load(filename))
		{
			my_mesh* new_mesh=new my_mesh();

			const int *tris=m_obj->getTris();
			const float* verts=m_obj->getVerts();
			int tris_c=m_obj->getTriCount(); 
			int verts_c=m_obj->getVertCount();

			new_mesh->triangles=(int*)malloc(sizeof(int)*tris_c*3);

			new_mesh->vertices=(float*)malloc(sizeof(float)*verts_c*3);

			for(int v=0;v<verts_c;v++)
			{
				new_mesh->vertices[v*3]=verts[v*3];
				new_mesh->vertices[v*3+1]=verts[v*3+1];
				new_mesh->vertices[v*3+2]=verts[v*3+2];
			}
			for(int t=0;t<tris_c;t++)
			{
				new_mesh->triangles[t*3]=tris[t*3];
				new_mesh->triangles[t*3+1]=tris[t*3+1];
				new_mesh->triangles[t*3+2]=tris[t*3+2];
			}
			new_mesh->chunk=0;
			new_mesh->ent=NULL;
			new_mesh->t_count=tris_c;
			new_mesh->v_count=verts_c;
			add_meshlist((my_mesh*)new_mesh);
			delete(m_obj);
			return (my_mesh*)new_mesh;
		}
		delete(m_obj);
	}
	MessageBoxA(NULL,"mesh from obj: no navbuilder or could not load file","error",MB_OK);
	return NULL;
}

//SIMPLE PURE GEOMETRY FILE FORMAT, CANT READ OBJ FILES ,USE MESH_FROM_OBJ_FILE FOR .OBJ
DLLFUNC void* load_mesh(char* filename)
{
	if(m_navbuilder)
	{
		FILE* file=fopen(filename,"r b");
		if (file)
		{
			my_mesh* pmesh=new my_mesh();
			int verts=0;int tris=0;
			fscanf(file,"# %i vertices\n",&verts);
			pmesh->v_count=verts;
			fscanf(file,"# %i triangles\n",&tris);
			pmesh->t_count=tris;

			pmesh->vertices=(float*)malloc(sizeof(float)*pmesh->v_count*3);
			pmesh->triangles=(int*)malloc(sizeof(int)*pmesh->t_count*3);

			for(int v=0;v<pmesh->v_count;v++)
			{
				float x,y,z;
				fscanf(file,"v %f %f %f\n",&x,&y,&z);
				pmesh->vertices[v*3]=x;
				pmesh->vertices[v*3+1]=y;
				pmesh->vertices[v*3+2]=z;
			}
			for(int t=0;t<pmesh->t_count;t++)
			{
				int p1,p2,p3;
				fscanf(file,"f %i %i %i\n",&p1,&p2,&p3);
				pmesh->triangles[t*3]=p1-1;
				pmesh->triangles[t*3+1]=p2-1;
				pmesh->triangles[t*3+2]=p3-1;
			}
			return(my_mesh*)pmesh;
		fclose(file);
		}
	}
	MessageBoxA(NULL,"load mesh:could not open file or no navbuilder","error",MB_OK);
return NULL;
}

//SIMPLE PURE GEOMETRY FILE FORMAT, COULD BE IMPORTED AS A WAVEFRONT OBJ IN MODEL APPLICATIONS
DLLFUNC void save_mesh(void* mesh,char* filename)
{
	if(!m_navbuilder)MessageBoxA(NULL,"save mesh: no navbuilder","error",MB_OK);
	if(m_navbuilder)
	{
		FILE* file=fopen(filename,"w+ b");
		if(!file)MessageBoxA(NULL,"save mesh: could not open file for writing","error",MB_OK);
		if(file)
		{
			my_mesh* pmesh=(my_mesh*)mesh;
			fprintf(file,"# %i vertices\n",pmesh->v_count);
			fprintf(file,"# %i triangles\n",pmesh->t_count);
			for(int v=0;v<pmesh->v_count;v++)
			{
				fprintf(file,"v %f %f %f\n",pmesh->vertices[v*3],pmesh->vertices[v*3+1],pmesh->vertices[v*3+2]);
			}
			for(int t=0;t<pmesh->t_count;t++)
			{
				fprintf(file,"f %i %i %i\n",pmesh->triangles[t*3]+1,pmesh->triangles[t*3+1]+1,pmesh->triangles[t*3+2]+1);
			}
		fclose(file);
		}
	}
}

//SAVE THE MESH GEOMETRY and data OF THE BUILD LIST
/*
DLLFUNC void save_build_list(char* filename)
{
	if(!m_navbuilder)MessageBoxA(NULL,"save build list: no navbuilder","error",MB_OK);
	if(m_navbuilder)
	{
		m_navbuilder->save_final_meshes(filename);
	}
}
*/

DLLFUNC float* get_vbuffer(void* mesh)
{
	if(m_navbuilder)
	{
		my_mesh* pmesh=(my_mesh*)mesh;
		return pmesh->vertices;
	}
	MessageBoxA(NULL,"get buffer: no navbuilder","error",MB_OK);
return NULL;
}

DLLFUNC int* get_tbuffer(void* mesh)
{
	if(m_navbuilder)
	{
		my_mesh* pmesh=(my_mesh*)mesh;
		return pmesh->triangles;
	}
	MessageBoxA(NULL,"get buffer: no navbuilder","error",MB_OK);
return NULL;
}

DLLFUNC int get_tricount(void* mesh)
{
	if(m_navbuilder)
	{
		my_mesh* pmesh=(my_mesh*)mesh;
		return pmesh->t_count;
	}
	MessageBoxA(NULL,"get tri count: no navbuilder","error",MB_OK);
	return 0;
}

DLLFUNC int get_vertcount(void* mesh)
{
	if(m_navbuilder)
	{
		my_mesh* pmesh=(my_mesh*)mesh;
		return (int)pmesh->v_count;
	}
	MessageBoxA(NULL,"get vert count: no navbuilder","error",MB_OK);
	return 0;
}

//////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////
//************************************
// NAVMESH INFORMATION/HELPER FUNCTIONS
// DEBUG MESH FUNCTIONS
//

DLLFUNC float peragro_ceiling(float val)
{
	return ceilf(val);
}

DLLFUNC void peragro_calc_box(ENTITY *ent,int chunk,var *_min,var *_max)
{
	if(!m_navbuilder || !ent)MessageBoxA(NULL,"calc box: no navbuilder or ent","error",MB_OK);
	if(m_navbuilder && ent)m_navbuilder->calc_bbox(ent,chunk,_min,_max);
}

DLLFUNC VECTOR* peragro_nearest_pnt(VECTOR* from)
{
	if(!from)MessageBoxA(NULL,"near navpoint: invalid vector","error",MB_OK);
	if(!m_navbuilder)MessageBoxA(NULL,"near navpoint: no navbuilder","error",MB_OK);
	if(m_navbuilder && from)return(m_navbuilder->Nearest_Nav_Point(from));
	return NULL;
}

DLLFUNC VECTOR* peragro_rnd_pnts(int points)
{
	if(!points)MessageBoxA(NULL,"random point: no points specified","error",MB_OK);
	if(!m_navbuilder)MessageBoxA(NULL,"random point: no navbuilder","error",MB_OK);
	if(m_navbuilder && points)return(m_navbuilder->Nav_Point_Random(points));
	return NULL;
}

DLLFUNC VECTOR* peragro_rnd_pnt_radius(VECTOR* from,float radius)
{
	if(!from || !radius)MessageBoxA(NULL,"random point: invalid vector or no radius specified","error",MB_OK);
	if(!m_navbuilder)MessageBoxA(NULL,"random point: no navbuilder","error",MB_OK);
	if(m_navbuilder &&from && radius)return(m_navbuilder->Nav_Point_Random_Radius(from,radius));
	return NULL;
}

DLLFUNC int peragro_tot_tiles()
{
	if(m_navbuilder)return(m_navbuilder->m_navMesh->getMaxTiles());
	if(!m_navbuilder)MessageBoxA(NULL,"get total tiles: no navbuilder.","error",MB_OK);
	return 0;
}

DLLFUNC void peragro_tile_box(int tileid,var *_min,var *_max)
{
	if(!m_navbuilder || tileid<0)MessageBoxA(NULL,"get tile box:no navbuilder or invalid tile id","error",MB_OK);
	if(m_navbuilder && tileid>-1)
	{
		m_navbuilder->get_tile_box(tileid,_min,_max);
	}
}

DLLFUNC int peragro_in_tmp_list()
{
	m_link *cur;
	int total=0;
	for(cur=tmp_tiles_list->LHead;cur!=NULL;cur=cur->LNext)
	{
		tmp_id* m_id=(tmp_id*)cur->Data; 
		if(m_id)
		{
			total+=1;
		}
	}
	return total;
}

DLLFUNC void peragro_mark_box(VECTOR *bmin,VECTOR *bmax)
{
	if(!m_navbuilder || !bmin || !bmax )MessageBoxA(NULL,"mark tmp tile list:no navbuilder or invalid vector","error",MB_OK);
	if(m_navbuilder && bmin && bmax)
	{
		m_link *cur;

		for(cur=tmp_tiles_list->LHead;cur!=NULL;cur=cur->LNext)
		{
			tmp_id* m_id=(tmp_id*)cur->Data; 
			if(m_id)
			{
				tmp_tiles_list->LPos =cur;
				tmp_tiles_list->Data=NULL;
				tmp_tiles_list->Remove();
			}
		}
		int cur_index=0;
		m_link *cur1;
		for(cur1=tiledata_list->LHead;cur1!=NULL;cur1=cur1->LNext)
		{
			tmp_id* m_id=(tmp_id*)cur1->Data; 
			if(m_id)
			{
				if(m_id->_min[0]> bmin->x &&
				   m_id->_min[1]> bmin->y &&
				   m_id->_max[0]< bmax->x &&
				   m_id->_max[1]< bmax->y   )
				{
					m_id->index=cur_index;
					tmp_tiles_list->Add((tmp_id*)m_id);
					cur_index+=1;
				}
			}
		}
	}
}

DLLFUNC void peragro_world_box(var *_min ,var *_max)
{
	if(!m_navbuilder || !_min || !_max)MessageBoxA(NULL,"get world box:no navbuilder or invalid vector","error",MB_OK);
	if(m_navbuilder && _min && _max)
	{
		_min[0]=_VAR(m_navbuilder->m_meshBMin[0]);
		_min[1]=_VAR(m_navbuilder->m_meshBMin[2]);
		_min[2]=_VAR(m_navbuilder->m_meshBMin[1]);

		_max[0]=_VAR(m_navbuilder->m_meshBMax[0]);
		_max[1]=_VAR(m_navbuilder->m_meshBMax[2]);
		_max[2]=_VAR(m_navbuilder->m_meshBMax[1]);
	}
}

//////////////////////////////////
//store all tiledata in displaylist

DLLFUNC void peragro_init_tiledata(VECTOR *bmin,VECTOR *bmax)
{
	if(!m_navbuilder || !bmin || !bmax)MessageBoxA(NULL,"initialize tiledata list:no navbuilder or invalid vector","error",MB_OK);
	if(m_navbuilder && bmin && bmax)
	{
		//clear tiles in list
		m_link* cur;
		for(cur=tiledata_list->LHead;cur!=NULL;cur=cur->LNext)
		{
			tmp_id* m_id=(tmp_id*)cur->Data; 
			if(m_id)
			{
				tiledata_list->LPos =cur;
				tiledata_list->Data=NULL;
				tiledata_list->Remove();
				if(m_id->meshdata)
				{
					delete m_id->meshdata; 
					delete m_id;
				}
			}
		}
		m_navbuilder->init_display_list(bmin,bmax,tiledata_list);
	}
}

DLLFUNC void del_mesh(void* mesh)
{
	if(!m_navbuilder)MessageBoxA(NULL,"del mesh:no navbuilder","error",MB_OK);
	if(m_navbuilder)
	{
		my_mesh* pmesh=(my_mesh*)mesh;
		
		delete [] pmesh->triangles; 
		delete [] pmesh->vertices; 
		pmesh->triangles=0;
		pmesh->vertices=0;
		delete pmesh;
		pmesh=0;
	}
}

DLLFUNC void* mesh_from_tmp_list(int index,var *_min,var* _max,int *tileid)
{
	if(!m_navbuilder || index<0 || !_min || !_max )
	{
		MessageBoxA(NULL,"mesh from tmp list:no navbuilder or invalid input params","error",MB_OK);
		return NULL;
	}

	if(m_navbuilder && index>-1 && _min &&_max)
	{
		m_link *cur;
		for(cur=tmp_tiles_list->LHead;cur!=NULL;cur=cur->LNext)
		{
			tmp_id* m_id=(tmp_id*)cur->Data; 
			if(m_id)
			{
				if(index==m_id->index)
				{
					_min[0]=m_id->_min[0]; 
					_min[1]=m_id->_min[1]; 
					_min[2]=m_id->_min[2]; 
					_max[0]=m_id->_max[0]; 
					_max[1]=m_id->_max[1]; 
					_max[2]=m_id->_max[2];
					*tileid=m_id->tile_id; 
					return (my_mesh*)m_id->meshdata; 
				}
			}
		}
	}

	return NULL;
}

//MDL FORMAT WITH JUST GEOMETRY DATA
DLLFUNC void save_mdl(char* filename,void* mesh,int swapindices)
{
	if(!m_navbuilder)MessageBoxA(NULL,"save mdl: no navbuilder","error",MB_OK);
	if(m_navbuilder)m_navbuilder->save_mdl(filename,mesh,swapindices);
}

//////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////
//************************************
// WORKER THREAD/S STATUS (BUSY/DONE)
//
DLLFUNC int peragro_thread(void* thandle)
{
	if(m_navbuilder)
	{
		HANDLE myhandle=(HANDLE*)thandle;

		DWORD result=WaitForSingleObject(myhandle,0);
		if(result==WAIT_OBJECT_0)
		{
			//dead
			return 0;
		}
		else
		{
			return 1;
			//alive
		}
	}
	MessageBoxA(NULL,"thread alive?: no navbuilder","error",MB_OK);
	return 0;
}

//////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////
//************************************
//	TILE MANIPULATION
//

//	SINGLE BOX AREA THREAD DATA
typedef struct temp_box
{
	VECTOR* bmin;
	VECTOR* bmax;

}temp_box;
// MULTIPLE ENTITY AREA BOXES THREAD DATA
typedef struct temp_entity_array
{
	ENTITY* ent;
	int chunk;
}temp_entity_array;
// MULTIPLE POINT AREA THREAD DATA
typedef struct temp_points
{
	VECTOR* pnt;
	int id;
}temp_points;

DWORD WINAPI Thread_Add_Single_Tile(LPVOID param)
{
	if(m_navbuilder)
	{
		VECTOR* pos=(VECTOR*)param;
		m_navbuilder->build_tile(pos);
		return 1;
	}
	MessageBoxA(NULL,"add tile: no navbuilder","error",MB_OK);
	return 1;
}

//	TILE WORKER THREAD
DWORD WINAPI Thread_Add_Tiles_Box(LPVOID param)
{
	if(m_navbuilder)
	{
		temp_box* mydata=(temp_box*)param;
		m_navbuilder->build_tiles_box(mydata->bmin,mydata->bmax);
		return 1;
	}
	MessageBoxA(NULL,"add tiles: no navbuilder","error",MB_OK);
	return 1;
}

// WORKER THREAD FOR TILES FROM ENTITY BOX LIST
DWORD WINAPI Thread_Add_Tiles_Box_list(LPVOID param)
{
	if(m_navbuilder)
	{
		m_link* tlist=(m_link*)param;
		for(tlist=tlist->LHead;tlist!=NULL;tlist=tlist->LNext)
		{
			temp_entity_array* tdata=(temp_entity_array*)tlist->Data;
			var _min[3],_max[3];
			peragro_calc_box(tdata->ent,tdata->chunk,_min,_max);

			VECTOR* bmin=new VECTOR;
			VECTOR* bmax=new VECTOR;
			bmin->x=_min[0];
			bmin->y=_min[1];
			bmin->z=_min[2];

			bmax->x=_max[0];
			bmax->y=_max[1];
			bmax->z=_max[2];

			m_navbuilder->build_tiles_box(bmin,bmax);
		}
		return 1;
	}
	MessageBoxA(NULL,"add tiles box list: no navbuilder","error",MB_OK);
	return 1;
}

// WORKER THREAD FOR TILES FROM POINT LIST
DWORD WINAPI Thread_Add_Tiles_pnts_list(LPVOID param)
{
	if(m_navbuilder)
	{
		m_link* tlist=(m_link*)param;
		for(tlist=tlist->LHead;tlist!=NULL;tlist=tlist->LNext)
		{
			temp_points* tdata=(temp_points*)tlist->Data;
			if(m_navbuilder)m_navbuilder->build_tile(tdata->pnt);
		}
		return 1;
	}
	MessageBoxA(NULL,"add tiles pnt list: no navbuilder","error",MB_OK);
	return 1;
}

DWORD WINAPI Thread_Add_All_Tiles(LPVOID param)
{
	if(!m_navbuilder)MessageBoxA(NULL,"add all tiles: no navbuilder","error",MB_OK);
	if(m_navbuilder)m_navbuilder->build_all_tiles();
	return 1;
}

DWORD WINAPI Thread_Save_All_Tiles(LPVOID param)
{
	if(!m_navbuilder)MessageBoxA(NULL,"save all tiles: no navbuilder","error",MB_OK);
	if(m_navbuilder)m_navbuilder->save_all_tiles((char*)param);
	return 1;
}

DWORD WINAPI Thread_Load_All_Tiles(LPVOID param)
{
	if(!m_navbuilder)MessageBoxA(NULL,"load all tiles: no navbuilder","error",MB_OK);
	if(m_navbuilder)m_navbuilder->load_all_tiles((char*)param);
	return 1;
}

DLLFUNC int peragro_tile_id(VECTOR* pos)
{
	if(!pos)MessageBoxA(NULL,"get tile id: invalid vector","error",MB_OK);
	if(m_navbuilder)return(m_navbuilder->get_tileid(pos));
	if(!m_navbuilder && pos)MessageBoxA(NULL,"get tile id: no navbuilder","error",MB_OK);
	return 0;
}

DLLFUNC void peragro_load_tile(char* filename)
{
	if(!m_navbuilder)MessageBoxA(NULL,"load tile: no navbuilder","error",MB_OK);
	if(m_navbuilder)m_navbuilder->loadtile(filename);
}

DLLFUNC void peragro_save_tile(char* filename,int tileid)
{
	if(!m_navbuilder)MessageBoxA(NULL,"save tile: no navbuilder","error",MB_OK);
	if(m_navbuilder)m_navbuilder->save_tile(filename,tileid);
}

DLLFUNC void peragro_rem_tile_id(int _tileid)
{
	if(_tileid >-1 && m_navbuilder)
	{
		m_navbuilder->remove_tile(_tileid);
	}
}

DLLFUNC void peragro_rem_tile_pos(VECTOR* pos)
{
	if(!pos)MessageBoxA(NULL,"remove tile: invalid vector","error",MB_OK);
	if(!m_navbuilder)MessageBoxA(NULL,"remove tile: no navbuilder","error",MB_OK);
	if(m_navbuilder && pos)m_navbuilder->remove_tile(pos);
}

DLLFUNC void peragro_rem_tiles_box(VECTOR* bmin,VECTOR* bmax)
{
	if(!bmin || !bmax)MessageBoxA(NULL,"remove tiles box: invalid vector","error",MB_OK);
	if(!m_navbuilder)MessageBoxA(NULL,"remove tiles: no navbuilder","error",MB_OK);
	if(m_navbuilder &&bmin &&bmax)m_navbuilder->rem_tiles_box(bmin,bmax);
}

DLLFUNC void* peragro_add_tiles_box(VECTOR* bmin,VECTOR* bmax)
{
	if(!bmin || !bmax)MessageBoxA(NULL,"built tiles box: invalid vector","error",MB_OK);
	if(!m_navbuilder)MessageBoxA(NULL,"build tiles box: no navbuilder","error",MB_OK);
	if(m_navbuilder && bmin && bmax)
	{
		temp_box *mydata=new temp_box;

		mydata->bmax =new VECTOR;
		mydata->bmin =new VECTOR;
		
		mydata->bmax=bmax;
		mydata->bmin=bmin;

		m_worker=new workerthread();
		HANDLE hthread=m_worker->get_handle();
		m_worker->set_func(Thread_Add_Tiles_Box);
		m_worker->set_param((temp_box*)mydata);
		m_worker->signal_run();
		Sleep(3);

		delete(mydata);
		if(!hthread)
		{
			MessageBoxA(NULL,"built tiles box: thread failed","error",MB_OK);
			return NULL;
		}
		return (HANDLE*)hthread;
	}
	return NULL;
}

DLLFUNC void* peragro_add_tiles_ent_box(ENTITY *ent,int chunk)
{
	if(m_navbuilder && ent)
	{
		temp_box *mydata=new temp_box;
		var _min[3],_max[3];
		peragro_calc_box(ent,chunk,_min,_max);
		
		mydata->bmax =new VECTOR;
		mydata->bmin =new VECTOR;

		mydata->bmax->x=_max[0];
		mydata->bmax->y=_max[1];
		mydata->bmax->z=_max[2];

		mydata->bmin->x=_min[0];
		mydata->bmin->y=_min[1];
		mydata->bmin->z=_min[2];

		m_worker=new workerthread();
		HANDLE hthread=m_worker->get_handle();
		m_worker->set_func(Thread_Add_Tiles_Box);
		m_worker->set_param((temp_box*)mydata);
		m_worker->signal_run();
		Sleep(3);

		delete(mydata);
		if(!hthread)
		{
			MessageBoxA(NULL,"built tiles ent box: thread failed","error",MB_OK);
			return NULL;
		}
		return (HANDLE*)hthread;
	}
	MessageBoxA(NULL,"build tile ent box: no navbuilder or ent","error",MB_OK);
	return NULL;
}


// ENTITY BOX LIST
DLLFUNC void add_temp_entlist(ENTITY* ent,int chunk)
{
	if(!m_navbuilder || !ent)MessageBoxA(NULL,"add tmp list: no navbuilder or ent","error",MB_OK);
	if(m_navbuilder && ent)
	{
		temp_entity_array* mydata=new temp_entity_array();
		mydata->ent=ent;
		mydata->chunk=chunk;
		my_temp_ent_list->Add((temp_entity_array*)mydata);
	}
}

DLLFUNC void rem_temp_entlist(ENTITY* ent,int chunk)
{
	if(!m_navbuilder || !ent)MessageBoxA(NULL,"remove tmp list: no navbuilder or ent","error",MB_OK);

	if(m_navbuilder && ent)
	{	
		m_link *cur;
		for(cur=my_temp_ent_list->LHead;cur!=NULL;cur=cur->LNext)
		{
			temp_entity_array *mydata=(temp_entity_array*)cur->Data;
			if(mydata->ent==ent)
			{
				if(mydata->chunk==chunk && chunk>-1)
				{
					my_temp_ent_list->LPos =cur;
					my_temp_ent_list->Data=NULL;
					my_temp_ent_list->Remove();
					delete(mydata);
					mydata=NULL;
				}
				else if(chunk==-1)
				{
					my_temp_ent_list->LPos =cur;
					my_temp_ent_list->Data=NULL;
					my_temp_ent_list->Remove();
					delete(mydata);
					mydata=NULL;
				}
			}
		}
	}
}

DLLFUNC void clear_temp_entlist()
{
	if(!m_navbuilder)	MessageBoxA(NULL,"clear tmp list:no navbuilder","error",MB_OK);

	if(m_navbuilder)
	{
		m_link *cur;
		for(cur=my_temp_ent_list->LHead;cur!=NULL;cur=cur->LNext)
		{
			temp_entity_array *mydata=(temp_entity_array*)cur->Data;
			my_temp_ent_list->LPos =cur;
			my_temp_ent_list->Data=NULL;
			my_temp_ent_list->Remove();
			delete(mydata);
			mydata=NULL;
		}
	}
}

DLLFUNC void* peragro_add_tiles_box_entlist()
{
	if(m_navbuilder)
	{
		m_worker=new workerthread();
		HANDLE hthread=m_worker->get_handle();
		m_worker->set_func(Thread_Add_Tiles_Box_list);
		m_worker->set_param((m_link*)my_temp_ent_list);
		m_worker->signal_run();
		Sleep(3);

		if(!hthread)
		{
			MessageBoxA(NULL,"built tiles box ent: thread failed","error",MB_OK);
			return NULL;
		}
		return (HANDLE*)hthread;
	}
	MessageBoxA(NULL,"build tiles box ent list: no navbuilder","error",MB_OK);
	return NULL;
}

// POINTS LIST
DLLFUNC void add_temp_pntlist(VECTOR* pnt,int id)
{
	if(!m_navbuilder || !pnt)	MessageBoxA(NULL,"add tmp pnt list: no navbuilder or invalid vector","error",MB_OK);

	if(m_navbuilder && pnt)
	{
		temp_points* mydata=new temp_points();
		mydata->pnt=pnt;
		mydata->id=id;
		my_temp_pnt_list->Add((temp_points*)mydata);
	}
}

DLLFUNC void rem_temp_pntlist(VECTOR* pnt,int id)
{
	if(!m_navbuilder || !pnt)	MessageBoxA(NULL,"rem tmp pnt list: no navbuilder or invalid vector","error",MB_OK);

	if(m_navbuilder && pnt)
	{
		m_link *cur;
		for(cur=my_temp_pnt_list->LHead;cur!=NULL;cur=cur->LNext)
		{
			temp_points *mydata=(temp_points*)cur->Data;
			if(mydata->pnt==pnt)
			{
				if(mydata->id==id && id>-1)
				{
					my_temp_pnt_list->LPos =cur;
					my_temp_pnt_list->Data=NULL;
					my_temp_pnt_list->Remove();
					delete(mydata);
					mydata=NULL;
				}
				else if(id==-1)
				{
					my_temp_pnt_list->LPos =cur;
					my_temp_pnt_list->Data=NULL;
					my_temp_pnt_list->Remove();
					delete(mydata);
					mydata=NULL;
				}
			}
		}
	}
}

DLLFUNC void clear_temp_pntlist()
{
	if(!m_navbuilder)MessageBoxA(NULL,"clear tmp list: no navbuilder","error",MB_OK);

	if(m_navbuilder)
	{
		m_link *cur;
		for(cur=my_temp_pnt_list->LHead;cur!=NULL;cur=cur->LNext)
		{
			temp_points *mydata=(temp_points*)cur->Data;
			my_temp_pnt_list->LPos =cur;
			my_temp_pnt_list->Data=NULL;
			my_temp_pnt_list->Remove();
			delete(mydata);
			mydata=NULL;
		}
	}
}

DLLFUNC void* peragro_add_tiles_pntlist()
{
	if(m_navbuilder)
	{
		m_worker=new workerthread();
		HANDLE hthread=m_worker->get_handle();
		m_worker->set_func(Thread_Add_Tiles_pnts_list);
		m_worker->set_param((m_link*)my_temp_pnt_list);
		m_worker->signal_run();
		Sleep(3);

		if(!hthread)
		{
			MessageBoxA(NULL,"built tiles pntlist: thread failed","error",MB_OK);
			return NULL;
		}
		return (HANDLE*)hthread;
	}
	MessageBoxA(NULL,"build tiles pnt list: no navbuilder","error",MB_OK);
	return NULL;
}

////////////////////////////////////////////////////////////

DLLFUNC void peragro_rem_tiles_ent_box(ENTITY *ent,int chunk)
{
	if(!m_navbuilder || !ent)	MessageBoxA(NULL,"remove tiles ent box: no navbuilder or ent","error",MB_OK);

	if(m_navbuilder && ent)
	{
		var _min[3],_max[3];
		peragro_calc_box(ent,chunk,_min,_max);
		
		VECTOR* bmax =new VECTOR;
		VECTOR* bmin =new VECTOR;

		bmax->x=_max[0];
		bmax->y=_max[1];
		bmax->z=_max[2];

		bmin->x=_min[0];
		bmin->y=_min[1];
		bmin->z=_min[2];

		m_navbuilder->rem_tiles_box(bmin,bmax);
		delete(bmax);
		delete(bmin);
	}
}

DLLFUNC void* peragro_add_tile(VECTOR* pos)
{
	if(!m_navbuilder || !pos)MessageBoxA(NULL,"build tile: no navbuilder or invalid vector","error",MB_OK);
	if(m_navbuilder && pos)
	{
		m_worker=new workerthread();
		HANDLE hthread=m_worker->get_handle();
		m_worker->set_func(Thread_Add_Single_Tile);
		m_worker->set_param((VECTOR*)pos);
		m_worker->signal_run();
		Sleep(3);

		if(!hthread)
		{
			MessageBoxA(NULL,"built tile: thread failed","error",MB_OK);
			return NULL;
		}
		return (HANDLE*)hthread;
		
	}
	return NULL;
}

DLLFUNC void* peragro_add_all_tiles()
{
	if(!m_navbuilder)MessageBoxA(NULL,"build all tiles: no navbuilder","error",MB_OK);
	if(m_navbuilder)
	{
		m_worker=new workerthread();
		HANDLE hthread=m_worker->get_handle();
		m_worker->set_func(Thread_Add_All_Tiles);
		m_worker->set_param(NULL);
		m_worker->signal_run();
		Sleep(3);

		if(!hthread)
		{
			MessageBoxA(NULL,"built all tiles: thread failed","error",MB_OK);
			return NULL;
		}
		return (HANDLE*)hthread;
	}
	return NULL;
}

DLLFUNC void peragro_rem_all_tiles()
{
	if(!m_navbuilder)	MessageBoxA(NULL,"remove all tiles: no navbuilder","error",MB_OK);
	if(m_navbuilder)m_navbuilder->remove_all_tiles();
}

DLLFUNC void* peragro_save_all_tiles(char* filename)
{
	if(!m_navbuilder)MessageBoxA(NULL,"save tiles: no navbuilder","error",MB_OK);
	if(m_navbuilder)
	{
		m_worker=new workerthread();
		HANDLE hthread=m_worker->get_handle();
		m_worker->set_func(Thread_Save_All_Tiles);
		m_worker->set_param((char*)filename);
		m_worker->signal_run();
		Sleep(3);

		if(!hthread)
		{
			MessageBoxA(NULL,"save tiles: thread failed","error",MB_OK);
			return NULL;
		}
		return (HANDLE*)hthread;
	}
	return NULL;
//	if(!m_navbuilder)MessageBoxA(NULL,"save tiles: no navbuilder","error",MB_OK);
//	if(m_navbuilder)m_navbuilder->save_all_tiles(filename);
}

DLLFUNC void* peragro_load_all_tiles(char* filename)
{
	if(!m_navbuilder)MessageBoxA(NULL,"load tiles: no navbuilder","error",MB_OK);
	if(m_navbuilder)
	{
		m_worker=new workerthread();
		HANDLE hthread=m_worker->get_handle();
		m_worker->set_func(Thread_Load_All_Tiles);
		m_worker->set_param((char*)filename);
		m_worker->signal_run();
		Sleep(3);

		if(!hthread)
		{
			MessageBoxA(NULL,"load tiles: thread failed","error",MB_OK);
			return NULL;
		}
		return (HANDLE*)hthread;
	}
	return NULL;
//	if(!m_navbuilder)MessageBoxA(NULL,"load tile: no navbuilder","error",MB_OK);
//	if(m_navbuilder)m_navbuilder->load_all_tiles(filename);
}

DLLFUNC void peragro_update_bounds()
{
	if(!m_navbuilder)MessageBoxA(NULL,"update bounds: no navbuilder","error",MB_OK);
	if(m_navbuilder)m_navbuilder->finalize_meshes();
}

//////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////
//************************************
// STANDARD PATH-FINDING FUNCTIONS
//
DLLFUNC VECTOR* peragro_findpath(VECTOR *from,VECTOR* to,int *num_points)
{
	if(!from || !to)MessageBoxA(NULL,"find path: invalid vector","error",MB_OK);
	if(!m_navbuilder)MessageBoxA(NULL,"find path: no navbuilder","error",MB_OK);
	if(m_navbuilder && from && to)return(m_navbuilder->Find_Path(from,to,num_points));
	return NULL;
}

//////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////
//************************************
// CROWD/AGENT FUNCTIONS
//
