#include "navbuilder.h"
#include "save_mdl7.h"

InputGeom *m_geom;
InputGeom *m_tempgeom;
int tempvc,temptc;
dtNavMeshQuery* m_navQuery;

static const int NAVMESHSET_MAGIC = 'M'<<24 | 'S'<<16 | 'E'<<8 | 'T'; //'MSET';
static const int NAVMESHSET_VERSION = 1;

struct NavMeshSetHeader
{
	int magic;
	int version;
	int numTiles;
	dtNavMeshParams params;
};

struct NavMeshTileHeader
{
	dtTileRef tileRef;
	int dataSize;
};

inline unsigned int nextPow2(unsigned int v)
{
	v--;
	v |= v >> 1;
	v |= v >> 2;
	v |= v >> 4;
	v |= v >> 8;
	v |= v >> 16;
	v++;
	return v;
}

inline unsigned int ilog2(unsigned int v)
{
	unsigned int r;
	unsigned int shift;
	r = (v > 0xffff) << 4; v >>= r;
	shift = (v > 0xff) << 3; v >>= shift; r |= shift;
	shift = (v > 0xf) << 2; v >>= shift; r |= shift;
	shift = (v > 0x3) << 1; v >>= shift; r |= shift;
	r |= (v >> 1);
	return r;
}

navbuilder::navbuilder(void)
{
	nc=0;
	tc=0;
	vc=0;

	m_keepInterResults=false;
	m_buildAll=true;
	m_totalBuildTimeMs=0;
	m_triareas=0;
	m_solid=0;
	m_chf=0;
	m_cset=0;
	m_pmesh=0;
	m_dmesh=0;
	m_maxTiles=0;
	m_maxPolysPerTile=0;
	m_tileSize=32;
	m_tileBuildTime=0;
	m_tileMemUsage=0;
	m_tileTriCount=0;
	m_navMesh=0;
	m_navQuery=0;
	m_crowd=0;
	m_ctx=0;
	m_dmesh=0;

	resetCommonSettings();
	memset(m_tileBmin, 0, sizeof(m_tileBmin));
	memset(m_tileBmax, 0, sizeof(m_tileBmax));
	m_navQuery = dtAllocNavMeshQuery();
	m_crowd = dtAllocCrowd();
	m_ctx=new BuildContext();
	m_navQuery=0;
	m_navQuery = dtAllocNavMeshQuery();

	my_build_list =new m_link();
}

navbuilder::~navbuilder(void)
{
	my_build_list->~m_link();
	cleanup();
	dtFreeNavMeshQuery(m_navQuery);
	dtFreeNavMesh(m_navMesh);
	dtFreeCrowd(m_crowd);
	m_navMesh = 0;
}

void navbuilder::cleanup()
{
	m_triareas = 0;
	rcFreeHeightField(m_solid);
	m_solid = 0;
	rcFreeCompactHeightfield(m_chf);
	m_chf = 0;
	rcFreeContourSet(m_cset);
	m_cset = 0;
	rcFreePolyMesh(m_pmesh);
	m_pmesh = 0;
	rcFreePolyMeshDetail(m_dmesh);
	m_dmesh = 0;
}

void navbuilder::resetCommonSettings()
{
	m_cellSize = 0.3f;
	m_cellHeight = 0.2f;
	m_agentHeight = 2.0f;
	m_agentRadius = 0.6f;
	m_agentMaxClimb = 0.9f;
	m_agentMaxSlope = 45.0f;
	m_regionMinSize = 8;
	m_regionMergeSize = 20;
	m_monotonePartitioning = false;
	m_edgeMaxLen = 12.0f;
	m_edgeMaxError = 1.3f;
	m_vertsPerPoly = 6.0f;
	m_detailSampleDist = 6.0f;
	m_detailSampleMaxError = 1.0f;
}

void navbuilder::calc_bbox(ENTITY *ent,int chunk,var *_min,var *_max)
{
	m_link *cur;
	for(cur=my_build_list->LTail;cur!=NULL;cur=cur->LPrev)
	{
		my_mesh *pmesh=(my_mesh*)cur->Data;
		if(pmesh->ent==ent)
		{
			if(pmesh->chunk==chunk && chunk>-1)
			{
				const float* vertices=pmesh->vertices;
				float tbmin[3];
				float tbmax[3];
				rcCalcBounds(vertices,pmesh->v_count,tbmin,tbmax);
				_max[0]=_VAR(tbmax[0]); 
				_max[1]=_VAR(tbmax[2]); 
				_max[2]=_VAR(tbmax[1]); 

				_min[0]=_VAR(tbmin[0]); 
				_min[1]=_VAR(tbmin[2]); 
				_min[2]=_VAR(tbmin[1]); 
			}
		}
	}
}

void navbuilder::save_final_meshes(char* filename)
{
	vc=0;
	tc=0;
	int mesh_offset=0;
	int total_vertices=0;
	int total_triangles=0;
	int tot_meshes=0;

	m_link *cur;

	for(cur=my_build_list->LHead;cur!=NULL;cur=cur->LNext)
	{
		my_mesh *pmesh=(my_mesh*)cur->Data;
		total_vertices+=pmesh->v_count;
		total_triangles+=pmesh->t_count;
		tot_meshes+=1;
	}

	int finalmesh_vertex=0;
	int curver=0;
	float* mergedvertices=(float*)malloc(sizeof(float)*total_vertices*3);

	//save data

	FILE* file=fopen(filename,"w+ b");
	if(!file)MessageBoxA(NULL,"save mesh: could not open file for writing","error",MB_OK);
	if(file)
	{
		fprintf(file,"# %i vertices\n",total_vertices);
		fprintf(file,"# %i triangles\n",total_triangles);
		fprintf(file,"# %i meshes in list\n",tot_meshes);
		int curmesh=0;

		for(cur=my_build_list->LHead;cur!=NULL;cur=cur->LNext)
		{
			my_mesh *pmesh=(my_mesh*)cur->Data;
			fprintf(file,"\n");
			fprintf(file,"# ------------mesh%i------------\n",curmesh);
			//error here
			//if(pmesh->ent)fprintf(file,"# mesh type %i\n",ent_type(pmesh->ent));
			fprintf(file,"# filename %s\n",pmesh->ent->type);
			fprintf(file,"# chunk %i\n",pmesh->chunk);
			fprintf(file,"# triangles %i\n",pmesh->t_count);
			fprintf(file,"# vertices %i\n",pmesh->v_count);
			fprintf(file,"# pos x %f y %f z %f\n",_FLOAT(pmesh->ent->x),_FLOAT(pmesh->ent->y),_FLOAT(pmesh->ent->z));
			fprintf(file,"\n");
			//vertex/triangle data
			for(int v=0;v<pmesh->v_count;v++)
			{
				fprintf(file,"v %f %f %f\n",pmesh->vertices[v*3],pmesh->vertices[v*3+1],pmesh->vertices[v*3+2]);
			}
			for(int t=0;t<pmesh->t_count;t++)
			{
				fprintf(file,"f %i %i %i\n",pmesh->triangles[t*3],pmesh->triangles[t*3+1],pmesh->triangles[t*3+2]);
			}
			curmesh+=1;
		}

	fclose(file);
	}
}

void* navbuilder::finalize_meshes()
{
	vc=0;
	tc=0;
	int mesh_offset=0;
	int total_vertices=0;
	int total_triangles=0;

	m_link *cur;

	for(cur=my_build_list->LHead;cur!=NULL;cur=cur->LNext)
	{
		my_mesh *pmesh=(my_mesh*)cur->Data;
		total_vertices+=pmesh->v_count;
		total_triangles+=pmesh->t_count;
	}

	int finalmesh_vertex=0;
	int curver=0;
	float* mergedvertices=(float*)malloc(sizeof(float)*total_vertices*3);

	for(cur=my_build_list->LHead;cur!=NULL;cur=cur->LNext)
	{
		my_mesh *pmesh=(my_mesh*)cur->Data;
		for(int v=0;v<pmesh->v_count;v++)
		{
			finalmesh_vertex+=1;
			float x=pmesh->vertices[v*3];
			float y=pmesh->vertices[v*3+1];
			float z=pmesh->vertices[v*3+2];
			mergedvertices[curver*3]=x;
			mergedvertices[curver*3+1]=y;
			mergedvertices[curver*3+2]=z;
			curver+=1;
		}
		mesh_offset+=finalmesh_vertex;
	}
	rcCalcBounds(mergedvertices,total_vertices,m_meshBMin,m_meshBMax);
	return NULL;
}

int navbuilder::Sample_From_Custombuild(custombuild *step)
{
	if (step!=NULL)
	{
		m_cellHeight=(float)step->CellHeight;
		m_cellSize=(float)step->CellSize;
		m_detailSampleDist=(float)step->DetailSampleDist;
		m_detailSampleMaxError=(float)step->DetailSampleMaxError;
		m_edgeMaxLen=(float)step->EdgeMaxLen;
		m_edgeMaxError=(float)step->EdgeMaxError;
		m_vertsPerPoly=(float)step->VertsPerPoly;
		m_regionMergeSize=(float)step->RegionMergeSize;
		m_regionMinSize=(float)step->RegionMinSize;
		m_agentMaxClimb=(float)step->AgentMaxClimb;
		m_agentHeight=(float)step->AgentHeight;
		m_agentRadius=(float)step->AgentRadius;
		m_agentMaxSlope=(float)step->AgentMaxSlope;
		if(step->buildalltiles ==1)m_buildAll=true;
		if(step->buildalltiles ==0)m_buildAll=false;
		if(step->monotonePartitioning ==1)m_monotonePartitioning =true;
		if(step->monotonePartitioning ==0)m_monotonePartitioning =false;
		m_tileSize=step->TileSize;
	return 1;
	}
	return 0;
}


void navbuilder::buildAllTiles()
{
	if (!m_navMesh)
	{
		MessageBoxA(NULL,"buildalltiles: no navmesh","error",MB_OK);
		return;
	}	

	const float* bmin =m_meshBMin;
	const float* bmax =m_meshBMax;

	int gw = 0, gh = 0;
	rcCalcGridSize(bmin, bmax, m_cellSize, &gw, &gh);
	const int ts = (int)m_tileSize;
	const int tw = (gw + ts-1) / ts;
	const int th = (gh + ts-1) / ts;
	const float tcs = m_tileSize*m_cellSize;

	m_ctx->startTimer(RC_TIMER_TEMP);
	
	for (int y = 0; y < th; y++)
	{
		for (int x = 0; x < tw; x++)
		{
			m_tileBmin[0] = bmin[0]+ x*tcs;
			m_tileBmin[1] = bmin[1];
			m_tileBmin[2] = bmin[2]+ y*tcs;
			
			m_tileBmax[0] = bmin[0]+ (x+1)*tcs;
			m_tileBmax[1] = bmax[1];
			m_tileBmax[2] = bmin[2]+ (y+1)*tcs;

			int dataSize = 0;
			unsigned char* data = buildTileMesh(x, y,m_tileBmin,m_tileBmax, dataSize);
			if (data)
			{
				m_navMesh->removeTile(m_navMesh->getTileRefAt(x,y,0),0,0);
				dtStatus status = m_navMesh->addTile(data,dataSize,DT_TILE_FREE_DATA,0,0);
				if (dtStatusFailed(status))
					dtFree(data);
			}

		}
	}
	
	m_ctx->stopTimer(RC_TIMER_TEMP);

	m_totalBuildTimeMs = m_ctx->getAccumulatedTime(RC_TIMER_TEMP)/1000.0f;
}

unsigned char* navbuilder::buildTileMesh(const int tx, const int ty, const float* bmin, const float* bmax, int& dataSize)
{
	m_tileMemUsage = 0;
	m_tileBuildTime = 0;
	
	cleanup();

	int ntris=0;
	int nverts=0;
	m_link *cur;
	for(cur=my_build_list->LHead;cur!=NULL;cur=cur->LNext)
	{
		my_mesh *ppmesh=(my_mesh*)cur->Data;
		if(ppmesh!=NULL)
		{
			ntris+=ppmesh->t_count; 
			nverts+=ppmesh->v_count; 
		}
	}
		
	memset(&m_cfg, 0, sizeof(m_cfg));
	m_cfg.cs = m_cellSize;
	m_cfg.ch = m_cellHeight;
	m_cfg.walkableSlopeAngle = m_agentMaxSlope;
	m_cfg.walkableHeight = (int)ceilf(m_agentHeight / m_cfg.ch);
	m_cfg.walkableClimb = (int)floorf(m_agentMaxClimb / m_cfg.ch);
	m_cfg.walkableRadius = (int)ceilf(m_agentRadius / m_cfg.cs);
	m_cfg.maxEdgeLen = (int)(m_edgeMaxLen / m_cellSize);
	m_cfg.maxSimplificationError = m_edgeMaxError;
	m_cfg.minRegionArea = (int)rcSqr(m_regionMinSize);
	m_cfg.mergeRegionArea = (int)rcSqr(m_regionMergeSize);
	m_cfg.maxVertsPerPoly = (int)m_vertsPerPoly;
	m_cfg.tileSize = (int)m_tileSize;
	m_cfg.borderSize = m_cfg.walkableRadius + 3;
	m_cfg.width = m_cfg.tileSize + m_cfg.borderSize*2;
	m_cfg.height = m_cfg.tileSize + m_cfg.borderSize*2;
	m_cfg.detailSampleDist = m_detailSampleDist < 0.9f ? 0 : m_cellSize * m_detailSampleDist;
	m_cfg.detailSampleMaxError = m_cellHeight * m_detailSampleMaxError;
	
	rcVcopy(m_cfg.bmin, bmin);
	rcVcopy(m_cfg.bmax, bmax);
	m_cfg.bmin[0] -= m_cfg.borderSize*m_cfg.cs;
	m_cfg.bmin[2] -= m_cfg.borderSize*m_cfg.cs;
	m_cfg.bmax[0] += m_cfg.borderSize*m_cfg.cs;
	m_cfg.bmax[2] += m_cfg.borderSize*m_cfg.cs;
	
	m_ctx->resetTimers();
	m_ctx->startTimer(RC_TIMER_TOTAL);
	
	m_solid = rcAllocHeightfield();
	if (!m_solid)
	{
		MessageBoxA(NULL,"buildNavigation: Out of memory 'solid'.","error",MB_OK);
		return 0;
	}
	if (!rcCreateHeightfield(m_ctx, *m_solid, m_cfg.width, m_cfg.height, m_cfg.bmin, m_cfg.bmax, m_cfg.cs, m_cfg.ch))
	{
		MessageBoxA(NULL,"buildNavigation: Could not create solid heightfield.","error",MB_OK);
		return 0;
	}
	
	m_triareas = new unsigned char[ntris];
	if (!m_triareas)
	{
		MessageBoxA(NULL,"buildNavigation: Out of memory 'm_triareas'","error",MB_OK);
		return 0;
	}

	float tbmin[2], tbmax[2];
	tbmin[0] = m_cfg.bmin[0];
	tbmin[1] = m_cfg.bmin[2];
	tbmax[0] = m_cfg.bmax[0];
	tbmax[1] = m_cfg.bmax[2];

	int curtriangle=0;
	for(cur=my_build_list->LHead;cur!=NULL;cur=cur->LNext)
	{
		my_mesh *ppmesh=(my_mesh*)cur->Data;
		if(ppmesh!=NULL)
		{
			m_tileTriCount +=ppmesh->t_count;
			const float *vertices=ppmesh->vertices;
			int nverts=ppmesh->v_count;
			float boundmin[3];
			float boundmax[3];
			rcCalcBounds(vertices,nverts,boundmin,boundmax);

			if(m_cfg.bmax[0] < boundmin[0])continue;
			if(m_cfg.bmin[0] > boundmax[0])continue;

			if(m_cfg.bmax[1] < boundmin[1])continue;
			if(m_cfg.bmin[1] > boundmax[1])continue;

			if(m_cfg.bmax[2] < boundmin[2])continue;
			if(m_cfg.bmin[2] > boundmax[2])continue;

			for (int t=0;t<ppmesh->t_count;++t)
			{
				float n[3];
				
				const float* v0 = &ppmesh->vertices[ppmesh->triangles[t*3]*3];
				const float* v1 = &ppmesh->vertices[ppmesh->triangles[t*3+1]*3];
				const float* v2 = &ppmesh->vertices[ppmesh->triangles[t*3+2]*3];

				unsigned char areaId;
   
				float e0[3], e1[3]; 
				for (int j = 0; j < 3; ++j) 
				{ 
					e0[j] = v1[j] - v0[j]; 
					e1[j] = v2[j] - v0[j]; 
				} 

				n[0] = e0[1]*e1[2] - e0[2]*e1[1]; 
				n[1] = e0[2]*e1[0] - e0[0]*e1[2]; 
				n[2] = e0[0]*e1[1] - e0[1]*e1[0]; 

				float d = sqrtf(n[0]*n[0] + n[1]*n[1] + n[2]*n[2]); 
				if (d > 0) 
				{ 
					d = 1.0f/d; 
					n[0] *= d; 
					n[1] *= d; 
					n[2] *= d; 
				} 
				if (acos(n[1]) < m_cfg.walkableSlopeAngle)
					 areaId=RC_WALKABLE_AREA;
				else
					areaId=RC_NULL_AREA;
					rcRasterizeTriangle(m_ctx,v0,v1,v2,areaId,*m_solid);
			
			}
		}
	}

	if (!m_keepInterResults)
	{
		delete [] m_triareas;
		m_triareas = 0;
	}
	
	rcFilterLowHangingWalkableObstacles(m_ctx, m_cfg.walkableClimb, *m_solid);
	rcFilterLedgeSpans(m_ctx, m_cfg.walkableHeight, m_cfg.walkableClimb, *m_solid);
	rcFilterWalkableLowHeightSpans(m_ctx, m_cfg.walkableHeight, *m_solid);
	
	m_chf = rcAllocCompactHeightfield();
	if (!m_chf)
	{
		MessageBoxA(NULL,"buildNavigation:  Out of memory 'chf'.","error",MB_OK);
		return 0;
	}

	if (!rcBuildCompactHeightfield(m_ctx, m_cfg.walkableHeight, m_cfg.walkableClimb, *m_solid, *m_chf))
	{
		MessageBoxA(NULL,"buildNavigation: Could not build compact data.","error",MB_OK);
		return 0;
	}
	
	if (!m_keepInterResults)
	{
		rcFreeHeightField(m_solid);
		m_solid = 0;
	}

	if (!rcErodeWalkableArea(m_ctx, m_cfg.walkableRadius, *m_chf))
	{
		MessageBoxA(NULL,"buildNavigation: Could not erode.","error",MB_OK);
		return 0;
	}

	if (m_monotonePartitioning)
	{
		if (!rcBuildRegionsMonotone(m_ctx, *m_chf, m_cfg.borderSize, m_cfg.minRegionArea, m_cfg.mergeRegionArea))
		{
			MessageBoxA(NULL,"buildNavigation:  Could not build regions.","error",MB_OK);
			return 0;
		}
	}
	else
	{
		if (!rcBuildDistanceField(m_ctx, *m_chf))
		{
			MessageBoxA(NULL,"buildNavigation: Could not build distance field.","error",MB_OK);
			return 0;
		}
		
		if (!rcBuildRegions(m_ctx, *m_chf, m_cfg.borderSize, m_cfg.minRegionArea, m_cfg.mergeRegionArea))
		{
			MessageBoxA(NULL,"buildNavigation: Could not build regions.","error",MB_OK);
			return 0;
		}
	}
 	
	m_cset = rcAllocContourSet();
	if (!m_cset)
	{
		MessageBoxA(NULL,"buildNavigation: Out of memory 'cset'.","error",MB_OK);
		return 0;
	}

	if (!rcBuildContours(m_ctx, *m_chf, m_cfg.maxSimplificationError, m_cfg.maxEdgeLen, *m_cset))
	{
		MessageBoxA(NULL,"buildNavigation: Could not create contours.","error",MB_OK);
		return 0;
	}
	
	if (m_cset->nconts == 0)
	{
		//MessageBoxA(NULL,"buildNavigation: nconts=0 try a smaller cellsize","error",MB_OK);
		return 0;
	}
	
	m_pmesh = rcAllocPolyMesh();
	if (!m_pmesh)
	{
		MessageBoxA(NULL,"buildNavigation: Out of memory 'pmesh'.","error",MB_OK);
		return 0;
	}
	if (!rcBuildPolyMesh(m_ctx, *m_cset, m_cfg.maxVertsPerPoly, *m_pmesh))
	{
		MessageBoxA(NULL,"buildNavigation: Could not triangulate contours.","error",MB_OK);
		return 0;
	}
	
	m_dmesh = rcAllocPolyMeshDetail();
	if (!m_dmesh)
	{
		MessageBoxA(NULL,"buildNavigation:  Out of memory 'dmesh'.","error",MB_OK);
		return 0;
	}
	
	if (!rcBuildPolyMeshDetail(m_ctx, *m_pmesh, *m_chf,
							   m_cfg.detailSampleDist, m_cfg.detailSampleMaxError,
							   *m_dmesh))
	{
		MessageBoxA(NULL,"buildNavigation: Could build polymesh detail.","error",MB_OK);
		return 0;
	}
	
	if (!m_keepInterResults)
	{
		rcFreeCompactHeightfield(m_chf);
		m_chf = 0;
		rcFreeContourSet(m_cset);
		m_cset = 0;
	}
	

	unsigned char* navData = 0;
	int navDataSize = 0;
	if (m_cfg.maxVertsPerPoly <= DT_VERTS_PER_POLYGON)
	{
		if (m_pmesh->nverts >= 0xffff)
		{
			MessageBoxA(NULL,"buildNavigation: Too many vertices per tile.","error",MB_OK);
			// The vertex indices are ushorts, and cannot point to more than 0xffff vertices.
			return 0;
		}
		
		for (int i = 0; i < m_pmesh->npolys; ++i)
		{
			if (m_pmesh->areas[i] == RC_WALKABLE_AREA)
				m_pmesh->areas[i] = SAMPLE_POLYAREA_GROUND;
			
			if (m_pmesh->areas[i] == SAMPLE_POLYAREA_GROUND ||
				m_pmesh->areas[i] == SAMPLE_POLYAREA_GRASS ||
				m_pmesh->areas[i] == SAMPLE_POLYAREA_ROAD)
			{
				m_pmesh->flags[i] = SAMPLE_POLYFLAGS_WALK;
			}
			else if (m_pmesh->areas[i] == SAMPLE_POLYAREA_WATER)
			{
				m_pmesh->flags[i] = SAMPLE_POLYFLAGS_SWIM;
			}
			else if (m_pmesh->areas[i] == SAMPLE_POLYAREA_DOOR)
			{
				m_pmesh->flags[i] = SAMPLE_POLYFLAGS_WALK | SAMPLE_POLYFLAGS_DOOR;
			}
		}
		
		dtNavMeshCreateParams params;
		memset(&params, 0, sizeof(params));
		params.verts = m_pmesh->verts;
		params.vertCount = m_pmesh->nverts;
		params.polys = m_pmesh->polys;
		params.polyAreas = m_pmesh->areas;
		params.polyFlags = m_pmesh->flags;
		params.polyCount = m_pmesh->npolys;
		params.nvp = m_pmesh->nvp;
		params.detailMeshes = m_dmesh->meshes;
		params.detailVerts = m_dmesh->verts;
		params.detailVertsCount = m_dmesh->nverts;
		params.detailTris = m_dmesh->tris;
		params.detailTriCount = m_dmesh->ntris;
		
		params.offMeshConVerts =0;
		params.offMeshConRad =0;
		params.offMeshConDir =0;
		params.offMeshConAreas =0;
		params.offMeshConFlags =0;
		params.offMeshConUserID =0;
		params.offMeshConCount =0;

		params.walkableHeight = m_agentHeight;
		params.walkableRadius = m_agentRadius;
		params.walkableClimb = m_agentMaxClimb;
		params.tileX = tx;
		params.tileY = ty;
		params.tileLayer = 0;
		rcVcopy(params.bmin, m_pmesh->bmin);
		rcVcopy(params.bmax, m_pmesh->bmax);
		params.cs = m_cfg.cs;
		params.ch = m_cfg.ch;
		params.buildBvTree = true;
		
		if (!dtCreateNavMeshData(&params, &navData, &navDataSize))
		{
			MessageBoxA(NULL,"buildNavigation:Could not build Detour navmesh.\n","error",MB_OK);
			return 0;
		}		
	}
	m_tileMemUsage = navDataSize/1024.0f;
	
	m_ctx->stopTimer(RC_TIMER_TOTAL);
	
	m_tileBuildTime = m_ctx->getAccumulatedTime(RC_TIMER_TOTAL)/1000.0f;
	dataSize = navDataSize;

	return navData;
}

int navbuilder::Nav_From_Custombuild(custombuild* step)
{
	Sample_From_Custombuild(step);

	finalize_meshes();  
	
	const float* bmin =m_meshBMin;
	const float* bmax =m_meshBMax;

		int gw = 0, gh = 0;
		rcCalcGridSize(bmin, bmax, m_cellSize, &gw, &gh);
		const int ts = (int)m_tileSize;

		const int tw = (gw + ts-1) / ts;
		const int th = (gh + ts-1) / ts;

		int tileBits = rcMin((int)ilog2(nextPow2(tw*th)), 14);
		if (tileBits > 14) tileBits = 14;
		int polyBits = 22 - tileBits;
		m_maxTiles = 1 << tileBits;
		m_maxPolysPerTile = 1 << polyBits;
		dtFreeNavMesh(m_navMesh);
		
		m_navMesh = dtAllocNavMesh();
		if (!m_navMesh)
		{
			MessageBoxA(NULL,"buildTiledNavigation: Could not allocate navmesh.","error",MB_OK);
			return 0;
		}

		dtNavMeshParams params;
		rcVcopy(params.orig,m_meshBMin);
		
		params.tileWidth = m_tileSize*m_cellSize;
		params.tileHeight = m_tileSize*m_cellSize;
		params.maxTiles =m_maxTiles;
		params.maxPolys =m_maxPolysPerTile;
		
		dtStatus status;
		
		status = m_navMesh->init(&params);
		if (dtStatusFailed(status))
		{
			MessageBoxA(NULL,"buildTiledNavigation: Could not init navmesh.","error",MB_OK);
			return 0;
		}
		
		status = m_navQuery->init(m_navMesh, 2048);
		if (dtStatusFailed(status))
		{
			MessageBoxA(NULL,"buildTiledNavigation: Could not init Detour navmesh query","error",MB_OK);
			return 0;
		}
		
		if (m_buildAll)
			buildAllTiles();
	
	return 1;
}

int navbuilder::Get_Poly_Count_tile(int tileid)
{
	int poly_count=0;
	const dtMeshTile* tile =m_navMesh->getTile(tileid);
	for (int pc = 0; pc < tile->header->polyCount; pc++) poly_count+=1;
	return (poly_count);
}

my_polygon* navbuilder::Get_Polygons_tile(int tileid)
{
	int polycount=0;
	const dtMeshTile* tile = m_navMesh->getTile(tileid);
	dtPolyRef base = m_navMesh->getPolyRefBase(tile);
	int tileNum = m_navMesh->decodePolyIdTile(base);

	for (int pc = 0; pc < tile->header->polyCount; ++pc) polycount+=1;

	my_polygon *polygons=new my_polygon[polycount];
	int curpoly=0;
			
	for (int i1 = 0; i1 < tile->header->polyCount; ++i1)
	{
		const dtPoly* p = &tile->polys[i1];
		if (p->getType() == DT_POLYTYPE_OFFMESH_CONNECTION)
		continue;
		
		const dtPolyDetail* pd = &tile->detailMeshes[i1];
		unsigned char ntris=pd->triCount;
		polygons[curpoly].triangles=new my_triangle[ntris];
		polygons[curpoly].num_triangles=ntris; 
		for (int j = 0; j < pd->triCount; ++j)
		{
			const unsigned char* t = &tile->detailTris[(pd->triBase+j)*4];
			for (int k = 0; k < 3; ++k)
			{
				float* v;
				if (t[k] < p->vertCount)
					v=&tile->verts[p->verts[t[k]]*3];
				else
					v=&tile->detailVerts[(pd->vertBase+t[k]-p->vertCount)*3];

				if(k==0)
				{
					polygons[curpoly].triangles[j].vertex1.x=_VAR(v[0]);
					polygons[curpoly].triangles[j].vertex1.y=_VAR(v[2]);
					polygons[curpoly].triangles[j].vertex1.z=_VAR(v[1]);
				}
				if(k==1)
				{
					polygons[curpoly].triangles[j].vertex2.x=_VAR(v[0]);
					polygons[curpoly].triangles[j].vertex2.y=_VAR(v[2]);
					polygons[curpoly].triangles[j].vertex2.z=_VAR(v[1]);
				}
				if(k==2)
				{
					polygons[curpoly].triangles[j].vertex3.x=_VAR(v[0]);
					polygons[curpoly].triangles[j].vertex3.y=_VAR(v[2]);
					polygons[curpoly].triangles[j].vertex3.z=_VAR(v[1]);
				}
			}	
		}
	curpoly+=1;

	}
return(polygons);
}

void navbuilder::save_mdl(char* filename,void* mesh,int swapindices)
{
	my_mesh* pmesh=(my_mesh*)mesh;

	int numtriangles=pmesh->t_count;
	int numvertices=pmesh->v_count;

	mdl7data *my_mdl=new mdl7data();
	my_mdl->bones=0;
	my_mdl->groups=1;
	my_mdl->skins=0;
	my_mdl->group_list=new cGroup();
	my_mdl->group_list->faceslist=(cFace*)malloc(sizeof(cFace)*numtriangles);
	my_mdl->group_list->vertexlist=(cVertex*)malloc(sizeof(cVertex)*numvertices);
	my_mdl->group_list->lodfacelist=NULL;
	my_mdl->group_list->lodfaces=0; 

	my_mdl->group_list->vertices=numvertices;
	my_mdl->group_list->faces=numtriangles;

	for(int v=0;v<numvertices;v++)
	{
		my_mdl->group_list->vertexlist[v].x=pmesh->vertices[v*3];
		my_mdl->group_list->vertexlist[v].y=pmesh->vertices[v*3+1];
		my_mdl->group_list->vertexlist[v].z=pmesh->vertices[v*3+2];
	}
	for(int t=0;t<numtriangles;t++)
	{
		my_mdl->group_list->faceslist[t].p1=pmesh->triangles[t*3];
		if(swapindices)
		{
			my_mdl->group_list->faceslist[t].p2=pmesh->triangles[t*3+2];
			my_mdl->group_list->faceslist[t].p3=pmesh->triangles[t*3+1];
		}
		else
		{
			my_mdl->group_list->faceslist[t].p2=pmesh->triangles[t*3+1];
			my_mdl->group_list->faceslist[t].p3=pmesh->triangles[t*3+2];
		}
	}
save_mdl7 (filename, *my_mdl);
}

void navbuilder::remove_tile(int _tileid)
{
	const dtMeshTile *tile=m_navMesh->getTile(_tileid);
	if (!tile || !tile->header || !tile->dataSize) return;
	else
	{
		m_navMesh->removeTile(m_navMesh->getTileRef(tile),0,0);
	}
}

void navbuilder::removeAllTiles()
{
	const float* bmin =m_meshBMin;
	const float* bmax =m_meshBMax;

	int gw = 0, gh = 0;
	rcCalcGridSize(bmin, bmax, m_cellSize, &gw, &gh);
	const int ts = (int)m_tileSize;
	const int tw = (gw + ts-1) / ts;
	const int th = (gh + ts-1) / ts;
	
	for (int y = 0; y < th; ++y)
	for (int x = 0; x < tw; ++x)
	m_navMesh->removeTile(m_navMesh->getTileRefAt(x,y,0),0,0);
}

void navbuilder::remove_tile(VECTOR* pos)
{
/*	float target[3];
	target[0]=_FLOAT(pos->x); 
	target[1]=_FLOAT(pos->z); 
	target[2]=_FLOAT(pos->y); 
*/
	float fromrays[3], fromraye[3]; 
	float torays[3], toraye[3]; 
	
	//path start
	fromrays[0]=_FLOAT(pos->x);
	fromrays[1]=_FLOAT(pos->z);
	fromrays[2]=_FLOAT(pos->y);
	
	fromraye[0]=_FLOAT(pos->x);
	fromraye[1]=-1;
	fromraye[2]=_FLOAT(pos->y);

	float hitt1=0;
	bool hit1 = raycastMesh(fromrays, fromraye, hitt1);
	float start_pos[3];
		
		start_pos[0] = fromrays[0] + (fromraye[0] - fromrays[0])*hitt1;
		start_pos[1] = fromrays[1] + (fromraye[1] - fromrays[1])*hitt1;
		start_pos[2] = fromrays[2] + (fromraye[2] - fromrays[2])*hitt1;

	if (!m_navMesh) return;
	
	const float* bmin = m_meshBMin; 
	const float* bmax = m_meshBMax; 

	const float ts = m_tileSize*m_cellSize;
	const int tx = (int)((start_pos[0] - bmin[0]) / ts);
	const int ty = (int)((start_pos[2] - bmin[2]) / ts);
	
	m_tileBmin[0] = bmin[0] + tx*ts;
	m_tileBmin[1] = bmin[1];
	m_tileBmin[2] = bmin[2] + ty*ts;
	
	m_tileBmax[0] = bmin[0] + (tx+1)*ts;
	m_tileBmax[1] = bmax[1];
	m_tileBmax[2] = bmin[2] + (ty+1)*ts;
	
	m_tileCol = duRGBA(128,32,16,64);
	
	m_navMesh->removeTile(m_navMesh->getTileRefAt(tx,ty,0),0,0);
}

void navbuilder::rem_tiles_box(VECTOR* bmin,VECTOR* bmax)
{
	const float* wbmin = m_meshBMin;
	const float* wbmax = m_meshBMax;

	const float ts = m_tileSize*m_cellSize;

	const int tx1 = (int)((_FLOAT(bmin->x) - wbmin[0]) / ts);
	const int ty1 = (int)((_FLOAT(bmin->y) - wbmin[2]) / ts);
	const int tx2 = (int)((_FLOAT(bmax->x) - wbmin[0]) / ts);
	const int ty2 = (int)((_FLOAT(bmax->y) - wbmin[2]) / ts);

	for (int y = ty1-1; y <ty2+2; ++y)
		for (int x =tx1-1; x < tx2+2; ++x)
			m_navMesh->removeTile(m_navMesh->getTileRefAt(x,y,0),0,0);
}


int navbuilder::build_tiles_box(VECTOR* bmin,VECTOR* bmax)
{
	finalize_meshes();

	const float* wbmin = m_meshBMin;
	const float* wbmax = m_meshBMax;
	
	const float ts = m_tileSize*m_cellSize;

	const int tx1 = (int)((_FLOAT(bmin->x) - wbmin[0]) / ts);
	const int ty1 = (int)((_FLOAT(bmin->y) - wbmin[2]) / ts);
	const int tx2 = (int)((_FLOAT(bmax->x) - wbmin[0]) / ts);
	const int ty2 = (int)((_FLOAT(bmax->y) - wbmin[2]) / ts);


	int dataSize = 0;

for (int y = ty1-1; y <ty2+2; ++y)
{
for (int x =tx1-1; x < tx2+2; ++x)
{
	m_tileBmin[0] = wbmin[0] + (x)*ts;
	m_tileBmin[1] = wbmin[1];
	m_tileBmin[2] = wbmin[2] + (y)*ts;
	
	m_tileBmax[0] = wbmin[0] + (x+1)*ts;
	m_tileBmax[1] = wbmax[1];
	m_tileBmax[2] = wbmin[2] + (y+1)*ts;
	

	unsigned char* data = buildTileMesh(x,y, m_tileBmin, m_tileBmax, dataSize);
	
	if (data)
	{
		m_navMesh->removeTile(m_navMesh->getTileRefAt(x,y,0),0,0);
		
		dtStatus status = m_navMesh->addTile(data,dataSize,DT_TILE_FREE_DATA,0,0);
		if (dtStatusFailed(status))
			dtFree(data);
	}
}
}
return 1;
}

int navbuilder::build_tile(VECTOR* pos)
{
	finalize_meshes();

/*	float target[3];
	target[0]=_FLOAT(pos->x); 
	target[1]=_FLOAT(pos->z); 
	target[2]=_FLOAT(pos->y); 
*/
	float fromrays[3], fromraye[3]; 
	float torays[3], toraye[3]; 
	
	//path start
	fromrays[0]=_FLOAT(pos->x);
	fromrays[1]=_FLOAT(pos->z);
	fromrays[2]=_FLOAT(pos->y);
	
	fromraye[0]=_FLOAT(pos->x);
	fromraye[1]=-1;
	fromraye[2]=_FLOAT(pos->y);

	float hitt1=0;
	bool hit1 = raycastMesh(fromrays, fromraye, hitt1);
	float start_pos[3];
		
		start_pos[0] = fromrays[0] + (fromraye[0] - fromrays[0])*hitt1;
		start_pos[1] = fromrays[1] + (fromraye[1] - fromrays[1])*hitt1;
		start_pos[2] = fromrays[2] + (fromraye[2] - fromrays[2])*hitt1;

	if (!m_navMesh) return 0;
		
	const float* bmin =m_meshBMin; 
	const float* bmax =m_meshBMax; 
	
	const float ts = m_tileSize*m_cellSize;
	const int tx = (int)((start_pos[0] - bmin[0]) / ts);
	const int ty = (int)((start_pos[2] - bmin[2]) / ts);
	
	m_tileBmin[0] = bmin[0] + tx*ts;
	m_tileBmin[1] = bmin[1];
	m_tileBmin[2] = bmin[2] + ty*ts;
	
	m_tileBmax[0] = bmin[0] + (tx+1)*ts;
	m_tileBmax[1] = bmax[1];
	m_tileBmax[2] = bmin[2] + (ty+1)*ts;
	
	m_tileCol = duRGBA(255,255,255,64);
	
	int dataSize = 0;
	unsigned char* data = buildTileMesh(tx, ty, m_tileBmin, m_tileBmax, dataSize);
	
	if (data)
	{
		m_navMesh->removeTile(m_navMesh->getTileRefAt(tx,ty,0),0,0);
		
		dtStatus status = m_navMesh->addTile(data,dataSize,DT_TILE_FREE_DATA,0,0);
		if (dtStatusFailed(status))
			dtFree(data);
		return 1;
	}
	return 0;
}

void navbuilder::build_all_tiles()
{
	finalize_meshes();
	if (!m_navMesh) return;
	
	const float* bmin = m_meshBMin; 
	const float* bmax = m_meshBMax; 

	int gw = 0, gh = 0;
	rcCalcGridSize(bmin, bmax, m_cellSize, &gw, &gh);
	const int ts = (int)m_tileSize;
	const int tw = (gw + ts-1) / ts;
	const int th = (gh + ts-1) / ts;
	const float tcs = m_tileSize*m_cellSize;

	
	m_ctx->startTimer(RC_TIMER_TEMP);

	for (int y = 0; y < th; ++y)
	{
		for (int x = 0; x < tw; ++x)
		{
			m_tileBmin[0] = bmin[0]+ x*tcs;
			m_tileBmin[1] = bmin[1];
			m_tileBmin[2] = bmin[2]+ y*tcs;
			
			m_tileBmax[0] = bmin[0]+ (x+1)*tcs;
			m_tileBmax[1] = bmax[1];
			m_tileBmax[2] = bmin[2]+ (y+1)*tcs;
			int dataSize = 0;
			unsigned char* data = buildTileMesh(x, y,m_tileBmin,m_tileBmax, dataSize);
			if (data)
			{
				m_navMesh->removeTile(m_navMesh->getTileRefAt(x,y,0),0,0);
				dtStatus status = m_navMesh->addTile(data,dataSize,DT_TILE_FREE_DATA,0,0);
				if (dtStatusFailed(status))
					dtFree(data);
			}
		}
	}
	
	m_ctx->stopTimer(RC_TIMER_TEMP);

	m_totalBuildTimeMs = m_ctx->getAccumulatedTime(RC_TIMER_TEMP)/1000.0f;
	
}

void navbuilder::remove_all_tiles()
{
	const float* bmin = m_meshBMin;
	const float* bmax = m_meshBMax;

	int gw = 0, gh = 0;
	rcCalcGridSize(bmin, bmax, m_cellSize, &gw, &gh);
	const int ts = (int)m_tileSize;
	const int tw = (gw + ts-1) / ts;
	const int th = (gh + ts-1) / ts;
	
	for (int y = 0; y < th; ++y)
		for (int x = 0; x < tw; ++x)
			m_navMesh->removeTile(m_navMesh->getTileRefAt(x,y,0),0,0);
}

void navbuilder::save_tile(char* file,int tileid)
{
	if (!m_navMesh) return;
	
	FILE* fp = fopen(file, "wb");
	if (!fp)
		return;
	
	NavMeshSetHeader header;
	header.magic = NAVMESHSET_MAGIC;
	header.version = NAVMESHSET_VERSION;
	header.numTiles = 0;

	const dtMeshTile* tile = m_navMesh->getTile(tileid);
	//if (!tile || !tile->header || !tile->dataSize) break;
	header.numTiles+=1;

	memcpy(&header.params, m_navMesh->getParams(), sizeof(dtNavMeshParams));
	fwrite(&header, sizeof(NavMeshSetHeader), 1, fp);

	NavMeshTileHeader tileHeader;
	tileHeader.tileRef = m_navMesh->getTileRef(tile);
	tileHeader.dataSize = tile->dataSize;
	fwrite(&tileHeader, sizeof(tileHeader), 1, fp);

	fwrite(tile->data, tile->dataSize, 1, fp);

	fclose(fp);
}

int navbuilder::loadtile(const char* path)
{
	FILE* fp = fopen(path, "rb");
	if (!fp) 
	{
		MessageBoxA(NULL,"load tile:could not load file","error",MB_OK);
		return 0;
	}
	
	NavMeshSetHeader header;
	fread(&header, sizeof(NavMeshSetHeader), 1, fp);
	if (header.magic != NAVMESHSET_MAGIC)
	{
		MessageBoxA(NULL,"load tile:could not load file","error",MB_OK);
		fclose(fp);
		return 0;
	}
	if (header.version != NAVMESHSET_VERSION)
	{
		MessageBoxA(NULL,"load tile:could not load file","error",MB_OK);
		fclose(fp);
		return 0;
	}
	
	for (int i = 0; i < header.numTiles; ++i)
	{
		NavMeshTileHeader tileHeader;
		fread(&tileHeader, sizeof(tileHeader), 1, fp);
		if (!tileHeader.tileRef || !tileHeader.dataSize)
			break;

		unsigned char* data = (unsigned char*)dtAlloc(tileHeader.dataSize, DT_ALLOC_PERM);
		if (!data) break;
		memset(data, 0, tileHeader.dataSize);
		fread(data, tileHeader.dataSize, 1, fp);
		
		m_navMesh->addTile(data, tileHeader.dataSize, DT_TILE_FREE_DATA, tileHeader.tileRef, 0);
	}
	
	fclose(fp);
	return 1;
}

void navbuilder::get_tile_box(int tileid,var *_min,var *_max)
{
	const dtMeshTile* tile = m_navMesh->getTile(tileid);
	if (!tile || !tile->header || !tile->dataSize) 
	{
		_min[0]=0;
		_min[1]=0;
		_min[2]=0;
		_max[0]=0;
		_max[1]=0;
		_max[2]=0;
	}
	else
	{
		_min[0]=_VAR(tile->header->bmin[0]);
		_min[1]=_VAR(tile->header->bmin[2]);
		_min[2]=_VAR(tile->header->bmin[1]);
		_max[0]=_VAR(tile->header->bmax[0]);
		_max[1]=_VAR(tile->header->bmax[2]);
		_max[2]=_VAR(tile->header->bmax[1]);
	}
}

void* navbuilder::mesh_from_tile(my_mesh* pmesh,int tileid)
{
	int total_tiles=0;
	int cur_tile=tileid;

	const dtMeshTile* tile = m_navMesh->getTile(cur_tile);
	if (!tile || !tile->header || !tile->dataSize) return NULL;
		
	int numpolys=Get_Poly_Count_tile(cur_tile) ;
	my_polygon* polygons=Get_Polygons_tile(cur_tile);
	my_triangle* triangles;
		
	int numtriangles=0;
	int numvertices=0;
	for(int p=0;p<numpolys;p++)
	{
		numtriangles+=polygons[p].num_triangles;
	}

	numvertices=numtriangles*3;
	pmesh->t_count=numtriangles;
	pmesh->v_count=numvertices;
	pmesh->triangles=(int*)malloc(sizeof(int)*numtriangles*3);
	pmesh->vertices=(float*)malloc(sizeof(float)*numvertices*3);

	int curver=0;
	int curtriangle=0;
	for(int p=0;p<numpolys;p++)
	{
		triangles=polygons[p].triangles;
		for(int t=0;t<polygons[p].num_triangles;t++)
		{
			float x1=_FLOAT(triangles[t].vertex1.x);  
			float y1=_FLOAT(triangles[t].vertex1.y);  
			float z1=_FLOAT(triangles[t].vertex1.z);  

			float x2=_FLOAT(triangles[t].vertex2.x);  
			float y2=_FLOAT(triangles[t].vertex2.y);  
			float z2=_FLOAT(triangles[t].vertex2.z);  

			float x3=_FLOAT(triangles[t].vertex3.x);  
			float y3=_FLOAT(triangles[t].vertex3.y);  
			float z3=_FLOAT(triangles[t].vertex3.z);

			pmesh->vertices[curver*3]=x1;
			pmesh->vertices[curver*3+1]=y1;
			pmesh->vertices[curver*3+2]=z1;

			pmesh->vertices[(curver+1)*3]=x2;
			pmesh->vertices[(curver+1)*3+1]=y2;
			pmesh->vertices[(curver+1)*3+2]=z2;

			pmesh->vertices[(curver+2)*3]=x3;
			pmesh->vertices[(curver+2)*3+1]=y3;
			pmesh->vertices[(curver+2)*3+2]=z3;

			pmesh->triangles[curtriangle*3]=curver;
			pmesh->triangles[curtriangle*3+1]=curver+1;
			pmesh->triangles[curtriangle*3+2]=curver+2;

			curtriangle+=1;
			curver+=3;
		}
	}
	delete [] polygons->triangles;
	polygons->triangles=0;
	delete [] polygons;
	polygons=0;
	return (my_mesh*)pmesh;
}

void navbuilder::init_display_list(VECTOR *bmin,VECTOR *bmax,m_link *the_list)
{
	const float* wbmin = m_meshBMin;
	const float* wbmax = m_meshBMax;
	
	const float ts = m_tileSize*m_cellSize;

	const int tx1 = (int)((_FLOAT(bmin->x) - wbmin[0]) / ts);
	const int ty1 = (int)((_FLOAT(bmin->y) - wbmin[2]) / ts);
	const int tx2 = (int)((_FLOAT(bmax->x) - wbmin[0]) / ts);
	const int ty2 = (int)((_FLOAT(bmax->y) - wbmin[2]) / ts);

	int dataSize = 0;

	int total_tiles=0;

for (int y = ty1-1; y <ty2+2; ++y)
	{
for (int x =tx1-1; x < tx2+2; ++x)
		{
			m_tileBmin[0] = wbmin[0] + (x)*ts;
			m_tileBmin[1] = wbmin[1];
			m_tileBmin[2] = wbmin[2] + (y)*ts;
			
			m_tileBmax[0] = wbmin[0] + (x+1)*ts;
			m_tileBmax[1] = wbmax[1];
			m_tileBmax[2] = wbmin[2] + (y+1)*ts;
			
			const dtMeshTile *tartile=m_navMesh->getTileAt(x,y,0);
			for(int i=0;i<m_navMesh->getMaxTiles();i++)
			{
				const dtMeshTile *curtiles = m_navMesh->getTile(i);
				if (!curtiles || !curtiles->header || !curtiles->dataSize) continue;
				if (!tartile || !tartile->header || !tartile->dataSize) continue;
				if(curtiles==tartile)
				{
					tmp_id *m_id=new tmp_id;
					m_id->tile_id=i;
					m_id->index=total_tiles;
					m_id->_min[0]=_VAR(tartile->header->bmin[0]);
					m_id->_min[1]=_VAR(tartile->header->bmin[2]);
					m_id->_min[2]=_VAR(tartile->header->bmin[1]);
					m_id->_max[0]=_VAR(tartile->header->bmax[0]);
					m_id->_max[1]=_VAR(tartile->header->bmax[2]);
					m_id->_max[2]=_VAR(tartile->header->bmax[1]);
					m_id->meshdata=new my_mesh();
					m_id->meshdata=(my_mesh*)mesh_from_tile(m_id->meshdata,m_id->tile_id);
					the_list->Add((tmp_id*)m_id);
					total_tiles+=1;
				}
			}
		}
	}
}

int navbuilder::get_tileid(VECTOR* pos)
{
/*	float target[3];
	target[0]=_FLOAT(pos->x); 
	target[1]=_FLOAT(pos->z); 
	target[2]=_FLOAT(pos->y); 
*/
	float fromrays[3], fromraye[3]; 
	float torays[3], toraye[3]; 
	
	//path start
	fromrays[0]=_FLOAT(pos->x);
	fromrays[1]=_FLOAT(pos->z);
	fromrays[2]=_FLOAT(pos->y);
	
	fromraye[0]=_FLOAT(pos->x);
	fromraye[1]=-1;
	fromraye[2]=_FLOAT(pos->y);

	float hitt1=0;
	bool hit1 = raycastMesh(fromrays, fromraye, hitt1);
	float start_pos[3];
		
		start_pos[0] = fromrays[0] + (fromraye[0] - fromrays[0])*hitt1;
		start_pos[1] = fromrays[1] + (fromraye[1] - fromrays[1])*hitt1;
		start_pos[2] = fromrays[2] + (fromraye[2] - fromrays[2])*hitt1;

	if (!m_navMesh) return -1;
	
	const float* bmin = m_meshBMin;
	const float* bmax = m_meshBMax;

	const float ts = m_tileSize*m_cellSize;
	const int tx = (int)((start_pos[0] - bmin[0]) / ts);
	const int ty = (int)((start_pos[2] - bmin[2]) / ts);
	
	m_tileBmin[0] = bmin[0] + tx*ts;
	m_tileBmin[1] = bmin[1];
	m_tileBmin[2] = bmin[2] + ty*ts;
	
	m_tileBmax[0] = bmin[0] + (tx+1)*ts;
	m_tileBmax[1] = bmax[1];
	m_tileBmax[2] = bmin[2] + (ty+1)*ts;
	
	m_tileCol = duRGBA(128,32,16,64);
	
	const dtMeshTile *tartile=m_navMesh->getTileAt(tx,ty,0);
	for(int i=0;i<m_navMesh->getMaxTiles();i++)
	{
		const dtMeshTile *curtiles = m_navMesh->getTile(i);
		if (!curtiles || !curtiles->header || !curtiles->dataSize) return -1;
		if (!tartile || !tartile->header || !tartile->dataSize) return -1;
		if(curtiles==tartile)
		{
			return i;
		}
	}
	return -1;
}

void navbuilder::save_all_tiles(char* file)
{
	if (!m_navMesh) return;
	
	FILE* fp = fopen(file, "wb");
	if (!fp)
		return;
	
	NavMeshSetHeader header;
	header.magic = NAVMESHSET_MAGIC;
	header.version = NAVMESHSET_VERSION;
	header.numTiles = 0;
	for (int i = 0; i < m_navMesh->getMaxTiles(); ++i)
	{
		const dtMeshTile* tile = m_navMesh->getTile(i);
		if (!tile || !tile->header || !tile->dataSize) continue;
		header.numTiles++;
	}
	memcpy(&header.params, m_navMesh->getParams(), sizeof(dtNavMeshParams));
	fwrite(&header, sizeof(NavMeshSetHeader), 1, fp);

	for (int i = 0; i < m_navMesh->getMaxTiles(); ++i)
	{
		const dtMeshTile* tile = m_navMesh->getTile(i);
		if (!tile || !tile->header || !tile->dataSize) continue;

		NavMeshTileHeader tileHeader;
		tileHeader.tileRef = m_navMesh->getTileRef(tile);
		tileHeader.dataSize = tile->dataSize;
		fwrite(&tileHeader, sizeof(tileHeader), 1, fp);

		fwrite(tile->data, tile->dataSize, 1, fp);
	}

	fclose(fp);
}

dtNavMesh* navbuilder::loadAll(const char* path)
{
	FILE* fp = fopen(path, "rb");
	if (!fp) 
	{
		MessageBoxA(NULL,"load tiles:could not load file","error",MB_OK);
		return 0;
	}
	
	NavMeshSetHeader header;
	fread(&header, sizeof(NavMeshSetHeader), 1, fp);
	if (header.magic != NAVMESHSET_MAGIC)
	{
		MessageBoxA(NULL,"load tiles:could not load file","error",MB_OK);
		fclose(fp);
		return 0;
	}
	if (header.version != NAVMESHSET_VERSION)
	{
		MessageBoxA(NULL,"load tiles:could not load file","error",MB_OK);
		fclose(fp);
		return 0;
	}
	
	dtNavMesh* mesh = dtAllocNavMesh();
	if (!mesh)
	{
		MessageBoxA(NULL,"load tiles:could not alloc navmesh","error",MB_OK);
		fclose(fp);
		return 0;
	}
	dtStatus status = mesh->init(&header.params);
	if (dtStatusFailed(status))
	{
		MessageBoxA(NULL,"load tiles:could not init navmesh","error",MB_OK);
		fclose(fp);
		return 0;
	}
		
	for (int i = 0; i < header.numTiles; ++i)
	{
		NavMeshTileHeader tileHeader;
		fread(&tileHeader, sizeof(tileHeader), 1, fp);
		if (!tileHeader.tileRef || !tileHeader.dataSize)
			break;

		unsigned char* data = (unsigned char*)dtAlloc(tileHeader.dataSize, DT_ALLOC_PERM);
		if (!data) break;
		memset(data, 0, tileHeader.dataSize);
		fread(data, tileHeader.dataSize, 1, fp);
		
		mesh->addTile(data, tileHeader.dataSize, DT_TILE_FREE_DATA, tileHeader.tileRef, 0);
	}
	
	fclose(fp);
	
	return mesh;
}

void navbuilder::load_all_tiles(char* file)
{
	dtFreeNavMesh(m_navMesh);
	m_navMesh =loadAll(file);
	if(m_navMesh)m_navQuery->init(m_navMesh, 2048);
	else MessageBoxA(NULL,"navmesh=NULL","error",MB_OK);
}

custombuild* navbuilder::Load_Custombuild(char* filename)
{
	custombuild *step=new custombuild;
	FILE* file=fopen(filename,"r b");
	if(!file)
	{
		MessageBoxA(NULL,"could not load file","error",MB_OK);
		return NULL;
	}
	if(file)
	{
		fscanf(file,"ch:%f,cs:%f\n",			&step->CellHeight,&step->CellSize);
		fscanf(file,"detailsampledist:%f\n",	&step->DetailSampleDist);
		fscanf(file,"detailsamplemaxerror:%f\n",&step->DetailSampleMaxError);
		fscanf(file,"maxedgelen:%f\n",			&step->EdgeMaxLen);
		fscanf(file,"maxsimplificationerror:%f\n",&step->EdgeMaxError);
		fscanf(file,"maxvertsperpoly:%f\n",		&step->VertsPerPoly);
		fscanf(file,"mergeregionsize:%f\n",		&step->RegionMergeSize);
		fscanf(file,"minregionsize:%f\n",		&step->RegionMinSize);
		fscanf(file,"tilesize:%f\n",			&step->TileSize);
		fscanf(file,"walkableclimb:%f\n",		&step->AgentMaxClimb);
		fscanf(file,"walkableheight:%f\n",		&step->AgentHeight);
		fscanf(file,"walkableradius:%f\n",		&step->AgentRadius);
		fscanf(file,"walkableslopeangle:%f\n",	&step->AgentMaxSlope);
		fscanf(file,"monotonepartition:%i\n",&step->monotonePartitioning);
		fscanf(file,"buildall:%i\n",&step->buildalltiles); 
		fclose(file);
		return step;
	}
	return NULL;
}

void navbuilder::Export_Custombuild(char* filename,custombuild *step)
{
	FILE* file=fopen(filename,"w+ b");
	if(!file)
	{
		MessageBoxA(NULL,"could not export file","error",MB_OK);
	}
	if(file)
	{
		fprintf(file,"ch:%f,cs:%f\n",			step->CellHeight,step->CellSize);
		fprintf(file,"detailsampledist:%f\n",	step->DetailSampleDist);
		fprintf(file,"detailsamplemaxerror:%f\n",step->DetailSampleMaxError);
		fprintf(file,"maxedgelen:%f\n",			step->EdgeMaxLen);
		fprintf(file,"maxsimplificationerror:%f\n",step->EdgeMaxError);
		fprintf(file,"maxvertsperpoly:%f\n",	step->VertsPerPoly);
		fprintf(file,"mergeregionsize:%f\n",	step->RegionMergeSize);
		fprintf(file,"minregionsize:%f\n",		step->RegionMinSize);
		fprintf(file,"tilesize:%f\n",			step->TileSize);
		fprintf(file,"walkableclimb:%f\n",		step->AgentMaxClimb);
		fprintf(file,"walkableheight:%f\n",		step->AgentHeight);
		fprintf(file,"walkableradius:%f\n",		step->AgentRadius);
		fprintf(file,"walkableslopeangle:%f\n",	step->AgentMaxSlope);
		fprintf(file,"monotonepartition:%i\n",step->monotonePartitioning);
		fprintf(file,"buildall:%i\n",step->buildalltiles);
	fclose(file);
	}
}



static bool intersectSegmentTriangle(const float* sp, const float* sq,
									 const float* a, const float* b, const float* c,
									 float &t)
{
	float v, w;
	float ab[3], ac[3], qp[3], ap[3], norm[3], e[3];
	rcVsub(ab, b, a);
	rcVsub(ac, c, a);
	rcVsub(qp, sp, sq);
	
	// Compute triangle normal. Can be precalculated or cached if
	// intersecting multiple segments against the same triangle
	rcVcross(norm, ab, ac);
	
	// Compute denominator d. If d <= 0, segment is parallel to or points
	// away from triangle, so exit early
	float d = rcVdot(qp, norm);
	if (d <= 0.0f) return false;
	
	// Compute intersection t value of pq with plane of triangle. A ray
	// intersects iff 0 <= t. Segment intersects iff 0 <= t <= 1. Delay
	// dividing by d until intersection has been found to pierce triangle
	rcVsub(ap, sp, a);
	t = rcVdot(ap, norm);
	if (t < 0.0f) return false;
	if (t > d) return false; // For segment; exclude this code line for a ray test
	
	// Compute barycentric coordinate components and test if within bounds
	rcVcross(e, qp, ap);
	v = rcVdot(ac, e);
	if (v < 0.0f || v > d) return false;
	w = -rcVdot(ab, e);
	if (w < 0.0f || v + w > d) return false;
	
	// Segment/ray intersects triangle. Perform delayed division
	t /= d;
	
	return true;
}

static bool isectSegAABB(const float* sp, const float* sq,
						 const float* amin, const float* amax,
						 float& tmin, float& tmax)
{
	static const float EPS = 1e-6f;
	
	float d[3];
	d[0] = sq[0] - sp[0];
	d[1] = sq[1] - sp[1];
	d[2] = sq[2] - sp[2];
	tmin = 0.0;
	tmax = 1.0f;
	
	for (int i = 0; i < 3; i++)
	{
		if (fabsf(d[i]) < EPS)
		{
			if (sp[i] < amin[i] || sp[i] > amax[i])
				return false;
		}
		else
		{
			const float ood = 1.0f / d[i];
			float t1 = (amin[i] - sp[i]) * ood;
			float t2 = (amax[i] - sp[i]) * ood;
			if (t1 > t2) { float tmp = t1; t1 = t2; t2 = tmp; }
			if (t1 > tmin) tmin = t1;
			if (t2 < tmax) tmax = t2;
			if (tmin > tmax) return false;
		}
	}
	
	return true;
}


bool navbuilder::raycastMesh(float* src, float* dst, float& tmin)
{
	float dir[3];
	rcVsub(dir, dst, src);
	tmin = 1.0f;
	bool hit = false;

	m_link *cur;


	for(cur=my_build_list->LHead;cur!=NULL;cur=cur->LNext)
	{
		my_mesh *pmesh=(my_mesh*)cur->Data;
	
		for (int j = 0; j < pmesh->t_count; j ++)
		{

			float t = 1;
			if (intersectSegmentTriangle(src, dst,
										 &pmesh->vertices[pmesh->triangles[j*3]*3],
										 &pmesh->vertices[pmesh->triangles[j*3+1]*3],
										 &pmesh->vertices[pmesh->triangles[j*3+2]*3], t))
			{
				if (t < tmin)
					tmin = t;
				hit = true;
			}
		}
	}
	
	return hit;
}

VECTOR* navbuilder::Find_Path(VECTOR *from,VECTOR* to,int *num_points)
{
/*	float start_pos[3];
	float end_pos[3];
	start_pos[0]=_FLOAT(from->x);
	start_pos[1]=_FLOAT(from->z);
	start_pos[2]=_FLOAT(from->y);

	end_pos[0]=_FLOAT(to->x);
	end_pos[1]=_FLOAT(to->z);
	end_pos[2]=_FLOAT(to->y);
*/
	float fromrays[3], fromraye[3]; 
	float torays[3], toraye[3]; 
	
	//path start
	fromrays[0]=_FLOAT(from->x);
	fromrays[1]=_FLOAT(from->z);
	fromrays[2]=_FLOAT(from->y);
	
	fromraye[0]=_FLOAT(from->x);
	fromraye[1]=-1;
	fromraye[2]=_FLOAT(from->y);

	//path end
	torays[0]=_FLOAT(to->x);
	torays[1]=_FLOAT(to->z);
	torays[2]=_FLOAT(to->y);

	toraye[0]=_FLOAT(to->x);
	toraye[1]=-1;
	toraye[2]=_FLOAT(to->y);

	float hitt1=0;
	float hitt2=0;
	bool hit1 = raycastMesh(fromrays, fromraye, hitt1);
	bool hit2 = raycastMesh(torays, toraye, hitt2);

	float start_pos[3];
		
		start_pos[0] = fromrays[0] + (fromraye[0] - fromrays[0])*hitt1;
		start_pos[1] = fromrays[1] + (fromraye[1] - fromrays[1])*hitt1;
		start_pos[2] = fromrays[2] + (fromraye[2] - fromrays[2])*hitt1;

		float end_pos[3];

		end_pos[0] = torays[0] + (toraye[0] - torays[0])*hitt2;
		end_pos[1] = torays[1] + (toraye[1] - torays[1])*hitt2;
		end_pos[2] = torays[2] + (toraye[2] - torays[2])*hitt2;

		float m_polyPickExt[3];
		dtQueryFilter m_filter;
		dtPolyRef m_startRef=0;
		dtPolyRef m_endRef=0;
		dtPolyRef m_polys[256];
		int m_npolys=0;

		float m_straightPath[256*3];
		unsigned char m_straightPathFlags[256];
		dtPolyRef m_straightPathPolys[256];

		m_filter.setIncludeFlags(SAMPLE_POLYFLAGS_ALL ^ SAMPLE_POLYFLAGS_DISABLED);
		m_filter.setExcludeFlags(0);

		m_navQuery->findNearestPoly(start_pos, m_polyPickExt, &m_filter, &m_startRef, 0);
		m_navQuery->findNearestPoly(end_pos, m_polyPickExt, &m_filter, &m_endRef, 0);

		m_navQuery->findPath(m_startRef, m_endRef, start_pos ,end_pos, &m_filter, m_polys, &m_npolys, 256);
		int m_nstraightPath = 0;
		
		float epos[3];
		epos[0]=end_pos[0];
		epos[1]=end_pos[1];
		epos[2]=end_pos[2];

		if (m_polys[m_npolys-1] != m_endRef)
		m_navQuery->closestPointOnPoly(m_polys[m_npolys-1], end_pos, epos);
   		m_navQuery->findStraightPath(start_pos, epos, m_polys, m_npolys,
								 m_straightPath, m_straightPathFlags,
								 m_straightPathPolys, &m_nstraightPath, 256);

		VECTOR* points=new VECTOR[m_nstraightPath];

		num_points[0]=m_nstraightPath;

		if (m_straightPath && num_points>0)
		{
			for (int i = 0; i < m_nstraightPath; ++i)
			{
				points[i].x=_VAR(m_straightPath[i*3]);
				points[i].z=_VAR(m_straightPath[i*3+1]);
				points[i].y=_VAR(m_straightPath[i*3+2]);
			}
		return points;
		}
	num_points=0;
	return NULL;
}

VECTOR* navbuilder::Nearest_Nav_Point(VECTOR* from)
{
/*	float start_pos[3];
	float end_pos[3];
	start_pos[0]=_FLOAT(from->x);
	start_pos[1]=_FLOAT(from->z);
	start_pos[2]=_FLOAT(from->y);
*/
	float fromrays[3], fromraye[3]; 
	float torays[3], toraye[3]; 
	
	//path start
	fromrays[0]=_FLOAT(from->x);
	fromrays[1]=_FLOAT(from->z);
	fromrays[2]=_FLOAT(from->y);
	
	fromraye[0]=_FLOAT(from->x);
	fromraye[1]=-1;
	fromraye[2]=_FLOAT(from->y);


	float hitt1=0;
	bool hit1 = raycastMesh(fromrays, fromraye, hitt1);

	float start_pos[3];
		
		start_pos[0] = fromrays[0] + (fromraye[0] - fromrays[0])*hitt1;
		start_pos[1] = fromrays[1] + (fromraye[1] - fromrays[1])*hitt1;
		start_pos[2] = fromrays[2] + (fromraye[2] - fromrays[2])*hitt1;


	float m_polyPickExt[3];
	dtQueryFilter m_filter;
	dtPolyRef m_startRef=0;

	m_navQuery->findNearestPoly(start_pos, m_polyPickExt, &m_filter, &m_startRef, 0);

	float pos[3];
	pos[0]=start_pos[0];
	pos[1]=start_pos[1];
	pos[2]=start_pos[2];

	m_navQuery->closestPointOnPoly(m_startRef,start_pos,pos);
	VECTOR* nearest=new VECTOR;
	nearest->x=_VAR(pos[0]);
	nearest->y=_VAR(pos[2]);
	nearest->z=_VAR(pos[1]);
	if(nearest)	return nearest;

	return NULL;
}

static float tfrand()
{
	return (float)rand()/(float)RAND_MAX;
}

VECTOR* navbuilder::Nav_Point_Random(int max_points)
{
	dtQueryFilter m_filter;
	dtPolyRef m_startRef=0;
	float *m_randPoints=new float[max_points*3];

	int m_nrandPoints = 0;

	if(m_navQuery)
	{
		for (int i = 0; i < max_points; i++)
		{
			float pt[3];
			dtPolyRef ref;
			dtStatus status = m_navQuery->findRandomPoint(&m_filter, tfrand, &ref, pt);
			if (dtStatusSucceed(status))
			{
				dtVcopy(&m_randPoints[m_nrandPoints*3], pt);
				m_nrandPoints++;
			}
		}
		if(m_nrandPoints>0)
		{	
			VECTOR* point=new VECTOR[m_nrandPoints];
			for(int i=0;i<m_nrandPoints;i++)
			{
				point[i].x=_VAR(m_randPoints[i*3]); 
				point[i].z=_VAR(m_randPoints[i*3+1]); 
				point[i].y=_VAR(m_randPoints[i*3+2]); 
			}
			if(point)	return point;
		}
	}
	return NULL;
}

VECTOR* navbuilder::Nav_Point_Random_Radius(VECTOR* from,float radius)
{
	
	float m_polyPickExt[3];
	dtQueryFilter m_filter;
	dtPolyRef m_startRef=0;

	float* m_randPoints=new float[3];
	float start_pos[3];

	start_pos[0]=_FLOAT(from->x);
	start_pos[1]=_FLOAT(from->z);
	start_pos[2]=_FLOAT(from->y);

/*	float fromrays[3], fromraye[3]; 
	float torays[3], toraye[3]; 
	
	//path start
	fromrays[0]=_FLOAT(from->x);
	fromrays[1]=_FLOAT(from->z);
	fromrays[2]=_FLOAT(from->y);
	
	fromraye[0]=_FLOAT(from->x);
	fromraye[1]=-1;
	fromraye[2]=_FLOAT(from->y);


	float hitt1=0;
	bool hit1 = raycastMesh(fromrays, fromraye, hitt1);

	float start_pos[3];
		
		start_pos[0] = fromrays[0] + (fromraye[0] - fromrays[0])*hitt1;
		start_pos[1] = fromrays[1] + (fromraye[1] - fromrays[1])*hitt1;
		start_pos[2] = fromrays[2] + (fromraye[2] - fromrays[2])*hitt1;
*/
	m_navQuery->findNearestPoly(start_pos, m_polyPickExt, &m_filter, &m_startRef, 0);

	float pos[3];
	pos[0]=start_pos[0];
	pos[1]=start_pos[1];
	pos[2]=start_pos[2];

	m_navQuery->closestPointOnPoly(m_startRef,start_pos,pos);

	float pt[3];
	dtPolyRef ref;
	dtStatus status = m_navQuery->findRandomPointAroundCircle(m_startRef,pos,radius, &m_filter, tfrand, &ref, pt);
	if (dtStatusSucceed(status))
	{
		VECTOR* points=new VECTOR;
		points->x=_VAR(pt[0]);
		points->z=_VAR(pt[1]);
		points->y=_VAR(pt[2]);
		if(points)return points;
	}
return NULL;
}

