//#pragma once

#define _USE_MATH_DEFINES
#include <math.h>
#include <stdio.h>
#include <string.h>
#include "SDL.h"
#include "SDL_opengl.h"
#include "imgui.h"
#include "InputGeom.h"
#include "Sample.h"
#include "Sample_TileMesh.h"
#include "Recast.h"
#include "RecastDebugDraw.h"
#include "DetourNavMesh.h"
#include "DetourNavMeshBuilder.h"
#include "DetourDebugDraw.h"
#include "NavMeshTesterTool.h"
#include "NavMeshPruneTool.h"
#include "OffMeshConnectionTool.h"
#include "ConvexVolumeTool.h"
#include "CrowdTool.h"

#include "stdafx.h"

#include <stdlib.h>
#include <malloc.h>
#include <memory.h>
#include <tchar.h>
#include <math.h.>
#include "..\\acknex\\adll.h"

#include "TestCase.h"
#include "Recast.h"
#include "DetourNavMeshQuery.h"
#include "DetourCommon.h"

#include "process.h"

#include "buildmeshstruct.h"

#ifndef navbuilder_h
#define navbuilder_h

typedef struct custombuild
{
	float CellSize;
	float CellHeight;
	float AgentHeight;
	float AgentRadius;
	float AgentMaxClimb;
	float AgentMaxSlope;
	float RegionMinSize;
	float RegionMergeSize;
	float EdgeMaxLen;
	float EdgeMaxError;
	float VertsPerPoly;
	float DetailSampleDist;
	float DetailSampleMaxError;
	int monotonePartitioning;
	float TileSize;
	int buildalltiles;
}custombuild;

typedef struct my_triangle
{
	VECTOR vertex1;
	VECTOR vertex2;
	VECTOR vertex3;
}my_triangle;

typedef struct my_polygon
{
	my_triangle *triangles;
	int num_triangles;
}my_polygon;

typedef struct tmp_id
{
	int tile_id;
	int index;
	var _min[3];
	var _max[3];
	my_mesh* meshdata;
}tmp_id;

//using for matrix transform functions
#include <irrlicht.h>
using namespace irr;
using namespace core;

class navbuilder
{
public:
	int tc;
	int vc;
	int nc;

	navbuilder(void);
	~navbuilder(void);


	bool raycastMesh(float* src, float* dst, float& tmin);
	
	void save_final_meshes(char* filename);

	void remove_tile(int _tileid);
	void get_tile_box(int tileid,var *min,var *max);
	void* mesh_from_tile(my_mesh* pmesh,int tile);
	void save_tile(char* file,int tileid);
	int loadtile(const char* path);
	int get_tileid(VECTOR* pos);
	void remove_tile(VECTOR* pos);
	int build_tile(VECTOR* pos);
	void build_all_tiles();
	void remove_all_tiles();
	void save_all_tiles(char *file);
	void load_all_tiles(char *file);
	void rem_tiles_box(VECTOR* bmin,VECTOR* bmax);
	int build_tiles_box(VECTOR* bmin,VECTOR* bmax);
	dtNavMesh* loadAll(const char* path);
	void buildAllTiles();
	void removeAllTiles();

	void save_mdl(char* filename,void* mesh,int swapindices);
	VECTOR* Find_Path(VECTOR *from,VECTOR* to,int *num_points);
	VECTOR* Nearest_Nav_Point(VECTOR* from);
	VECTOR* Nav_Point_Random(int max_points);
	VECTOR* Nav_Point_Random_Radius(VECTOR* from,float radius);
	void calc_bbox(ENTITY *ent,int chunk,var *min,var *max);
	void init_display_list(VECTOR *bmin,VECTOR *bmax,m_link *the_list);

	my_polygon* navbuilder::Get_Polygons_tile(int tile);
	int Get_Poly_Count_tile(int tileid);
	void* finalize_meshes();

	int Sample_From_Custombuild(custombuild *step);
	int Nav_From_Custombuild(custombuild* step);
	void Export_Custombuild(char* filename,custombuild *step);
	custombuild* Load_Custombuild(char* filename);


	unsigned char* buildTileMesh(const int tx, const int ty, const float* bmin, const float* bmax, int& dataSize);
	void cleanup();
	void resetCommonSettings();

	m_link* my_build_list;
	FILE* file;

	class dtNavMesh* m_navMesh;
	class dtNavMeshQuery* m_navQuery;
	class dtCrowd* m_crowd;
	rcPolyMeshDetail* m_dmesh;
	unsigned char m_navMeshDrawFlags;

	float m_cellSize;
	float m_cellHeight;
	float m_agentHeight;
	float m_agentRadius;
	float m_agentMaxClimb;
	float m_agentMaxSlope;
	float m_regionMinSize;
	float m_regionMergeSize;
	bool m_monotonePartitioning;
	float m_edgeMaxLen;
	float m_edgeMaxError;
	float m_vertsPerPoly;
	float m_detailSampleDist;
	float m_detailSampleMaxError;
	float m_meshBMin[3], m_meshBMax[3];
	float m_tileSize;

	float m_tileBmin[3];
	float m_tileBmax[3];

	SampleTool* m_tool;
	SampleToolState* m_toolStates[MAX_TOOLS];
	
	BuildContext* m_ctx;
	bool m_buildAll;

	bool m_keepInterResults;
	float m_totalBuildTimeMs;

	unsigned char* m_triareas;
	rcHeightfield* m_solid;
	rcCompactHeightfield* m_chf;
	rcContourSet* m_cset;
	rcPolyMesh* m_pmesh;
	rcConfig m_cfg;	
	int m_maxTiles;
	int m_maxPolysPerTile;
	
	unsigned int m_tileCol;
	float m_tileBuildTime;
	float m_tileMemUsage;
	int m_tileTriCount;
};


#endif