#include "threadmanager.h"

threadmanager::threadmanager(void)
{
	this->initial_amount=0;
	this->threads_list=new m_link();
	this->jobs_list=new m_link();
	this->run=true;
	
	this->hrun=CreateEvent(NULL, TRUE, FALSE, NULL);

	//start main procedure
	UINT uThreadId;
	this->my_handle=(HANDLE)_beginthreadex(NULL,0,threadmanager::ThreadProc,this,0,(UINT*)&uThreadId);
	Sleep(1);
}

threadmanager::~threadmanager(void)
{
	this->run=false;
	this->threads_list->~m_link();
	this->jobs_list->~m_link();
}

//signal thread to run
void threadmanager::signal_run()
{
	SetEvent(this->hrun);
}

HANDLE threadmanager::get_handle()
{
	return this->my_handle;
}

//create worker threads of threads amount
void threadmanager::init(int threads)
{
	for(int i=0;i<threads;i++)
	{
		workerthread *m_thread=new workerthread();
		m_thread->set_func(NULL);
		m_thread->set_param(NULL);
		//store threads
		this->threads_list->Add((workerthread*)m_thread);
		this->initial_amount+=1;
	}
}

//count total threads in list
int threadmanager::total_threads()
{
	m_link *cur;
	int total=0;
	for(cur=this->threads_list->LHead;cur!=NULL;cur=cur->LNext)
	{
		workerthread* m_thread=(workerthread*)cur->Data;
		if(m_thread)
		{
			total+=1;
		}
	}
	return total;
}

//count total jobs in list
int threadmanager::total_jobs()
{
	m_link *cur;
	int total=0;
	for(cur=this->jobs_list->LHead;cur!=NULL;cur=cur->LNext)
	{
		_jobs* m_job=(_jobs*)cur->Data;
		if(m_job)
		{
			total+=1;
		}
	}
	return total;
}

//add a job to the list
void threadmanager::add_job(LPTHREAD_START_ROUTINE pfunc,LPVOID param)
{
	_jobs *m_job=new _jobs;
	m_job->param=param;
	m_job->pfunc=pfunc;
	this->jobs_list->Add((_jobs*)m_job);
}

void threadmanager::stop()
{
	this->run=false;
}

_jobs* threadmanager::find_job()
{
	m_link *cur;
	for(cur=this->jobs_list->LHead;cur!=NULL;cur=cur->LNext)
	{
		_jobs *m_job=(_jobs*)cur->Data;
		if(m_job)
		{
			return m_job;
		}
	}
	return NULL;
}

workerthread* threadmanager::find_free_thread()
{
	m_link *cur;
	for(cur=this->threads_list->LHead;cur!=NULL;cur=cur->LNext)
	{
		workerthread* m_thread=(workerthread*)cur->Data;
		if(m_thread)
		{
			//thread is free
			if(m_thread->is_free() && !m_thread->is_deleted())
			{
				return m_thread;
			}
		}
	}
	return NULL;
}

void threadmanager::remove_empty_threads()
{
	m_link *cur;
	for(cur=this->threads_list->LHead;cur!=NULL;cur=cur->LNext)
	{
		workerthread* m_thread=(workerthread*)cur->Data;
		//thread is deleted
		if(m_thread && m_thread->is_deleted())
		{
			this->threads_list->LPos =cur;
			this->threads_list->Data=NULL;
			this->threads_list->Remove();
			delete m_thread;
			m_thread=NULL;
		}
	}
}

void threadmanager::remove_job(_jobs* this_job)
{
	m_link *cur;
	for(cur=this->jobs_list->LHead;cur!=NULL;cur=cur->LNext)
	{
		_jobs* m_job=(_jobs*)cur->Data;
		if(m_job==this_job)
		{
			this->jobs_list->LPos =cur;
			this->jobs_list->Data=NULL;
			this->jobs_list->Remove();
			delete m_job;
			m_job=NULL;
		}
	}
}

void threadmanager::maintain_thread_amount(int last_amount)
{
	//create more threads if threads deleted
	if(last_amount<this->initial_amount)
	{
		workerthread *m_thread=new workerthread();
		m_thread->set_func(NULL);
		m_thread->set_param(NULL);
		//store threads
		this->threads_list->Add((workerthread*)m_thread);
	}
}


UINT __stdcall threadmanager::ThreadProc(LPVOID param)
{
	threadmanager* manager =(threadmanager*)param;// static_cast<threadmanager*>(param);

	DWORD result=WaitForSingleObject(manager->hrun,INFINITE);

	//manager->run=true;
//	while(manager->run)
	if(result==WAIT_OBJECT_0)
	{
		//delete empty threads
		manager->remove_empty_threads();

		int tot_jobs=manager->total_jobs();
		int tot_threads=manager->total_threads();

		//replenish thread pool
		//manager->maintain_thread_amount(tot_threads);


		//if theres threads & jobs
		if(tot_jobs>0 && tot_threads>0)
		{
			//find free thread
			workerthread* m_thread=manager->find_free_thread();
			if(m_thread)
			{
				//find a job
				_jobs* m_job=manager->find_job();
				if(m_job)
				{
					//assign job
					m_thread->set_func(m_job->pfunc);
					m_thread->set_param(m_job->param);

					//run thread job
					m_thread->signal_run();
					Sleep(1);
					//remove job
					manager->remove_job(m_job);
				}
			}
		}

	//	Sleep(1);

		//all threads were depleted
		//thread manager =done
		if(tot_threads==0)
		{
			manager->run=false;
		}
	}
	return 0;
}