#include "workerthread.h"

workerthread::workerthread(void)
{
	this->hrun=NULL;
	this->my_handle=NULL;
	this->params=NULL;
	this->pfunc=NULL;
	this->free=true;
	this->deleted=false;

	this->hrun=CreateEvent(NULL, TRUE, FALSE, NULL);

	UINT uThreadId=0;
	this->my_handle=(HANDLE)_beginthreadex(NULL,0,workerthread::ThreadProc,this,0,(UINT*)&uThreadId);
}

workerthread::~workerthread(void)
{
	this->free=false;
	this->shutdown();
}


bool workerthread::is_busy()
{
	DWORD result=WaitForSingleObject(this->my_handle,0);
	if(result==WAIT_OBJECT_0)
	{
		//thread is done
		return false;
	}
	//thread is busy
	return true;
}


//is the thread deleted
bool workerthread::is_deleted()
{
	return this->deleted;
}

//is the thread free for work
bool workerthread::is_free()
{
	return this->free; 
}

//return the handle of the thread
HANDLE workerthread::get_handle()
{
	return this->my_handle;
}

//signal thread to run
void workerthread::signal_run()
{
	SetEvent(this->hrun);
}

//set function to execute
void workerthread::set_func(LPTHREAD_START_ROUTINE pfunc)
{
	this->pfunc=pfunc;
}

//set param to pass to the function to execute
void workerthread::set_param(LPVOID param)
{
	this->params=param;
}

//shutdown the thread
void workerthread::shutdown()
{
	_endthreadex(1);
	this->deleted=true;
	this->hrun=NULL;
	this->my_handle=NULL;
	this->params=NULL;
	this->pfunc=NULL;
	CloseHandle(this->my_handle);
	CloseHandle(this->hrun);
}

//run a function that waits until a run is signaled
//then run the function procedure
UINT __stdcall workerthread::ThreadProc(LPVOID param)
{
	workerthread* thisthread = static_cast<workerthread*>(param);
	//wait until run is signaled
	DWORD result=WaitForSingleObject(thisthread->hrun,INFINITE);

	//run signaled
	if(result==WAIT_OBJECT_0)
	{
		thisthread->free=false;
		LPTHREAD_START_ROUTINE proc=thisthread->pfunc; 
		LPVOID par=thisthread->params;
		
		//execute function
		if(proc)proc(par);
		Sleep(1);
	}

	//thread returned from proc and is done 
	thisthread->shutdown();

	return 0;
}