//refer to the pdf for more details

//how to build the navmesh:

//step 1:
//add the needed include files 
//#include "peragro.h" (the main include file for using the plugin)
//#include "helper.c"  (debug draw functions and small build setup example)
//#include "wandering_bob.c" (this is a pathfinding example)

//step 2:
//load your level or create your level by code

//step 3:
//create the navmesh as shown in this example by following this order:

//create the navmesh builder object if it does not exist
//check if the navmesh builder is created
//add all the needed geometry for navmeshing to the build list

//start the build process:
 //(in this example i use the pre made setup function "start_build" in helper.c )
 
//wait for the build process to complete
//ready for pathfinding 
//optional do a debug display

//step 4:
//clean up after your done 

/////////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////////
#include <litec.h>
#include <d3d9.h> 
#include <acknex.h>
#include <default.c>
#include <mtlFX.c>

#include "peragro.h"
#include "helper.c"
#include "wandering_bob.c"
/////////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////////


/////////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////////
//clean(removes the navmesh builder object if it exist)
function clean_up()
{
	if (peragro_valid()) peragro_destroy();
}
/////////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////////


/////////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////////
//navmeshing
void make_world()
{
	//create the navmesh builder object if it does not exist
	if (!peragro_valid()) peragro_create();
	
	//check if the navmesh builder is created
	if (peragro_valid())
	{
		//add all the needed geometry for navmeshing to the build list
		
		ENTITY* ent1=ent_for_name("machinegun_mdl_001");
		ENTITY* t=ent_for_name("terrain_hmp_001");

		//add prefered models 
		//you could mark models with a flag in wed if you want to do a loop
		//through all models and find the ones you need to be part of the path mesh
		mesh_from_ent(ent1,0);
		//add all the blocks
		mesh_from_level_blocks(level_ent,ent_status(level_ent,16),0);
		//add all chunks of the terrain
		mesh_from_hmp(t,ent_status(t,16),0);

		//start the build process at a tilesize of 32 and build all tiles
		//this function is pre made inside of helper.c
		start_build(32,1);
		
		//wait for the build thread to complete
		while(peragro_thread(mainbuild)==1)
		{
			busybitmap.angle-=21*time_step;
			wait(1);
		}
	//ready for pathfinding 
		make_points();
		ent_create("redguard.mdl",vector(0,100,200),go_bob);
		

	//optional debug display
		draw_debug();
	}
}
/////////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////////


/////////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////////
//startup
function main()
{
	max_particles = 900000;
	video_switch(10,32,0);
	vec_set(sky_color,vector(1,0,0));

	fps_max=60;

	//activate level of detail for testing
	terrain_lod = 4; 
	
	level_load("test.wmb");
	wait(2);
	def_move();
	def_debug();
	on_exit=clean_up();

	show_busy_bitmap();

	camera.arc=90;
	camera.tilt=-33;
	camera.pan=360;
	camera.clip_near=1;

	vec_set(camera.x,vector(-500,0,1500));

	make_world();
}
