#include "peragro.h"

//////////////////////////////////////
//////////////////////////////////////
//************************************
//display options

int _draw_boxes	=0;
int _draw_meshes	=1;
int _draw_ids		=1;


var _draw_height	=0;
var _alpha			=60;
var bsize			=1500;

//////////////////////////////////////
//////////////////////////////////////
//************************************
//function pre-definitions
void draw_tiles_near_box(VECTOR* _bmin,VECTOR * _bmax,int draw_boxes,int draw_meshes,int draw_ids,var draw_height,var _alpha);


TEXT* tile_id_text =
{
   strings = 1;
   flags = CENTER_X;
}

//////////////////////////////////////
//////////////////////////////////////
//************************************
//busy icon

BMAP* coil="coil.pcx";
PANEL* busybitmap;

void show_busy_bitmap()
{
	busybitmap=pan_create(NULL,100);
	busybitmap.pos_x		=20;
	busybitmap.pos_y		=150;
	busybitmap.bmap		=coil;
	busybitmap.center_x 	=128;
	busybitmap.center_y 	=128;
	busybitmap.scale_x	*=0.5;
	busybitmap.scale_y	*=0.5;
	set(busybitmap,OVERLAY|SHOW);	
}


//////////////////////////////////////
//////////////////////////////////////
//************************************
//DEBUG DISPLAY

void draw_debug()
{
	var wmin[3];
	var wmax[3];
	VECTOR wbmin;
	VECTOR wbmax;
	VECTOR cbmin;
	VECTOR cbmax;
	
	peragro_world_box(wmin,wmax);
	vec_set(wbmin , vector(wmin[0],wmin[1],wmin[2]));
	vec_set(wbmax , vector(wmax[0],wmax[1],wmax[2]));
	
	peragro_init_tiledata(wbmin,wbmax);
	
	while(1)
	{
		vec_set(cbmin , vector(camera.x-bsize,camera.y-bsize,camera.z-100));
		vec_set(cbmax , vector(camera.x+bsize,camera.y+bsize,camera.z+100));
		
		draw_box3d(cbmin,cbmax,vector(0,255,0),100);
		
		draw_tiles_near_box(cbmin,cbmax,_draw_boxes,_draw_meshes,_draw_ids,_draw_height,_alpha);
		
		wait(1);
	}
}

void draw_tiles_near_box(VECTOR* _bmin,VECTOR * _bmax,int draw_boxes,int draw_meshes,int draw_ids,var draw_height,var _alpha)
{
	peragro_mark_box(_bmin,_bmax);

	int total=peragro_in_tmp_list();

	if(total>0)
	{
		void* nearmesh;
		int i;
		for (i=0;i<total;i++)
		{
			var tmin[3];
			var tmax[3];
			int mtileid;

			nearmesh=mesh_from_tmp_list(i,tmin,tmax,mtileid);

			if(nearmesh)
			{
	
				//tile box
				if(draw_boxes)
				{
					VECTOR tbmin;
					VECTOR tbmax;
					vec_set(tbmin,vector(tmin[0],tmin[1],tmin[2]));
					vec_set(tbmax,vector(tmax[0],tmax[1],tmax[2]));
					draw_box3d(tbmin,tbmax,vector(0,0,255),_alpha);
				}
				
				//draw tile ids
				if(draw_ids)
				{
					VECTOR hitpos;

					vec_set(hitpos,vector((tmin[0]+tmax[0])/2,(tmin[1]+tmax[1])/2,tmax[2]));
					
					var dist=vec_dist(hitpos,vector((tmin[0]+tmax[0])/2,(tmin[1]+tmax[1])/2,tmin[2]));
					
					var res=c_trace(hitpos,vector(hitpos.x,hitpos.y,hitpos.z-dist),IGNORE_ME|IGNORE_SPRITES|IGNORE_CONTENT);
			  		if(res>0)vec_set(hitpos,vector(hit.x,hit.y,hit.z));
			  		else vec_set(hitpos,vector((tmin[0]+tmax[0])/2,(tmin[1]+tmax[1])/2,tmax[2]));
					
					VECTOR textpos;
					vec_set(textpos, vector(hitpos.x,hitpos.y,hitpos.z+draw_height));
	
					if(vec_to_screen(textpos,camera))
					{
						STRING* mstring="";
	
						tile_id_text.pos_x = textpos.x;
						tile_id_text.pos_y = textpos.y;
						str_printf(mstring,"%d",(long)mtileid);
						str_cpy((tile_id_text.pstring)[0],mstring);
	
						draw_obj(tile_id_text);
						draw_line3d(vector(hitpos.x,hitpos.y,hitpos.z),NULL,_alpha);
						draw_line3d(vector(hitpos.x,hitpos.y,hitpos.z),COLOR_GREEN,_alpha);
						draw_line3d(vector(hitpos.x,hitpos.y,hitpos.z+draw_height),COLOR_GREEN,_alpha);
						draw_point3d(vector(hitpos.x,hitpos.y,hitpos.z),COLOR_GREEN,_alpha,5);
					}
				}
				
				//detail
				if(draw_meshes)
				{
	
					int vcount=get_vertcount (nearmesh);
					int tcount=get_tricount  (nearmesh);
				
					int*   tbuf=get_tbuffer (nearmesh);
					float* vbuf=get_vbuffer (nearmesh);
					int i;
	
					for(i=0;i<tcount;i++)
					{
						//POINTS
						//triangle i point 1
						draw_point3d(vector(vbuf[tbuf[i*3]*3],vbuf[tbuf[i*3]*3+1],vbuf[tbuf[i*3]*3+2]+draw_height),COLOR_WHITE,_alpha,2);
						//triangle i point 2
						draw_point3d(vector(vbuf[tbuf[i*3+1]*3],vbuf[tbuf[i*3+1]*3+1],vbuf[tbuf[i*3+1]*3+2]+draw_height),COLOR_WHITE,_alpha,2);
						//triangle i point 3
						draw_point3d(vector(vbuf[tbuf[i*3+2]*3],vbuf[tbuf[i*3+2]*3+1],vbuf[tbuf[i*3+2]*3+2]+draw_height),COLOR_WHITE,_alpha,2);
						
						//TRIANGLES
						//triangle i point 1
						draw_line3d(vector(vbuf[tbuf[i*3]*3],vbuf[tbuf[i*3]*3+1],vbuf[tbuf[i*3]*3+2]+draw_height),NULL,_alpha);
						draw_line3d(vector(vbuf[tbuf[i*3]*3],vbuf[tbuf[i*3]*3+1],vbuf[tbuf[i*3]*3+2]+draw_height),COLOR_WHITE,_alpha);
						
						//triangle i point 2
						draw_line3d(vector(vbuf[tbuf[i*3+1]*3],vbuf[tbuf[i*3+1]*3+1],vbuf[tbuf[i*3+1]*3+2]+draw_height),COLOR_WHITE,_alpha);
						
						//triangle i point 3
						draw_line3d(vector(vbuf[tbuf[i*3+2]*3],vbuf[tbuf[i*3+2]*3+1],vbuf[tbuf[i*3+2]*3+2]+draw_height),COLOR_WHITE,_alpha);
						
						//triangle i point 1
						draw_line3d(vector(vbuf[tbuf[i*3]*3],vbuf[tbuf[i*3]*3+1],vbuf[tbuf[i*3]*3+2]+draw_height),COLOR_WHITE,_alpha);
					}
				}
				
				
			}
		}
	}
}

/////////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////////
//sample function for setting up build filtering values 
custombuild *mybuild;
peragro_thr mainbuild;

float character_radius=30;
float character_height=400;

function start_build(var tsize,int alltiles)
{
	//create default build values
	mybuild=cbuild_create();
	mybuild.TileSize  			 =(float)tsize;
	mybuild.CellSize  			 =character_radius/2;
	mybuild.CellHeight			 =mybuild.CellSize/2;
	mybuild.AgentHeight			 =peragro_ceiling(character_height/mybuild.CellHeight);
	mybuild.AgentRadius			 =peragro_ceiling(character_radius/mybuild.CellSize);
	mybuild.AgentMaxSlope		 =(float)45;
	mybuild.AgentMaxClimb		 =peragro_ceiling((character_height/2)/mybuild.CellHeight);
	//mybuild.RegionMinSize		 =(float)1;
	//mybuild.RegionMergeSize	 =(float)20;
	//mybuild.EdgeMaxLen			 =mybuild.AgentRadius*8;
	mybuild.EdgeMaxLen			 =0;
	//mybuild.EdgeMaxError		 =(float)1.3;
	//mybuild.VertsPerPoly		 =6;
	//mybuild.DetailSampleDist	 =(float)12;
	//mybuild.DetailSampleMaxError=(float)20;
	mybuild.monotonePartitioning=0;
	mybuild.buildalltiles=alltiles;

	//save build to file
	cbuild_save("custombuild.txt",mybuild);
	//load build from file
	mybuild=cbuild_load("custombuild.txt");

	mainbuild=peragro_init(mybuild);
	//clean 	
	cbuild_remove(mybuild);
}
/////////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////////

