
//////////////////////////////////////
//////////////////////////////////////
//************************************
// TYPES
//

typedef void* 		peragro_thr;

//////////////////////////////////////
//////////////////////////////////////
//************************************
// HELPER/DEBUG FUNCTIONS
//
void  peragro_update_bounds();
int   peragro_in_tmp_list	();
void  peragro_init_tiledata(VECTOR *bmin , VECTOR *bmax);
void  peragro_mark_box		(VECTOR *bmin , VECTOR *bmax);
void* mesh_from_tmp_list	(int index ,var *_min , var* _max , int *tileid);

float peragro_ceiling	(float val);
void  peragro_calc_box  (ENTITY *ent , int chunk , var *_min , var *_max);
void  peragro_world_box	(var *_min , var *_max);

VECTOR* peragro_rnd_pnts	    (int points);
VECTOR* peragro_nearest_pnt 	 (VECTOR* from);
VECTOR* peragro_rnd_pnt_radius (VECTOR* from , float radius);

//////////////////////////////////////
//////////////////////////////////////
//************************************
// BUILD PARAM FUNCTIONS
//

typedef struct custombuild
{
	float CellSize;
	float CellHeight;
	float AgentHeight;
	float AgentRadius;
	float AgentMaxClimb;
	float AgentMaxSlope;
	float RegionMinSize;
	float RegionMergeSize;
	float EdgeMaxLen;
	float EdgeMaxError;
	float VertsPerPoly;
	float DetailSampleDist;
	float DetailSampleMaxError;
	int   monotonePartitioning;
	float TileSize;
	int   buildalltiles;
}custombuild;

custombuild* cbuild_create ();
custombuild* cbuild_load   (char* filename);

void cbuild_remove (custombuild* mybuild);
void  cbuild_save   (char* filename , custombuild *step);


//////////////////////////////////////
//////////////////////////////////////
//************************************
// PERAGRO OBJECT FUNCTIONS
//

int peragro_valid ();
void peragro_create  ();
void peragro_destroy ();

peragro_thr peragro_init (custombuild* step);

//////////////////////////////////////
//////////////////////////////////////
//************************************
// STANDARD PATH-FINDING FUNCTIONS
//

VECTOR* peragro_findpath (VECTOR *from , VECTOR* to , int *num_points);

//////////////////////////////////////
//////////////////////////////////////
//************************************
// CROWD/AGENT FUNCTIONS
//


//////////////////////////////////////
//////////////////////////////////////
//************************************
// INPUT GEOMETRY FUNCTIONS
//

void mesh_from_level_blocks		  (ENTITY* level , int chunks, int _lod);
void* mesh_from_level_blocks_chunk (ENTITY* level , int chunks, int _lod);

void trans_mesh_ent	(void* mesh , ENTITY* ent);
void trans_mesh_arg	(void* mesh , ANGLE* angle , VECTOR* scale , VECTOR* pos);

void* mesh_from_ent		  (ENTITY* ent, int _lod);
void  mesh_from_hmp		  (ENTITY* ent , int chunks, int _lod);
void* mesh_from_hmp_chunk (ENTITY* ent , int chunk, int _lod);

void* mesh_from_obj_file	(char* filename);
void* mesh_from_buffers		(D3DVERTEX* vbuffer , short* tbuffer , int verts , int tris);

int get_vertcount (void* mesh);
int get_tricount  (void* mesh);

int*   get_tbuffer (void* mesh);
float* get_vbuffer (void* mesh);


//SIMPLE PURE GEOMETRY FILE FORMAT, COULD BE IMPORTED AS A WAVEFRONT OBJ IN MODEL APPLICATIONS
void save_mesh  (void* mesh , char* filename);

//SIMPLE PURE GEOMETRY FILE FORMAT, CANT READ OBJ FILES ,USE MESH_FROM_OBJ_FILE FOR .OBJ
void* load_mesh (char* filename);

//MDL FORMAT WITH JUST GEOMETRY ,LIMITED ,RATHER USE save_mesh
void save_mdl	 (char* filename , void* mesh , int swapindices);

void del_mesh	 (void* mesh);

//////////////////////////////////////
//////////////////////////////////////
//************************************
// TILE MANIPULATION FUNCTIONS
//

void peragro_rem_tile_id		 (int _tileid);
void peragro_rem_tile_pos		 (VECTOR* pos);
void peragro_rem_tiles_box	  	 (VECTOR* bmin , VECTOR* bmax);
void peragro_rem_tiles_ent_box (ENTITY *ent , int chunk);

peragro_thr peragro_add_all_tiles ();
peragro_thr peragro_save_all_tiles	 (char* filename);
peragro_thr peragro_load_all_tiles	 (char* filename);
peragro_thr peragro_add_tile			 (VECTOR* pos);
peragro_thr peragro_add_tiles_box	 (VECTOR* bmin , VECTOR* bmax);
peragro_thr peragro_add_tiles_ent_box(ENTITY *ent , int chunk);

int  peragro_tot_tiles  ();
void peragro_tile_box	(int tileid , var *_min , var *_max);

//build from multible entity boxes in a list all at once
void clear_temp_entlist();
peragro_thr peragro_add_tiles_box_entlist();

void rem_temp_entlist (ENTITY* ent , int chunk);
void add_temp_entlist (ENTITY* ent , int chunk);


//build from multiple positions in a list all at once 
void clear_temp_pntlist ();
peragro_thr peragro_add_tiles_pntlist ();

void rem_temp_pntlist (VECTOR* pnt , int id);
void add_temp_pntlist (VECTOR* pnt , int id);

//save/load individual tiles
void peragro_save_tile	(char* filename , int tileid);
void peragro_load_tile	(char* filename);
int peragro_tile_id		(VECTOR* pos);

//////////////////////////////////////
//////////////////////////////////////
//************************************
// INPUT GEOMETRY LIST FUNCTIONS
//
void  clear_meshlist ();
void  add_meshlist   (void* mesh);
void  rem_meshlist   (void* mesh);

void  rem_meshlist_ent  (ENTITY* ent , int chunk);
void* get_meshlist_ent	(ENTITY* ent , int chunk);

//////////////////////////////////////
//////////////////////////////////////
//************************************
// THREAD STATUS (DONE/BUSY)
//
int peragro_thread (void* thandle);


